@@ -15,46 +15,13 @@ from ibis.expr.api import *  # noqa: F401,F403
 try:
     import importlib.metadata as importlib_metadata
 except ImportError:
-    # TODO: remove this when Python 3.7 support is dropped
+    # TODO: remove this when Python 3.9 support is dropped
     import importlib_metadata
 
 __all__ = ['api', 'ir', 'util', 'IbisError', 'options']
 __all__ += api.__all__
 
-ibis.config.register_option(
-    'interactive', False, validator=ibis.config.is_bool
-)
-ibis.config.register_option('verbose', False, validator=ibis.config.is_bool)
-ibis.config.register_option('verbose_log', None)
-ibis.config.register_option(
-    'graphviz_repr',
-    True,
-    """\
-Whether to render expressions as GraphViz PNGs when repr-ing in a Jupyter
-notebook.
-""",
-    validator=ibis.config.is_bool,
-)
-ibis.config.register_option('default_backend', None)
-with ibis.config.config_prefix('context_adjustment'):
-    ibis.config.register_option(
-        'time_col',
-        'time',
-        'Name of the timestamp col for execution with a timecontext'
-        'See ibis.expr.timecontext for details.',
-        validator=ibis.config.is_str,
-    )
-with ibis.config.config_prefix('sql'):
-    ibis.config.register_option(
-        'default_limit',
-        10_000,
-        'Number of rows to be retrieved for an unlimited table expression',
-    )
-
-try:
-    __version__ = importlib_metadata.version(__name__)
-except Exception:
-    __version__ = importlib_metadata.version("ibis-framework")
+__version__ = "2.1.1"
 
 
 def __getattr__(name: str) -> BaseBackend:
@@ -99,8 +66,7 @@ def __getattr__(name: str) -> BaseBackend:
 
     # The first time a backend is loaded, we register its options, and we set
     # it as an attribute of `ibis`, so `__getattr__` is not called again for it
-    with ibis.config.config_prefix(name):
-        backend.register_options()
+    backend.register_options()
 
     setattr(ibis, name, backend)
     return backend
