@@ -1,41 +1,42 @@
+from __future__ import annotations
+
 import abc
-from typing import Optional
+from typing import Any, Mapping
 
 import ibis.expr.operations as ops
 import ibis.expr.schema as sch
 import ibis.expr.types as ir
 import ibis.util as util
 from ibis.backends.base import BaseBackend
+from ibis.backends.base.sql.compiler import Compiler
 from ibis.expr.typing import TimeContext
 
-from .compiler import Compiler
-
-__all__ = 'BaseSQLBackend'
+__all__ = [
+    'BaseSQLBackend',
+]
 
 
 class BaseSQLBackend(BaseBackend):
-    """
-    Base backend class for backends that compile to SQL.
-    """
+    """Base backend class for backends that compile to SQL."""
 
     compiler = Compiler
     table_class = ops.DatabaseTable
     table_expr_class = ir.TableExpr
 
-    def table(self, name, database=None):
-        """Create a table expression.
-
-        Create a table expression that references a particular table in the
-        database.
+    def table(self, name: str, database: str | None = None) -> ir.TableExpr:
+        """Construct a table expression.
 
         Parameters
         ----------
-        name : string
-        database : string, optional
+        name
+            Table name
+        database
+            Database name
 
         Returns
         -------
-        table : TableExpr
+        TableExpr
+            Table expression
         """
         qualified_name = self._fully_qualified_name(name, database)
         schema = self.get_schema(qualified_name)
@@ -46,16 +47,18 @@ class BaseSQLBackend(BaseBackend):
         # XXX
         return name
 
-    def sql(self, query):
+    def sql(self, query: str) -> ir.TableExpr:
         """Convert a SQL query to an Ibis table expression.
 
         Parameters
         ----------
-        query : string
+        query
+            SQL string
 
         Returns
         -------
-        table : TableExpr
+        TableExpr
+            Table expression
         """
         # Get the schema by adding a LIMIT 0 on to the end of the query. If
         # there is already a limit in the query, we find and remove it
@@ -63,20 +66,26 @@ class BaseSQLBackend(BaseBackend):
         schema = self._get_schema_using_query(limited_query)
         return ops.SQLQueryResult(query, schema, self).to_expr()
 
-    def raw_sql(self, query: str, results=False):
-        """Execute a given query string.
+    def _get_schema_using_query(self, query):
+        raise NotImplementedError(
+            f"Backend {self.name} does not support .sql()"
+        )
+
+    def raw_sql(self, query: str, results: bool = False) -> Any:
+        """Execute a query string.
 
         Could have unexpected results if the query modifies the behavior of
         the session in a way unknown to Ibis; be careful.
 
         Parameters
         ----------
-        query : string
-          DML or DDL statement
+        query
+            DML or DDL statement
 
         Returns
         -------
-        Backend cursor
+        Any
+            Backend cursor
         """
         # TODO results is unused, it can be removed
         # (requires updating Impala tests)
@@ -88,28 +97,38 @@ class BaseSQLBackend(BaseBackend):
             return cursor
         cursor.release()
 
-    def execute(self, expr, params=None, limit='default', **kwargs):
-        """Compile and execute the given Ibis expression.
+    def execute(
+        self,
+        expr: ir.Expr,
+        params: Mapping[ir.ScalarExpr, Any] | None = None,
+        limit: str = 'default',
+        **kwargs: Any,
+    ):
+        """Compile and execute an Ibis expression.
 
         Compile and execute Ibis expression using this backend client
         interface, returning results in-memory in the appropriate object type
 
         Parameters
         ----------
-        expr : Expr
-        limit : int, default None
-          For expressions yielding result yets; retrieve at most this number of
-          values/rows. Overrides any limit already set on the expression.
-        params : not yet implemented
-        kwargs : Backends can receive extra params. For example, clickhouse
-            uses this to receive external_tables as dataframes.
+        expr
+            Ibis expression
+        limit
+            For expressions yielding result sets; retrieve at most this number
+            of values/rows. Overrides any limit already set on the expression.
+        params
+            Named unbound parameters
+        kwargs
+            Backend specific arguments. For example, the clickhouse backend
+            uses this to receive `external_tables` as a dictionary of pandas
+            DataFrames.
 
         Returns
         -------
-        output : input type dependent
-          Table expressions: pandas.DataFrame
-          Array expressions: pandas.Series
-          Scalar expressions: Python scalar value
+        DataFrame | Series | Scalar
+            * `TableExpr`: pandas.DataFrame
+            * `ColumnExpr`: pandas.Series
+            * `ScalarExpr`: Python scalar value
         """
         # TODO Reconsider having `kwargs` here. It's needed to support
         # `external_tables` in clickhouse, but better to deprecate that
@@ -134,17 +153,25 @@ class BaseSQLBackend(BaseBackend):
     def fetch_from_cursor(self, cursor, schema):
         """Fetch data from cursor."""
 
-    def ast_schema(self, query_ast):
+    def ast_schema(self, query_ast, **kwargs) -> sch.Schema:
         """Return the schema of the expression.
 
+        Parameters
+        ----------
+        query_ast
+            The AST of the query
+        kwargs
+            Backend specific parameters
+
         Returns
         -------
         Schema
+            An ibis schema
 
         Raises
         ------
         ValueError
-            if self.expr doesn't have a schema.
+            if `self.expr` doesn't have a schema.
         """
         dml = getattr(query_ast, 'dml', query_ast)
         expr = getattr(dml, 'parent_expr', getattr(dml, 'table_set', None))
@@ -159,43 +186,58 @@ class BaseSQLBackend(BaseBackend):
                 'schema'.format(type(self.expr))
             )
 
-    def _log(self, sql):
+    def _log(self, sql: str) -> None:
         """Log the SQL, usually to the standard output.
 
         This method can be implemented by subclasses. The logging happens
         when `ibis.options.verbose` is `True`.
         """
-        pass
 
     def compile(
         self,
-        expr,
-        limit=None,
-        params=None,
-        timecontext: Optional[TimeContext] = None,
-    ):
-        """Translate expression.
+        expr: ir.Expr,
+        limit: str | None = None,
+        params: Mapping[ir.Expr, Any] | None = None,
+        timecontext: TimeContext | None = None,
+    ) -> Any:
+        """Compille an Ibis expression.
 
-        Translate expression to one or more queries according to
-        backend target.
+        Parameters
+        ----------
+        expr
+            Ibis expression
+        limit
+            For expressions yielding result sets; retrieve at most this number
+            of values/rows. Overrides any limit already set on the expression.
+        params
+            Named unbound parameters
+        timecontext
+            Additional information about data source time boundaries
 
         Returns
         -------
-        output : single query or list of queries
+        Any
+            The output of compilation. The type of this value depends on the
+            backend.
         """
         return self.compiler.to_ast_ensure_limit(
             expr, limit, params=params
         ).compile()
 
-    def explain(self, expr, params=None):
-        """Explain expression.
+    def explain(
+        self,
+        expr: ir.Expr | str,
+        params: Mapping[ir.Expr, Any] | None = None,
+    ) -> str:
+        """Explain an expression.
 
-        Query for and return the query plan associated with the indicated
-        expression or SQL query.
+        Return the query plan associated with the indicated expression or SQL
+        query.
 
         Returns
         -------
-        plan : string
+        str
+            Query plan
         """
         if isinstance(expr, ir.Expr):
             context = self.compiler.make_context(params=params)
@@ -214,3 +256,15 @@ class BaseSQLBackend(BaseBackend):
         cur.release()
 
         return '\n'.join(['Query:', util.indent(query, 2), '', *result])
+
+    @classmethod
+    def has_operation(cls, operation: type[ops.ValueOp]) -> bool:
+        translator = cls.compiler.translator_class
+        op_classes = translator._registry.keys() | translator._rewrites.keys()
+        return operation in op_classes
+
+    def _create_temp_view(self, view, definition):
+        raise NotImplementedError(
+            f"The {self.name} backend does not implement temporary view "
+            "creation"
+        )
