@@ -1,3 +1,4 @@
+import functools
 import operator
 from typing import Any, Dict
 
@@ -11,9 +12,8 @@ import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 import ibis.expr.types as ir
 import ibis.expr.window as W
-
-from .database import AlchemyTable
-from .geospatial import geospatial_supported
+from ibis.backends.base.sql.alchemy.database import AlchemyTable
+from ibis.backends.base.sql.alchemy.geospatial import geospatial_supported
 
 
 def variance_reduction(func_name):
@@ -56,26 +56,30 @@ def fixed_arity(sa_func, arity):
         if arity != len(expr.op().args):
             raise com.IbisError('incorrect number of args')
 
-        return _varargs_call(sa_func, t, expr)
+        return _varargs_call(sa_func, t, expr.op().args)
 
     return formatter
 
 
+def _varargs_call(sa_func, t, args):
+    trans_args = []
+    for raw_arg in args:
+        arg = t.translate(raw_arg)
+        try:
+            arg = arg.scalar_subquery()
+        except AttributeError:
+            pass
+        trans_args.append(arg)
+    return sa_func(*trans_args)
+
+
 def varargs(sa_func):
     def formatter(t, expr):
-        op = expr.op()
-        trans_args = [t.translate(arg) for arg in op.arg]
-        return sa_func(*trans_args)
+        return _varargs_call(sa_func, t, expr.op().arg)
 
     return formatter
 
 
-def _varargs_call(sa_func, t, expr):
-    op = expr.op()
-    trans_args = [t.translate(arg) for arg in op.args]
-    return sa_func(*trans_args)
-
-
 def get_sqla_table(ctx, table):
     if ctx.has_ref(table, parent_contexts=True):
         ctx_level = ctx
@@ -93,18 +97,44 @@ def get_sqla_table(ctx, table):
     return sa_table
 
 
+def get_col_or_deferred_col(sa_table, colname):
+    """
+    Get a `ColumnExpr`, or create a "deferred" column.
+
+    This is to handle the case when selecting a column from a join, which
+    happens when a join expression is cached during join traversal
+
+    We'd like to avoid generating a subquery just for selection but in
+    sqlalchemy the Join object is not selectable. However, at this point
+    know that the column can be referred to unambiguously
+
+    Later the expression is assembled into
+    `sa.select([sa.column(colname)]).select_from(table_set)` (roughly)
+    where `table_set` is `sa_table` above.
+    """
+    try:
+        return sa_table.exported_columns[colname]
+    except KeyError:
+        return sa.column(colname)
+
+
 def _table_column(t, expr):
     op = expr.op()
     ctx = t.context
     table = op.table
 
     sa_table = get_sqla_table(ctx, table)
-    out_expr = getattr(sa_table.c, op.name)
+
+    out_expr = get_col_or_deferred_col(sa_table, op.name)
 
     # If the column does not originate from the table set in the current SELECT
     # context, we should format as a subquery
     if t.permit_subquery and ctx.is_foreign_expr(table):
-        return sa.select([out_expr])
+        try:
+            subq = sa_table.subquery()
+        except AttributeError:
+            subq = sa_table
+        return sa.select(subq.c[out_expr.name])
 
     return out_expr
 
@@ -116,13 +146,13 @@ def _table_array_view(t, expr):
 
 
 def _exists_subquery(t, expr):
-    from .query_builder import AlchemyCompiler
+    from ibis.backends.base.sql.alchemy.query_builder import AlchemyCompiler
 
     op = expr.op()
     ctx = t.context
 
     filtered = op.foreign_table.filter(op.predicates).projection(
-        [ir.literal(1).name(ir.unnamed)]
+        [ir.literal(1).name(ir.core.unnamed)]
     )
 
     sub_ctx = ctx.subcontext()
@@ -161,20 +191,30 @@ def _not_contains(t, expr):
 def reduction(sa_func):
     def formatter(t, expr):
         op = expr.op()
-        *args, where = op.args
-
-        return _reduction_format(t, sa_func, where, *args)
+        if op.where is not None:
+            arg = t.translate(op.where.ifelse(op.arg, ibis.NA))
+        else:
+            arg = t.translate(op.arg)
+        return sa_func(arg)
 
     return formatter
 
 
-def _reduction_format(t, sa_func, where, arg, *args):
-    if where is not None:
-        arg = t.translate(where.ifelse(arg, ibis.NA))
+def _group_concat(t, expr):
+    op = expr.op()
+    sep = t.translate(op.sep)
+    if op.where is not None:
+        arg = t.translate(op.where.ifelse(op.arg, ibis.NA))
     else:
-        arg = t.translate(arg)
+        arg = t.translate(op.arg)
+    return sa.func.group_concat(arg, sep)
 
-    return sa_func(arg, *map(t.translate, args))
+
+def _alias(t, expr):
+    # just compile the underlying argument because the naming is handled
+    # by the translator for the top level expression
+    op = expr.op()
+    return t.translate(op.arg)
 
 
 def _literal(t, expr):
@@ -263,10 +303,10 @@ def unary(sa_func):
     return fixed_arity(sa_func, 1)
 
 
-def _string_like(t, expr):
-    arg, pattern, escape = expr.op().args
-    result = t.translate(arg).like(t.translate(pattern), escape=escape)
-    return result
+def _string_like(method_name, t, expr):
+    op = expr.op()
+    method = getattr(t.translate(op.arg), method_name)
+    return method(t.translate(op.pattern), escape=op.escape)
 
 
 def _startswith(t, expr):
@@ -297,7 +337,9 @@ def _cumulative_to_window(translator, expr, window):
 
     klass = _cumulative_to_reduction[type(op)]
     new_op = klass(*op.args)
-    new_expr = expr._factory(new_op, name=expr._name)
+    new_expr = new_op.to_expr()
+    if expr.has_name():
+        new_expr = new_expr.name(expr.get_name())
 
     if type(new_op) in translator._rewrites:
         new_expr = translator._rewrites[type(new_op)](new_expr)
@@ -406,7 +448,13 @@ def _sort_key(t, expr):
     return sort_direction(t.translate(by))
 
 
+def _string_join(t, expr):
+    sep, elements = expr.op().args
+    return sa.func.concat_ws(t.translate(sep), *map(t.translate, elements))
+
+
 sqlalchemy_operation_registry: Dict[Any, Any] = {
+    ops.Alias: _alias,
     ops.And: fixed_arity(sql.and_, 2),
     ops.Or: fixed_arity(sql.or_, 2),
     ops.Not: unary(sa.not_),
@@ -422,7 +470,7 @@ sqlalchemy_operation_registry: Dict[Any, Any] = {
     ops.Min: reduction(sa.func.min),
     ops.Max: reduction(sa.func.max),
     ops.CountDistinct: _count_distinct,
-    ops.GroupConcat: reduction(sa.func.group_concat),
+    ops.GroupConcat: _group_concat,
     ops.Between: fixed_arity(sa.between, 3),
     ops.IsNull: _is_null,
     ops.NotNull: _not_null,
@@ -431,7 +479,7 @@ sqlalchemy_operation_registry: Dict[Any, Any] = {
     ops.TypeOf: unary(sa.func.typeof),
     ops.Literal: _literal,
     ops.ValueList: _value_list,
-    ops.NullLiteral: lambda *args: sa.null(),
+    ops.NullLiteral: lambda *_: sa.null(),
     ops.SimpleCase: _simple_case,
     ops.SearchedCase: _searched_case,
     ops.TableColumn: _table_column,
@@ -454,10 +502,13 @@ sqlalchemy_operation_registry: Dict[Any, Any] = {
     ops.Uppercase: unary(sa.func.upper),
     ops.StringAscii: unary(sa.func.ascii),
     ops.StringLength: unary(sa.func.length),
+    ops.StringJoin: _string_join,
     ops.StringReplace: fixed_arity(sa.func.replace, 3),
-    ops.StringSQLLike: _string_like,
+    ops.StringSQLLike: functools.partial(_string_like, "like"),
+    ops.StringSQLILike: functools.partial(_string_like, "ilike"),
     ops.StartsWith: _startswith,
     ops.EndsWith: _endswith,
+    ops.StringConcat: varargs(sa.func.concat),
     # math
     ops.Ln: unary(sa.func.ln),
     ops.Exp: unary(sa.func.exp),
@@ -469,6 +520,12 @@ sqlalchemy_operation_registry: Dict[Any, Any] = {
     ops.FloorDivide: _floor_divide,
     # other
     ops.SortKey: _sort_key,
+    ops.Date: unary(lambda arg: sa.cast(arg, sa.DATE)),
+    ops.DateFromYMD: fixed_arity(sa.func.date, 3),
+    ops.TimeFromHMS: fixed_arity(sa.func.time, 3),
+    ops.TimestampFromYMDHMS: lambda t, expr: sa.func.make_timestamp(
+        *map(t.translate, expr.op().args[:6])  # ignore timezone
+    ),
 }
 
 
@@ -575,8 +632,8 @@ if geospatial_supported:
         #   ST_GeomFromEWKT
         #   ST_GeomFromText
     }
-
-    sqlalchemy_operation_registry.update(_geospatial_functions)
+else:
+    _geospatial_functions = {}
 
 
 for _k, _v in _binary_ops.items():
