@@ -1,12 +1,15 @@
-from .base import DDL, DML
-from .query_builder import (
+from ibis.backends.base.sql.compiler.base import DDL, DML
+from ibis.backends.base.sql.compiler.query_builder import (
     Compiler,
     Select,
     SelectBuilder,
     TableSetFormatter,
     Union,
 )
-from .translator import ExprTranslator, QueryContext
+from ibis.backends.base.sql.compiler.translator import (
+    ExprTranslator,
+    QueryContext,
+)
 
 __all__ = (
     'Compiler',
