@@ -6,8 +6,9 @@ import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 import ibis.expr.types as ir
 import ibis.util as util
-
-from .extract_subqueries import ExtractSubqueries
+from ibis.backends.base.sql.compiler.extract_subqueries import (
+    ExtractSubqueries,
+)
 
 
 class _AnyToExistsTransform:
@@ -34,7 +35,7 @@ class _AnyToExistsTransform:
         else:
             op = ops.NotExistsSubquery(self.foreign_table, self.predicates)
 
-        expr_type = dt.boolean.column_type()
+        expr_type = dt.boolean.column
         return expr_type(op)
 
     def _visit(self, expr):
@@ -284,7 +285,7 @@ class SelectBuilder:
                 )
                 return table_expr, _get_scalar(name)
             else:
-                base_table = ir.find_base_table(expr)
+                base_table = ir.relations.find_base_table(expr)
                 if base_table is None:
                     # exprs with no table refs
                     # TODO(phillipc): remove ScalarParameter hack
@@ -317,20 +318,11 @@ class SelectBuilder:
                 # Something more complicated.
                 base_table = L.find_source_table(expr)
 
-                if isinstance(op, ops.DistinctColumn):
-                    expr = op.arg
-                    try:
-                        name = op.arg.get_name()
-                    except Exception:
-                        name = 'tmp'
+                if not expr.has_name():
+                    expr = expr.name('tmp')
 
-                    table_expr = base_table.projection(
-                        [expr.name(name)]
-                    ).distinct()
-                    result_handler = _get_column(name)
-                else:
-                    table_expr = base_table.projection([expr.name('tmp')])
-                    result_handler = _get_column('tmp')
+                table_expr = base_table.projection([expr])
+                result_handler = _get_column(expr.get_name())
 
             return table_expr, result_handler
         else:
@@ -479,7 +471,8 @@ class SelectBuilder:
                     new_args.append(arg)
 
             if not unchanged:
-                return expr._factory(type(op)(*new_args))
+                new_op = type(op)(*new_args)
+                return new_op.to_expr()
             else:
                 return expr
         else:
@@ -512,8 +505,11 @@ class SelectBuilder:
             binwidth = op.binwidth
             base = op.base
 
-        bucket = (op.arg - base) / binwidth
-        return bucket.floor().name(expr._name)
+        bucket = ((op.arg - base) / binwidth).floor()
+        if expr.has_name():
+            bucket = bucket.name(expr.get_name())
+
+        return bucket
 
     def _analyze_filter_exprs(self):
         # What's semantically contained in the filter predicates may need to be
@@ -555,13 +551,12 @@ class SelectBuilder:
             left = self._visit_filter(op.left)
             right = self._visit_filter(op.right)
             unchanged = left is op.left and right is op.right
-            if not unchanged:
-                return expr._factory(type(op)(left, right))
-            else:
+            if unchanged:
                 return expr
-        elif isinstance(
-            op, (ops.Any, ops.BooleanValueOp, ops.TableColumn, ops.Literal)
-        ):
+            else:
+                new_op = type(op)(left, right)
+                return new_op.to_expr()
+        elif isinstance(op, (ops.Any, ops.TableColumn, ops.Literal)):
             return expr
         elif isinstance(op, ops.ValueOp):
             visited = [
@@ -573,7 +568,8 @@ class SelectBuilder:
                 if new is not old:
                     unchanged = False
             if not unchanged:
-                return expr._factory(type(op)(*visited))
+                new_op = type(op)(*visited)
+                return new_op.to_expr()
             else:
                 return expr
         else:
@@ -632,11 +628,6 @@ class SelectBuilder:
 
         # hm, is this the best place for this?
         root_op = source_expr.op()
-        if isinstance(root_op, ops.Join) and not isinstance(
-            root_op, ops.MaterializedJoin
-        ):
-            # Unmaterialized join
-            source_expr = source_expr.materialize()
 
         if isinstance(root_op, ops.TableNode):
             self._collect(source_expr, toplevel=True)
