@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import itertools
 import operator
-from typing import Callable, Iterator
+from typing import Callable, Iterable, Iterator
 
 import ibis
 import ibis.common.exceptions as com
@@ -13,10 +13,10 @@ from ibis.backends.base.sql.registry import (
     operation_registry,
     quote_identifier,
 )
+from ibis.expr.types.core import unnamed
 
 
 class QueryContext:
-
     """Records bits of information used during ibis AST to SQL translation.
 
     Notably, table aliases (for subquery construction) and scalar query
@@ -41,16 +41,18 @@ class QueryContext:
     def _to_sql(self, expr, ctx):
         return self.compiler.to_sql(expr, ctx)
 
-    def collapse(self, queries):
-        """Turn a sequence of queries into something executable.
+    def collapse(self, queries: Iterable[str]) -> str:
+        """Turn an iterable of queries into something executable.
 
         Parameters
         ----------
-        queries : List[str]
+        queries
+            Iterable of query strings
 
         Returns
         -------
-        query : str
+        query
+            A single query string
         """
         return '\n\n'.join(queries)
 
@@ -74,7 +76,7 @@ class QueryContext:
             pass
 
         op = expr.op()
-        if isinstance(op, ops.SQLQueryResult):
+        if isinstance(op, (ops.SQLQueryResult, ops.SQLStringView)):
             result = op.query
         else:
             result = self._compile_subquery(expr)
@@ -130,10 +132,7 @@ class QueryContext:
         self.table_refs[key] = alias
 
     def get_ref(self, expr):
-        """
-        Get the alias being used throughout a query to refer to a particular
-        table or inline view
-        """
+        """Return the alias used to refer to an expression."""
         key = self._get_table_key(expr)
         top = self.top_context
 
@@ -185,10 +184,7 @@ class QueryContext:
 
 
 class ExprTranslator:
-
-    """Class that performs translation of ibis expressions into executable
-    SQL.
-    """
+    """Translates ibis expressions into a compilation target."""
 
     _registry = operation_registry
     _rewrites: dict[ops.Node, Callable] = {}
@@ -205,10 +201,24 @@ class ExprTranslator:
         # For now, governing whether the result will have a name
         self.named = named
 
+    def _needs_name(self, expr):
+        if not self.named:
+            return False
+
+        op = expr.op()
+        if isinstance(op, ops.TableColumn):
+            # This column has been given an explicitly different name
+            return expr.get_name() != op.name
+
+        return expr.get_name() is not unnamed
+
+    def name(self, translated, name, force=True):
+        return '{} AS {}'.format(
+            translated, quote_identifier(name, force=force)
+        )
+
     def get_result(self):
-        """
-        Build compiled SQL expression from the bottom up and return as a string
-        """
+        """Compile SQL expression into a string."""
         translated = self.translate(self.expr)
         if self._needs_name(self.expr):
             # TODO: this could fail in various ways
@@ -218,36 +228,16 @@ class ExprTranslator:
 
     @classmethod
     def add_operation(cls, operation, translate_function):
-        """
-        Adds an operation to the operation registry. In general, operations
-        should be defined directly in the registry, in `registry.py`. But
-        there are couple of exceptions why this is needed. Operations defined
-        by Ibis users (not Ibis or backend developers). and UDF, which are
-        added dynamically.
+        """Add an operation to the operation registry.
+
+        In general, operations should be defined directly in the registry, in
+        `registry.py`. There are couple of exceptions why this is needed.
+
+        Operations defined by Ibis users (not Ibis or backend developers), and
+        UDFs which are added dynamically.
         """
         cls._registry[operation] = translate_function
 
-    def _needs_name(self, expr):
-        if not self.named:
-            return False
-
-        op = expr.op()
-        if isinstance(op, ops.TableColumn):
-            # This column has been given an explicitly different name
-            if expr.get_name() != op.name:
-                return True
-            return False
-
-        if expr.get_name() is ir.unnamed:
-            return False
-
-        return True
-
-    def name(self, translated, name, force=True):
-        return '{} AS {}'.format(
-            translated, quote_identifier(name, force=force)
-        )
-
     def translate(self, expr):
         # The operation node type the typed expression wraps
         op = expr.op()
@@ -272,7 +262,13 @@ class ExprTranslator:
 
     def _trans_param(self, expr):
         raw_value = self.context.params[expr.op()]
-        literal = ibis.literal(raw_value, type=expr.type())
+        dtype = expr.type()
+        if isinstance(dtype, dt.Struct):
+            literal = ibis.struct(raw_value, type=dtype)
+        elif isinstance(dtype, dt.Map):
+            literal = ibis.map(raw_value, type=dtype)
+        else:
+            literal = ibis.literal(raw_value, type=dtype)
         return self.translate(literal)
 
     @classmethod
@@ -331,7 +327,11 @@ def _bucket(expr):
         stmt = stmt.when(cmp(op.buckets[-1], op.arg), bucket_id)
         bucket_id += 1
 
-    return stmt.end().name(expr._name)
+    result = stmt.end()
+    if expr.has_name():
+        result = result.name(expr.get_name())
+
+    return result
 
 
 @rewrites(ops.CategoryLabel)
@@ -345,7 +345,11 @@ def _category_label(expr):
     if op.nulls is not None:
         stmt = stmt.else_(op.nulls)
 
-    return stmt.end().name(expr._name)
+    result = stmt.end()
+    if expr.has_name():
+        result = result.name(expr.get_name())
+
+    return result
 
 
 @rewrites(ops.Any)
@@ -357,7 +361,7 @@ def _any_expand(expr):
 @rewrites(ops.NotAny)
 def _notany_expand(expr):
     arg = expr.op().args[0]
-    return arg.max() == 0
+    return arg.max() == ibis.literal(0, type=arg.type())
 
 
 @rewrites(ops.All)
@@ -369,7 +373,7 @@ def _all_expand(expr):
 @rewrites(ops.NotAll)
 def _notall_expand(expr):
     arg = expr.op().args[0]
-    return arg.min() == 0
+    return arg.min() == ibis.literal(0, type=arg.type())
 
 
 @rewrites(ops.Cast)
