@@ -6,12 +6,20 @@ import ibis.common.exceptions as com
 import ibis.expr.operations as ops
 import ibis.expr.types as ir
 import ibis.util as util
+from ibis.backends.clickhouse.identifiers import quote_identifier
 
-from .identifiers import quote_identifier
+# TODO(kszucs): should inherit operation registry from the base compiler
+
+
+def _alias(translator, expr):
+    # just compile the underlying argument because the naming is handled
+    # by the translator for the top level expression
+    op = expr.op()
+    return translator.translate(op.arg)
 
 
 def _cast(translator, expr):
-    from .client import ClickhouseDataType
+    from ibis.backends.clickhouse.client import ClickhouseDataType
 
     op = expr.op()
     arg, target = op.args
@@ -74,6 +82,58 @@ def _fixed_arity(func_name, arity):
     return formatter
 
 
+def _array_index_op(translator, expr):
+    op = expr.op()
+
+    arr = op.args[0]
+    idx = op.args[1]
+
+    arr_ = translator.translate(arr)
+    idx_ = _parenthesize(translator, idx)
+
+    correct_idx = f'if({idx_} >= 0, {idx_} + 1, {idx_})'
+
+    return f'arrayElement({arr_}, {correct_idx})'
+
+
+def _array_repeat_op(translator, expr):
+    op = expr.op()
+    arr, times = op.args
+
+    arr_ = _parenthesize(translator, arr)
+    times_ = _parenthesize(translator, times)
+
+    select = 'arrayFlatten(groupArray(arr))'
+    from_ = f'(select {arr_} as arr from system.numbers limit {times_})'
+    return f'(select {select} from {from_})'
+
+
+def _array_slice_op(translator, expr):
+    op = expr.op()
+    arg, start, stop = op.args
+
+    start_ = _parenthesize(translator, start)
+    arg_ = translator.translate(arg)
+
+    start_correct_ = f'if({start_} < 0, {start_}, {start_} + 1)'
+
+    if stop is not None:
+        stop_ = _parenthesize(translator, stop)
+
+        cast_arg_ = f'if({arg_} = [], CAST({arg_} AS Array(UInt8)), {arg_})'
+        neg_start_ = f'(arrayCount({cast_arg_}) + {start_})'
+        diff_fmt = f'greatest(-0, {stop_} - {{}})'.format
+
+        length_ = (
+            f'if({stop_} < 0, {stop_}, '
+            + f'if({start_} < 0, {diff_fmt(neg_start_)}, {diff_fmt(start_)}))'
+        )
+
+        return f'arraySlice({arg_}, {start_correct_}, {length_})'
+
+    return f'arraySlice({arg_}, {start_correct_})'
+
+
 def _agg(func):
     def formatter(translator, expr):
         return _aggregate(translator, func, *expr.op().args)
@@ -132,14 +192,14 @@ def _varargs(func_name):
 
 
 def _arbitrary(translator, expr):
-    arg, how, where = expr.op().args
+    op = expr.op()
     functions = {
         None: 'any',
         'first': 'any',
         'last': 'anyLast',
         'heavy': 'anyHeavy',
     }
-    return _aggregate(translator, functions[how], arg, where=where)
+    return _aggregate(translator, functions[op.how], op.arg, where=op.where)
 
 
 def _substring(translator, expr):
@@ -173,14 +233,14 @@ def _string_find(translator, expr):
 
 def _regex_extract(translator, expr):
     op = expr.op()
-    arg, pattern, index = op.args
-    arg_, pattern_ = translator.translate(arg), translator.translate(pattern)
+    arg_ = translator.translate(op.arg)
+    pattern_ = translator.translate(op.pattern)
+    index = op.index
 
+    base = f"extractAll(CAST({arg_} AS String), {pattern_})"
     if index is not None:
-        index_ = translator.translate(index)
-        return f'extractAll({arg_}, {pattern_})[{index_} + 1]'
-
-    return f'extractAll({arg_}, {pattern_})'
+        return f"{base}[{translator.translate(index)} + 1]"
+    return base
 
 
 def _parse_url(translator, expr):
@@ -326,13 +386,21 @@ def _literal(translator, expr):
             value = value.strftime('%Y-%m-%d')
         return f"toDate('{value!s}')"
     elif isinstance(expr, ir.ArrayValue):
-        return str(list(value))
+        return str(list(_tuple_to_list(value)))
     elif isinstance(expr, ir.SetScalar):
         return '({})'.format(', '.join(map(repr, value)))
     else:
         raise NotImplementedError(type(expr))
 
 
+def _tuple_to_list(t: tuple):
+    for element in t:
+        if util.is_iterable(element):
+            yield list(_tuple_to_list(element))
+        else:
+            yield element
+
+
 class _CaseFormatter:
     def __init__(self, translator, base, cases, results, default):
         self.translator = translator
@@ -443,7 +511,7 @@ def _exists_subquery(translator, expr):
     op = expr.op()
     ctx = translator.context
 
-    dummy = ir.literal(1).name(ir.unnamed)
+    dummy = ir.literal(1).name(ir.core.unnamed)
 
     filtered = op.foreign_table.filter(op.predicates)
     expr = filtered.projection([dummy])
@@ -482,10 +550,10 @@ def _table_column(translator, expr):
 
 
 def _string_split(translator, expr):
-    value, sep = expr.op().args
-    return 'splitByString({}, {})'.format(
-        translator.translate(sep), translator.translate(value)
-    )
+    op = expr.op()
+    delim = translator.translate(op.delimiter)
+    val = translator.translate(op.arg)
+    return f"splitByString({delim}, CAST({val} AS String))"
 
 
 def _string_join(translator, expr):
@@ -499,12 +567,10 @@ def _string_join(translator, expr):
     )
 
 
-def _string_repeat(translator, expr):
-    value, times = expr.op().args
-    result = 'arrayStringConcat(arrayMap(x -> {}, range({})))'.format(
-        translator.translate(value), translator.translate(times)
-    )
-    return result
+def _string_concat(translator, expr):
+    args = expr.op().arg
+    args_formatted = ", ".join(map(translator.translate, args))
+    return f"arrayStringConcat([{args_formatted}])"
 
 
 def _string_like(translator, expr):
@@ -514,6 +580,14 @@ def _string_like(translator, expr):
     )
 
 
+def _string_ilike(translator, expr):
+    op = expr.op()
+    return 'lower({}) LIKE lower({})'.format(
+        translator.translate(op.arg),
+        translator.translate(op.pattern),
+    )
+
+
 def _group_concat(translator, expr):
     arg, sep, where = expr.op().args
     if where is not None:
@@ -523,6 +597,13 @@ def _group_concat(translator, expr):
     )
 
 
+def _string_right(translator, expr):
+    op = expr.op()
+    arg = translator.translate(op.arg)
+    nchars = translator.translate(op.nchars)
+    return f"substring({arg}, -({nchars}))"
+
+
 # TODO: clickhouse uses different string functions
 #       for ascii and utf-8 encodings,
 
@@ -551,6 +632,7 @@ _unary_ops = {ops.Negate: _negate, ops.Not: _not}
 
 
 operation_registry = {
+    ops.Alias: _alias,
     # Unary operations
     ops.TypeOf: _unary('toTypeName'),
     ops.IsNan: _unary('isNaN'),
@@ -594,16 +676,25 @@ operation_registry = {
     ops.StringJoin: _string_join,
     ops.StringSplit: _string_split,
     ops.StringSQLLike: _string_like,
-    ops.Repeat: _string_repeat,
+    ops.StringSQLILike: _string_ilike,
+    ops.StartsWith: _fixed_arity("startsWith", 2),
+    ops.EndsWith: _fixed_arity("endsWith", 2),
+    ops.LPad: _fixed_arity("leftPad", 3),
+    ops.RPad: _fixed_arity("rightPad", 3),
+    ops.LStrip: _unary('trimLeft'),
+    ops.RStrip: _unary('trimRight'),
+    ops.Strip: _unary('trimBoth'),
+    ops.Repeat: _fixed_arity("repeat", 2),
+    ops.StringConcat: _string_concat,
     ops.RegexSearch: _fixed_arity('match', 2),
-    # TODO: extractAll(haystack, pattern)[index + 1]
     ops.RegexExtract: _regex_extract,
     ops.RegexReplace: _fixed_arity('replaceRegexpAll', 3),
     ops.ParseURL: _parse_url,
+    ops.StrRight: _string_right,
     # Temporal operations
     ops.Date: _unary('toDate'),
     ops.DateTruncate: _truncate,
-    ops.TimestampNow: lambda *args: 'now()',
+    ops.TimestampNow: lambda *_: 'now()',
     ops.TimestampTruncate: _truncate,
     ops.TimeTruncate: _truncate,
     ops.IntervalFromInteger: _interval_from_integer,
@@ -618,7 +709,7 @@ operation_registry = {
     ops.ExtractMinute: _unary('toMinute'),
     ops.ExtractSecond: _unary('toSecond'),
     # Other operations
-    ops.E: lambda *args: 'e()',
+    ops.E: lambda *_: 'e()',
     ops.Literal: _literal,
     ops.ValueList: _value_list,
     ops.Cast: _cast,
@@ -644,6 +735,10 @@ operation_registry = {
     ops.ExistsSubquery: _exists_subquery,
     ops.NotExistsSubquery: _exists_subquery,
     ops.ArrayLength: _unary('length'),
+    ops.ArrayIndex: _array_index_op,
+    ops.ArrayConcat: _fixed_arity('arrayConcat', 2),
+    ops.ArrayRepeat: _array_repeat_op,
+    ops.ArraySlice: _array_slice_op,
 }
 
 
