@@ -1,5 +1,9 @@
+import pytest
+
 import ibis
 
+pytest.importorskip("clickhouse_driver")
+
 
 def test_column_ref_quoting(translate):
     schema = [('has a space', 'double')]
