@@ -4,8 +4,7 @@ import dask.dataframe as dd
 
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
-
-from ..dispatch import execute_node
+from ibis.backends.dask.dispatch import execute_node
 
 
 @execute_node.register(ops.Cast, dd.Series, dt.Decimal)
