@@ -1,18 +1,20 @@
 import operator
 from operator import methodcaller
 
-import dask.array as da
-import dask.dataframe as dd
 import numpy as np
 import numpy.testing as npt
 import pandas as pd
 import pytest
-from dask.dataframe.utils import tm
 
 import ibis
 import ibis.expr.datatypes as dt
 
-from ...execution import execute
+da = pytest.importorskip("dask.array")
+dd = pytest.importorskip("dask.dataframe")
+
+from dask.dataframe.utils import tm  # noqa: E402
+
+from ibis.backends.dask.execution import execute  # noqa: E402
 
 
 def test_table_column(t, df):
@@ -533,13 +535,6 @@ def test_complex_sort_by(t, df):
     )
 
 
-def test_distinct(t, df):
-    expr = t.dup_strings.distinct()
-    result = expr.compile()
-    expected = df.dup_strings.unique()
-    tm.assert_series_equal(result.compute(), expected.compute())
-
-
 def test_count_distinct(t, df):
     expr = t.dup_strings.nunique()
     result = expr.compile()
@@ -625,7 +620,7 @@ def test_mutate_after_group_by(t, df):
 def test_groupby_with_unnamed_arithmetic(t, df):
     expr = t.groupby(t.dup_strings).aggregate(
         naive_variance=(
-            (t.plain_float64 ** 2).sum() - t.plain_float64.mean() ** 2
+            (t.plain_float64**2).sum() - t.plain_float64.mean() ** 2
         )
         / t.plain_float64.count()
     )
@@ -635,7 +630,7 @@ def test_groupby_with_unnamed_arithmetic(t, df):
         .groupby('dup_strings')
         .agg(
             {
-                'plain_float64': lambda x: ((x ** 2).sum() - x.mean() ** 2)
+                'plain_float64': lambda x: ((x**2).sum() - x.mean() ** 2)
                 / x.count()
             }
         )
@@ -790,6 +785,10 @@ def test_round(t, df):
     npt.assert_almost_equal(result, expected, decimal=precision)
 
 
+@pytest.mark.xfail(
+    raises=NotImplementedError,
+    reason="MultiQuantile is not implemented for the dask backend",
+)
 def test_quantile_groupby(batting, batting_df):
     def q_fun(x, quantile, interpolation):
         res = x.quantile(quantile, interpolation=interpolation).tolist()
@@ -811,8 +810,8 @@ def test_quantile_groupby(batting, batting_df):
 
 
 def test_summary_numeric(batting, batting_df):
-    expr = batting.G.summary()
-    result = expr.compile()
+    expr = batting.aggregate(batting.G.summary())
+    result = expr.execute()
     assert len(result) == 1
 
     G = batting_df.G
@@ -830,26 +829,24 @@ def test_summary_numeric(batting, batting_df):
 
 def test_summary_numeric_group_by(batting, batting_df):
     expr = batting.groupby('teamID').G.summary()
-    result = expr.compile()
+    result = expr.execute()
     expected = (
         batting_df.groupby('teamID')
         .G.apply(
-            lambda s: dd.from_pandas(
-                pd.DataFrame(
-                    {
-                        'count': s.count(),
-                        'nulls': s.isnull().sum(),
-                        'min': s.min(),
-                        'max': s.max(),
-                        'sum': s.sum(),
-                        'mean': s.mean(),
-                        'approx_nunique': s.nunique(),
-                    },
-                    index=[0],
-                ),
-                npartitions=1,
+            lambda s: pd.DataFrame(
+                {
+                    'count': s.count(),
+                    'nulls': s.isnull().sum(),
+                    'min': s.min(),
+                    'max': s.max(),
+                    'sum': s.sum(),
+                    'mean': s.mean(),
+                    'approx_nunique': s.nunique(),
+                },
+                index=[0],
             )
         )
+        .compute()
         .reset_index(level=1, drop=True)
         .reset_index()
     )
@@ -859,8 +856,8 @@ def test_summary_numeric_group_by(batting, batting_df):
 
 
 def test_summary_non_numeric(batting, batting_df):
-    expr = batting.teamID.summary()
-    result = expr.compile()
+    expr = batting.aggregate(batting.teamID.summary())
+    result = expr.execute()
     assert len(result) == 1
     assert len(result.columns) == 3
     expected = {
@@ -873,22 +870,20 @@ def test_summary_non_numeric(batting, batting_df):
 
 def test_summary_non_numeric_group_by(batting, batting_df):
     expr = batting.groupby('teamID').playerID.summary()
-    result = expr.compile()
+    result = expr.execute()
     expected = (
         batting_df.groupby('teamID')
         .playerID.apply(
-            lambda s: dd.from_pandas(
-                pd.DataFrame(
-                    {
-                        'count': s.count(),
-                        'nulls': s.isnull().sum(),
-                        'uniques': s.nunique(),
-                    },
-                    index=[0],
-                ),
-                npartitions=1,
+            lambda s: pd.DataFrame(
+                {
+                    'count': s.count(),
+                    'nulls': s.isnull().sum(),
+                    'uniques': s.nunique(),
+                },
+                index=[0],
             )
         )
+        .compute()
         .reset_index(level=1, drop=True)
         .reset_index()
     )
@@ -994,7 +989,7 @@ def test_difference(client, df1, intersect_df2):
     merged = df1.merge(
         intersect_df2, on=list(df1.columns), how="outer", indicator=True
     )
-    expected = merged[merged["_merge"] != "both"].drop("_merge", 1)
+    expected = merged[merged["_merge"] != "both"].drop("_merge", axis=1)
 
     # force same index
     result = result.compute().reset_index(drop=True)
