@@ -1,6 +1,5 @@
 import collections
 
-import dask.dataframe as dd
 import numpy as np
 import pandas as pd
 import pandas.testing as tm
@@ -11,11 +10,7 @@ import ibis.expr.datatypes as dt
 import ibis.expr.types as ir
 from ibis.udf.vectorized import analytic, elementwise, reduction
 
-from ..udf import nullable
-
-# --------
-# Fixtures
-# --------
+dd = pytest.importorskip("dask.dataframe")
 
 
 @pytest.fixture
@@ -315,16 +310,6 @@ def test_udaf_groupby_multikey_tzcol(t_timestamp, df_timestamp):
     tm.assert_frame_equal(result, expected)
 
 
-def test_nullable():
-    t = ibis.table([('a', 'int64')])
-    assert nullable(t.a.type()) == (type(None),)
-
-
-def test_nullable_non_nullable_field():
-    t = ibis.table([('a', dt.String(nullable=False))])
-    assert nullable(t.a.type()) == ()
-
-
 def test_compose_udfs(t2, df2):
     expr = times_two(add_one(t2.a))
     result = expr.execute().reset_index(drop=True)
