@@ -1,10 +1,13 @@
 import pandas.testing as tm
-import pyarrow.compute as pc
+import pytest
 
 import ibis.expr.datatypes as dt
 import ibis.expr.types as ir
 from ibis.udf.vectorized import elementwise, reduction
 
+pytest.importorskip("datafusion")
+pc = pytest.importorskip("pyarrow.compute")
+
 
 @elementwise(input_type=['string'], output_type='int64')
 def my_string_length(arr, **kwargs):
