@@ -8,6 +8,8 @@ import ibis.expr.datatypes as dt
 import ibis.expr.schema as sch
 from ibis.backends.impala.pandas_interop import DataFrameWriter  # noqa: E402
 
+pytest.importorskip("impala")
+
 
 @pytest.fixture
 def exhaustive_df():
@@ -71,7 +73,7 @@ def exhaustive_df():
                     8.8000001907348633,
                     9.8999996185302734,
                 ],
-                dtype='f4',
+                dtype='f8',
             ),
             'int_col': np.array([0, 1, 2, 3, 4, 5, 6, 7, 8, 9], dtype='i4'),
             'month': [11, 11, 11, 11, 2, 11, 11, 11, 11, 11],
@@ -125,6 +127,7 @@ def test_alltypes_roundtrip(con, alltypes_df):
     _check_roundtrip(con, alltypes_df)
 
 
+@pytest.mark.hdfs
 def test_writer_cleanup_deletes_hdfs_dir(con, hdfs, alltypes_df):
     writer = DataFrameWriter(con, alltypes_df)
 
@@ -139,6 +142,7 @@ def test_writer_cleanup_deletes_hdfs_dir(con, hdfs, alltypes_df):
     assert not hdfs.exists(path)
 
 
+@pytest.mark.hdfs
 def test_writer_cleanup_context_manager(con, hdfs, alltypes_df):
     with DataFrameWriter(con, alltypes_df) as writer:
         path = writer.write_temp_csv()
@@ -174,11 +178,6 @@ def test_insert(con, temp_table_db, exhaustive_df):
     tm.assert_frame_equal(result, exhaustive_df)
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_insert_partition():
-    assert False
-
-
 def test_round_trip_exhaustive(con, exhaustive_df):
     _check_roundtrip(con, exhaustive_df)
 
