@@ -21,8 +21,7 @@ import ibis.expr.rules as rlz
 import ibis.udf.validate as v
 import ibis.util as util
 from ibis.backends.base.sql.registry import fixed_arity, sql_type_names
-
-from .compiler import ImpalaExprTranslator
+from ibis.backends.impala.compiler import ImpalaExprTranslator
 
 __all__ = [
     'add_operation',
@@ -71,7 +70,8 @@ class ScalarFunction(Function):
         fields = {
             f'_{i}': rlz.value(dtype) for i, dtype in enumerate(self.inputs)
         }
-        fields['output_type'] = rlz.shape_like('args', self.output)
+        fields['output_dtype'] = self.output
+        fields['output_shape'] = rlz.shape_like('args')
         return type(f"UDF_{self.name}", (ops.ValueOp,), fields)
 
 
@@ -80,7 +80,8 @@ class AggregateFunction(Function):
         fields = {
             f'_{i}': rlz.value(dtype) for i, dtype in enumerate(self.inputs)
         }
-        fields['output_type'] = lambda op: self.output.scalar_type()
+        fields['output_dtype'] = self.output
+        fields['output_shape'] = rlz.Shape.SCALAR
         fields['_reduction'] = True
         return type(f"UDA_{self.name}", (ops.ValueOp,), fields)
 
@@ -340,8 +341,8 @@ _impala_to_ibis_type = {
     'smallint': 'int16',
     'int': 'int32',
     'bigint': 'int64',
-    'float': 'float',
-    'double': 'double',
+    'float': 'float32',
+    'double': 'float64',
     'string': 'string',
     'varchar': 'string',
     'char': 'string',
