@@ -675,7 +675,7 @@ class Moving(Window):
     __slots__ = ()
 
     def __init__(self, preceding, max_lookback, *args, **kwargs):
-        from .core import timedelta_types
+        from ibis.backends.pandas.core import timedelta_types
 
         ibis_dtype = getattr(preceding, 'type', lambda: None)()
         preceding = compute_window_spec(ibis_dtype, preceding)
