@@ -120,14 +120,18 @@ import ibis.expr.types as ir
 import ibis.expr.window as win
 import ibis.util
 from ibis.backends.base import BaseBackend
+from ibis.backends.pandas import aggcontext as agg_ctx
+from ibis.backends.pandas.dispatch import (
+    execute_literal,
+    execute_node,
+    post_execute,
+    pre_execute,
+)
+from ibis.backends.pandas.trace import trace
 from ibis.expr.scope import Scope
 from ibis.expr.timecontext import canonicalize_context
 from ibis.expr.typing import TimeContext
 
-from . import aggcontext as agg_ctx
-from .dispatch import execute_literal, execute_node, post_execute, pre_execute
-from .trace import trace
-
 integer_types = np.integer, int
 floating_types = (numbers.Real,)
 numeric_types = integer_types + floating_types
@@ -539,9 +543,9 @@ See ``computable_args`` in ``execute_until_in_scope``
 
 @compute_time_context.register(ops.Node)
 def compute_time_context_default(
-    node,
+    node: ops.Node,
+    scope: Scope,
     timecontext: Optional[TimeContext] = None,
-    scope: Optional[Scope] = None,
     **kwargs,
 ):
     return [timecontext for arg in node.inputs if is_computable_input(arg)]
