@@ -20,14 +20,10 @@ import ibis.common.exceptions as com
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 import ibis.expr.types as ir
-from ibis.expr.scope import Scope
-from ibis.expr.timecontext import get_time_col
-from ibis.expr.typing import TimeContext
-
-from .. import Backend as PandasBackend
-from .. import aggcontext as agg_ctx
-from ..client import PandasTable
-from ..core import (
+from ibis.backends.pandas import Backend as PandasBackend
+from ibis.backends.pandas import aggcontext as agg_ctx
+from ibis.backends.pandas.client import PandasTable
+from ibis.backends.pandas.core import (
     boolean_types,
     execute,
     fixed_width_types,
@@ -39,9 +35,12 @@ from ..core import (
     timedelta_types,
     timestamp_types,
 )
-from ..dispatch import execute_literal, execute_node
-from ..execution import constants
-from ..execution.util import coerce_to_output
+from ibis.backends.pandas.dispatch import execute_literal, execute_node
+from ibis.backends.pandas.execution import constants
+from ibis.backends.pandas.execution.util import coerce_to_output
+from ibis.expr.scope import Scope
+from ibis.expr.timecontext import get_time_col
+from ibis.expr.typing import TimeContext
 
 
 # By default return the literal value
@@ -388,18 +387,34 @@ def execute_table_column_df_or_df_groupby(op, data, **kwargs):
     return data[op.name]
 
 
-@execute_node.register(ops.Aggregation, pd.DataFrame)
+@execute_node.register(
+    ops.Aggregation,
+    pd.DataFrame,
+    tuple,
+    tuple,
+    tuple,
+    tuple,
+    tuple,
+)
 def execute_aggregation_dataframe(
-    op, data, scope=None, timecontext: Optional[TimeContext] = None, **kwargs
+    op,
+    data,
+    metrics,
+    by,
+    having,
+    predicates,
+    sort_keys,
+    scope=None,
+    timecontext: Optional[TimeContext] = None,
+    **kwargs,
 ):
-    assert op.metrics, 'no metrics found during aggregation execution'
+    assert metrics, 'no metrics found during aggregation execution'
 
-    if op.sort_keys:
+    if sort_keys:
         raise NotImplementedError(
             'sorting on aggregations not yet implemented'
         )
 
-    predicates = op.predicates
     if predicates:
         predicate = functools.reduce(
             operator.and_,
@@ -414,7 +429,7 @@ def execute_aggregation_dataframe(
 
     if op.by:
         grouping_key_pairs = list(
-            zip(op.by, map(operator.methodcaller('op'), op.by))
+            zip(by, map(operator.methodcaller('op'), by))
         )
         grouping_keys = [
             by_op.name
@@ -440,21 +455,21 @@ def execute_aggregation_dataframe(
             execute(metric, scope=scope, timecontext=timecontext, **kwargs),
             metric,
         )
-        for metric in op.metrics
+        for metric in metrics
     ]
 
     result = pd.concat(pieces, axis=1)
 
     # If grouping, need a reset to get the grouping key back as a column
-    if op.by:
+    if by:
         result = result.reset_index()
 
     result.columns = [columns.get(c, c) for c in result.columns]
 
-    if op.having:
+    if having:
         # .having(...) is only accessible on groupby, so this should never
         # raise
-        if not op.by:
+        if not by:
             raise ValueError(
                 'Filtering out aggregation values is not allowed without at '
                 'least one grouping key'
@@ -464,8 +479,8 @@ def execute_aggregation_dataframe(
         predicate = functools.reduce(
             operator.and_,
             (
-                execute(having, scope=scope, timecontext=timecontext, **kwargs)
-                for having in op.having
+                execute(h, scope=scope, timecontext=timecontext, **kwargs)
+                for h in having
             ),
         )
         assert len(predicate) == len(
@@ -780,11 +795,6 @@ def execute_between(op, data, lower, upper, **kwargs):
     return data.between(lower, upper)
 
 
-@execute_node.register(ops.DistinctColumn, pd.Series)
-def execute_series_distinct(op, data, **kwargs):
-    return pd.Series(data.unique(), name=data.name)
-
-
 @execute_node.register(ops.Union, pd.DataFrame, pd.DataFrame, bool)
 def execute_union_dataframe_dataframe(
     op, left: pd.DataFrame, right: pd.DataFrame, distinct, **kwargs
@@ -808,7 +818,7 @@ def execute_difference_dataframe_dataframe(
     merged = left.merge(
         right, on=list(left.columns), how='outer', indicator=True
     )
-    result = merged[merged["_merge"] != "both"].drop("_merge", 1)
+    result = merged[merged["_merge"] != "both"].drop("_merge", axis=1)
     return result
 
 
@@ -837,6 +847,13 @@ def execute_node_self_reference_dataframe(op, data, **kwargs):
     return data
 
 
+@execute_node.register(ops.Alias, object)
+def execute_alias(op, _, **kwargs):
+    # just compile the underlying argument because the naming is handled
+    # by the translator for the top level expression
+    return execute(op.arg, **kwargs)
+
+
 @execute_node.register(ops.ValueList, collections.abc.Sequence)
 def execute_node_value_list(op, _, **kwargs):
     return [execute(arg, **kwargs) for arg in op.values]
@@ -963,9 +980,9 @@ def execute_node_log_number_number(op, value, base, **kwargs):
     return math.log(value, base)
 
 
-@execute_node.register(ops.DropNa, pd.DataFrame)
-def execute_node_dropna_dataframe(op, df, **kwargs):
-    subset = [col.get_name() for col in op.subset] if op.subset else None
+@execute_node.register(ops.DropNa, pd.DataFrame, tuple)
+def execute_node_dropna_dataframe(op, df, subset, **kwargs):
+    subset = [col.get_name() for col in subset] if subset else None
     return df.dropna(how=op.how, subset=subset)
 
 
@@ -976,7 +993,7 @@ def execute_node_fillna_dataframe_scalar(op, df, replacements, **kwargs):
 
 @execute_node.register(ops.FillNa, pd.DataFrame)
 def execute_node_fillna_dataframe_dict(op, df, **kwargs):
-    return df.fillna(op.replacements)
+    return df.fillna(dict(op.replacements))
 
 
 @execute_node.register(ops.IfNull, pd.Series, simple_types)
@@ -1022,7 +1039,10 @@ def execute_node_nullif_scalar_series(op, value, series, **kwargs):
 
 
 def coalesce(values):
-    return functools.reduce(lambda x, y: x if not pd.isnull(x) else y, values)
+    return functools.reduce(
+        lambda a1, a2: np.where(pd.isnull(a1), a2, a1),
+        values,
+    )
 
 
 @toolz.curry
