@@ -7,8 +7,7 @@ import pandas as pd
 from pandas.core.groupby import SeriesGroupBy
 
 import ibis.expr.operations as ops
-
-from ..dispatch import execute_node
+from ibis.backends.pandas.dispatch import execute_node
 
 
 @execute_node.register(ops.StructField, collections.abc.Mapping)
