@@ -10,18 +10,12 @@ from multipledispatch import Dispatcher
 from pandas.core.groupby import SeriesGroupBy
 
 import ibis.common.exceptions as com
+import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 import ibis.expr.window as win
-from ibis.expr.scope import Scope
-from ibis.expr.timecontext import (
-    construct_time_context_aware_series,
-    get_time_col,
-)
-from ibis.expr.typing import TimeContext
-
-from .. import aggcontext as agg_ctx
-from ..aggcontext import AggregationContext
-from ..core import (
+from ibis.backends.pandas import aggcontext as agg_ctx
+from ibis.backends.pandas.aggcontext import AggregationContext
+from ibis.backends.pandas.core import (
     compute_time_context,
     date_types,
     execute,
@@ -30,8 +24,14 @@ from ..core import (
     timedelta_types,
     timestamp_types,
 )
-from ..dispatch import execute_node, pre_execute
-from ..execution import util
+from ibis.backends.pandas.dispatch import execute_node, pre_execute
+from ibis.backends.pandas.execution import util
+from ibis.expr.scope import Scope
+from ibis.expr.timecontext import (
+    construct_time_context_aware_series,
+    get_time_col,
+)
+from ibis.expr.typing import TimeContext
 
 
 def _post_process_empty(
@@ -264,6 +264,14 @@ def execute_window_op(
     clients=None,
     **kwargs,
 ):
+    if window.how == "range" and any(
+        not isinstance(ob.type(), (dt.Time, dt.Date, dt.Timestamp))
+        for ob in window._order_by
+    ):
+        raise NotImplementedError(
+            "The pandas backend only implements range windows with temporal "
+            "ordering keys"
+        )
     operand = op.expr
     # pre execute "manually" here because otherwise we wouldn't pickup
     # relevant scope changes from the child operand since we're managing
@@ -273,7 +281,7 @@ def execute_window_op(
     adjusted_timecontext = None
     if timecontext:
         arg_timecontexts = compute_time_context(
-            op, timecontext=timecontext, clients=clients
+            op, timecontext=timecontext, clients=clients, scope=scope
         )
         # timecontext is the original time context required by parent node
         # of this WindowOp, while adjusted_timecontext is the adjusted context
@@ -560,7 +568,16 @@ def execute_series_dense_rank(op, data, **kwargs):
     return data.rank(method='dense', ascending=True).astype('int64') - 1
 
 
-@execute_node.register(ops.PercentRank, (pd.Series, SeriesGroupBy))
+@execute_node.register(ops.PercentRank, SeriesGroupBy)
+def execute_series_group_by_percent_rank(op, data, **kwargs):
+    return (
+        data.rank(method="min", ascending=True)
+        .sub(1)
+        .div(data.transform("count").sub(1))
+    )
+
+
+@execute_node.register(ops.PercentRank, pd.Series)
 def execute_series_percent_rank(op, data, **kwargs):
     # TODO(phillipc): Handle ORDER BY
-    return data.rank(method='min', ascending=True, pct=True)
+    return data.rank(method="min", ascending=True).sub(1).div(len(data) - 1)
