@@ -1,36 +1,17 @@
 import pandas as pd
 import pandas.testing as tm
 import pytest
-from pytest import param
 
 import ibis
-import ibis.common.exceptions as com
-from ibis.backends.pandas import Backend
 
-join_type = pytest.mark.parametrize(
+# SEMI and ANTI are checked in backend tests
+mutating_join_type = pytest.mark.parametrize(
     'how',
-    [
-        'inner',
-        'left',
-        'right',
-        'outer',
-        param(
-            'semi',
-            marks=pytest.mark.xfail(
-                raises=NotImplementedError, reason='Semi join not implemented'
-            ),
-        ),
-        param(
-            'anti',
-            marks=pytest.mark.xfail(
-                raises=NotImplementedError, reason='Anti join not implemented'
-            ),
-        ),
-    ],
+    ['inner', 'left', 'right', 'outer'],
 )
 
 
-@join_type
+@mutating_join_type
 def test_join(how, left, right, df1, df2):
     expr = left.join(right, left.key == right.key, how=how)[
         left, right.other_value, right.key3
@@ -50,7 +31,7 @@ def test_cross_join(left, right, df1, df2):
     tm.assert_frame_equal(result[expected.columns], expected)
 
 
-@join_type
+@mutating_join_type
 def test_join_project_left_table(how, left, right, df1, df2):
     expr = left.join(right, left.key == right.key, how=how)[left, right.key3]
     result = expr.execute()
@@ -69,7 +50,7 @@ def test_cross_join_project_left_table(left, right, df1, df2):
     tm.assert_frame_equal(result[expected.columns], expected)
 
 
-@join_type
+@mutating_join_type
 def test_join_with_multiple_predicates(how, left, right, df1, df2):
     expr = left.join(
         right, [left.key == right.key, left.key2 == right.key3], how=how
@@ -81,7 +62,7 @@ def test_join_with_multiple_predicates(how, left, right, df1, df2):
     tm.assert_frame_equal(result[expected.columns], expected)
 
 
-@join_type
+@mutating_join_type
 def test_join_with_multiple_predicates_written_as_one(
     how, left, right, df1, df2
 ):
@@ -96,7 +77,7 @@ def test_join_with_multiple_predicates_written_as_one(
     tm.assert_frame_equal(result[expected.columns], expected)
 
 
-@join_type
+@mutating_join_type
 def test_join_with_invalid_predicates(how, left, right):
     predicate = (left.key == right.key) & (left.key2 <= right.key3)
     expr = left.join(right, predicate, how=how)
@@ -109,7 +90,7 @@ def test_join_with_invalid_predicates(how, left, right):
         expr.execute()
 
 
-@join_type
+@mutating_join_type
 @pytest.mark.xfail(reason='Hard to detect this case')
 def test_join_with_duplicate_non_key_columns(how, left, right, df1, df2):
     left = left.mutate(x=left.value * 2)
@@ -122,7 +103,7 @@ def test_join_with_duplicate_non_key_columns(how, left, right, df1, df2):
         expr.execute()
 
 
-@join_type
+@mutating_join_type
 def test_join_with_duplicate_non_key_columns_not_selected(
     how, left, right, df1, df2
 ):
@@ -142,7 +123,7 @@ def test_join_with_duplicate_non_key_columns_not_selected(
     tm.assert_frame_equal(result[expected.columns], expected)
 
 
-@join_type
+@mutating_join_type
 def test_join_with_post_expression_selection(how, left, right, df1, df2):
     join = left.join(right, left.key == right.key, how=how)
     expr = join[left.key, left.value, right.other_value]
@@ -153,7 +134,7 @@ def test_join_with_post_expression_selection(how, left, right, df1, df2):
     tm.assert_frame_equal(result[expected.columns], expected)
 
 
-@join_type
+@mutating_join_type
 def test_join_with_post_expression_filter(how, left):
     lhs = left[['key', 'key2']]
     rhs = left[['key2', 'value']]
@@ -171,7 +152,7 @@ def test_join_with_post_expression_filter(how, left):
     tm.assert_frame_equal(result, expected)
 
 
-@join_type
+@mutating_join_type
 def test_multi_join_with_post_expression_filter(how, left, df1):
     lhs = left[['key', 'key2']]
     rhs = left[['key2', 'value']]
@@ -198,7 +179,7 @@ def test_multi_join_with_post_expression_filter(how, left, df1):
     tm.assert_frame_equal(result, expected)
 
 
-@join_type
+@mutating_join_type
 def test_join_with_non_trivial_key(how, left, right, df1, df2):
     # also test that the order of operands in the predicate doesn't matter
     join = left.join(right, right.key.length() == left.key.length(), how=how)
@@ -218,7 +199,7 @@ def test_join_with_non_trivial_key(how, left, right, df1, df2):
     tm.assert_frame_equal(result[expected.columns], expected)
 
 
-@join_type
+@mutating_join_type
 def test_join_with_non_trivial_key_project_table(how, left, right, df1, df2):
     # also test that the order of operands in the predicate doesn't matter
     join = left.join(right, right.key.length() == left.key.length(), how=how)
@@ -240,7 +221,7 @@ def test_join_with_non_trivial_key_project_table(how, left, right, df1, df2):
     tm.assert_frame_equal(result[expected.columns], expected)
 
 
-@join_type
+@mutating_join_type
 def test_join_with_project_right_duplicate_column(client, how, left, df1, df3):
     # also test that the order of operands in the predicate doesn't matter
     right = client.table('df3')
@@ -343,12 +324,7 @@ def test_keyed_asof_join_with_tolerance(
     "how",
     [
         "left",
-        pytest.param(
-            "right",
-            marks=pytest.mark.xfail(
-                raises=AttributeError, reason="right_join is not an ibis API"
-            ),
-        ),
+        "right",
         "inner",
         "outer",
     ],
@@ -361,14 +337,10 @@ def test_keyed_asof_join_with_tolerance(
         pytest.param(lambda join: join.select(["a0", "a1"]), id="select"),
     ],
 )
-@pytest.mark.xfail(
-    raises=(com.IbisError, AttributeError),
-    reason="Select from unambiguous joins not implemented",
-)
 def test_select_on_unambiguous_join(how, func):
     df_t = pd.DataFrame({'a0': [1, 2, 3], 'b1': list("aab")})
     df_s = pd.DataFrame({'a1': [2, 3, 4], 'b2': list("abc")})
-    con = Backend().connect({"t": df_t, "s": df_s})
+    con = ibis.pandas.connect({"t": df_t, "s": df_s})
     t = con.table("t")
     s = con.table("s")
     method = getattr(t, f"{how}_join")
@@ -390,10 +362,6 @@ def test_select_on_unambiguous_join(how, func):
         pytest.param(lambda join: join.select(["a0", "a1"]), id="select"),
     ],
 )
-@pytest.mark.xfail(
-    raises=(com.IbisError, AttributeError),
-    reason="Select from unambiguous joins not implemented",
-)
 @merge_asof_minversion
 def test_select_on_unambiguous_asof_join(func):
     df_t = pd.DataFrame(
@@ -402,7 +370,7 @@ def test_select_on_unambiguous_asof_join(func):
     df_s = pd.DataFrame(
         {'a1': [2, 3, 4], 'b2': pd.date_range("20171230", periods=3)}
     )
-    con = Backend().connect({"t": df_t, "s": df_s})
+    con = ibis.pandas.connect({"t": df_t, "s": df_s})
     t = con.table("t")
     s = con.table("s")
     join = t.asof_join(s, t.b1 == s.b2)
@@ -415,7 +383,7 @@ def test_select_on_unambiguous_asof_join(func):
     tm.assert_frame_equal(result, expected)
 
 
-def test_materialized_join():
+def test_outer_join():
     df = pd.DataFrame({"test": [1, 2, 3], "name": ["a", "b", "c"]})
     df_2 = pd.DataFrame({"test_2": [1, 5, 6], "name_2": ["d", "e", "f"]})
 
@@ -428,7 +396,6 @@ def test_materialized_join():
         ibis_table_2,
         predicates=ibis_table_1["test"] == ibis_table_2["test_2"],
     )
-    joined = joined.materialize()
     result = joined.execute()
     expected = pd.merge(
         df,
@@ -438,3 +405,62 @@ def test_materialized_join():
         how="outer",
     )
     tm.assert_frame_equal(result, expected)
+
+
+def test_mutate_after_join():
+    # GH3090
+    df = pd.DataFrame(
+        {
+            "p_Order_Priority": ["C", "H", "L", "M"],
+            "p_count": [9, 9, 15, 11],
+            "p_density": [0.204545, 0.204545, 0.340909, 0.250000],
+        }
+    )
+    df_2 = pd.DataFrame(
+        {
+            "q_Order_Priority": ["C", "H", "L", "M"],
+            "q_count": [13, 21, 12, 10],
+            "q_density": [0.232143, 0.375000, 0.214286, 0.178571],
+        }
+    )
+
+    conn = ibis.pandas.connect({"df": df, "df_2": df_2})
+
+    ibis_table_1 = conn.table("df")
+    ibis_table_2 = conn.table("df_2")
+
+    joined = ibis_table_1.outer_join(
+        ibis_table_2,
+        predicates=(
+            ibis_table_1["p_Order_Priority"]
+            == ibis_table_2["q_Order_Priority"]
+        ),
+    )
+
+    joined = joined.mutate(
+        bins=(
+            joined["p_Order_Priority"]
+            .isnull()
+            .ifelse(joined["q_Order_Priority"], joined["p_Order_Priority"])
+        ),
+        p_count=joined["p_count"].fillna(0),
+        q_count=joined["q_count"].fillna(0),
+        p_density=joined.p_density.fillna(1e-10),
+        q_density=joined.q_density.fillna(1e-10),
+        features="Order_Priority",
+    )
+
+    expected = pd.DataFrame(
+        {
+            "p_Order_Priority": list("CHLM"),
+            "p_count": [9, 9, 15, 11],
+            "p_density": [0.204545, 0.204545, 0.340909, 0.250000],
+            "q_Order_Priority": list("CHLM"),
+            "q_count": [13, 21, 12, 10],
+            "q_density": [0.232143, 0.375000, 0.214286, 0.178571],
+            "bins": list("CHLM"),
+            "features": ["Order_Priority"] * 4,
+        }
+    )
+    result = joined.execute()
+    tm.assert_frame_equal(result, expected)
