@@ -345,15 +345,10 @@ def test_frame_limit(t, df, offset):
     tm.assert_frame_equal(result[expected.columns], expected)
 
 
-@pytest.mark.xfail(
-    raises=AttributeError, reason='TableColumn does not implement limit'
-)
 @pytest.mark.parametrize('offset', [0, 2])
 def test_series_limit(t, df, offset):
-    n = 5
-    s_expr = t.plain_int64.limit(n, offset=offset)
-    result = s_expr.execute()
-    tm.assert_series_equal(result, df.plain_int64.iloc[offset : offset + n])
+    with pytest.raises(AttributeError):
+        t.plain_int64.limit(5, offset=offset)
 
 
 @pytest.mark.parametrize(
@@ -400,13 +395,6 @@ def test_complex_sort_by(t, df):
     tm.assert_frame_equal(result[expected.columns], expected)
 
 
-def test_distinct(t, df):
-    expr = t.dup_strings.distinct()
-    result = expr.execute()
-    expected = pd.Series(df.dup_strings.unique(), name='dup_strings')
-    tm.assert_series_equal(result, expected)
-
-
 def test_count_distinct(t, df):
     expr = t.dup_strings.nunique()
     result = expr.execute()
@@ -485,7 +473,7 @@ def test_mutate_after_group_by(t, df):
 def test_groupby_with_unnamed_arithmetic(t, df):
     expr = t.groupby(t.dup_strings).aggregate(
         naive_variance=(
-            (t.plain_float64 ** 2).sum() - t.plain_float64.mean() ** 2
+            (t.plain_float64**2).sum() - t.plain_float64.mean() ** 2
         )
         / t.plain_float64.count()
     )
@@ -494,7 +482,7 @@ def test_groupby_with_unnamed_arithmetic(t, df):
         df.groupby('dup_strings')
         .agg(
             {
-                'plain_float64': lambda x: ((x ** 2).sum() - x.mean() ** 2)
+                'plain_float64': lambda x: ((x**2).sum() - x.mean() ** 2)
                 / x.count()
             }
         )
@@ -682,7 +670,7 @@ def test_summary_execute(t):
 
 
 def test_summary_numeric(batting, batting_df):
-    expr = batting.G.summary()
+    expr = batting.aggregate(batting.G.summary())
     result = expr.execute()
     assert len(result) == 1
 
@@ -728,7 +716,7 @@ def test_summary_numeric_group_by(batting, batting_df):
 
 
 def test_summary_non_numeric(batting, batting_df):
-    expr = batting.teamID.summary()
+    expr = batting.aggregate(batting.teamID.summary())
     result = expr.execute()
     assert len(result) == 1
     assert len(result.columns) == 3
@@ -855,7 +843,7 @@ def test_difference(client, df1, intersect_df2):
     merged = df1.merge(
         intersect_df2, on=list(df1.columns), how="outer", indicator=True
     )
-    expected = merged[merged["_merge"] != "both"].drop("_merge", 1)
+    expected = merged[merged["_merge"] != "both"].drop("_merge", axis=1)
     tm.assert_frame_equal(result, expected)
 
 
