@@ -1,9 +1,12 @@
+import io
+from datetime import date
 from operator import methodcaller
 
 import numpy as np
 import pandas as pd
 import pytest
-from pandas.util import testing as tm
+from packaging.version import parse as vparse
+from pandas import testing as tm
 
 import ibis
 import ibis.common.exceptions as com
@@ -67,7 +70,7 @@ class CustomAggContext(AggregationContext):
     def agg(self, grouped_data, function, *args, **kwargs):
         upper_indices = pd.Series(range(1, len(self.parent) + 2))
         window_sizes = (
-            grouped_data.rolling(self.preceding.value + 1)
+            grouped_data.rolling(self.preceding.value + 1, min_periods=0)
             .count()
             .reset_index(drop=True)
         )
@@ -497,7 +500,7 @@ def test_project_scalar_after_join():
             'value': [0, 1, np.nan, 3, 4, np.nan, 6, 7, 8],
         }
     )
-    con = Backend().connect({'left': left_df, 'right': right_df})
+    con = ibis.pandas.connect({'left': left_df, 'right': right_df})
     left, right = map(con.table, ('left', 'right'))
 
     joined = left.outer_join(right, left.ints == right.group)
@@ -510,12 +513,13 @@ def test_project_scalar_after_join():
 
 def test_project_list_scalar():
     df = pd.DataFrame({'ints': range(3)})
-    con = Backend().connect({'df': df})
-    expr = con.table('df')
-    result = expr.mutate(res=expr.ints.quantile([0.5, 0.95])).execute()
-    tm.assert_series_equal(
-        result.res, pd.Series([[1.0, 1.9] for _ in range(0, 3)], name='res')
-    )
+    con = ibis.pandas.connect({'df': df})
+    table = con.table('df')
+    expr = table.mutate(res=table.ints.quantile([0.5, 0.95]))
+    result = expr.execute()
+
+    expected = pd.Series([[1.0, 1.9] for _ in range(0, 3)], name='res')
+    tm.assert_series_equal(result.res, expected)
 
 
 @pytest.mark.parametrize(
@@ -530,7 +534,7 @@ def test_window_with_preceding_expr(index):
     start = 2
     data = np.arange(start, start + len(time))
     df = pd.DataFrame({'value': data, 'time': time}, index=index(time))
-    client = Backend().connect({'df': df})
+    client = ibis.pandas.connect({'df': df})
     t = client.table('df')
     expected = (
         df.set_index('time')
@@ -546,6 +550,11 @@ def test_window_with_preceding_expr(index):
     tm.assert_series_equal(result, expected)
 
 
+@pytest.mark.xfail(
+    condition=vparse("1.4") <= vparse(pd.__version__) < vparse("1.4.2"),
+    raises=ValueError,
+    reason="https://github.com/pandas-dev/pandas/pull/44068",
+)
 def test_window_with_mlb():
     index = pd.date_range('20170501', '20170507')
     data = np.random.randn(len(index), 3)
@@ -554,7 +563,7 @@ def test_window_with_mlb():
         .rename_axis('time')
         .reset_index(drop=False)
     )
-    client = Backend().connect({'df': df})
+    client = ibis.pandas.connect({'df': df})
     t = client.table('df')
     rows_with_mlb = rows_with_max_lookback(5, ibis.interval(days=10))
     expr = t.mutate(
@@ -584,17 +593,16 @@ def test_window_with_mlb():
 
 
 def test_window_has_pre_execute_scope():
-    signature = ops.Lag, Backend
     called = [0]
 
-    @pre_execute.register(*signature)
+    @pre_execute.register(ops.Lag, Backend)
     def test_pre_execute(op, client, **kwargs):
         called[0] += 1
         return Scope()
 
     data = {'key': list('abc'), 'value': [1, 2, 3], 'dup': list('ggh')}
     df = pd.DataFrame(data, columns=['key', 'value', 'dup'])
-    client = Backend().connect({'df': df})
+    client = ibis.pandas.connect({'df': df})
     t = client.table('df')
     window = ibis.window(order_by='value')
     expr = t.key.lag(1).over(window).name('foo')
@@ -604,7 +612,10 @@ def test_window_has_pre_execute_scope():
     # once in window op at the top to pickup any scope changes before computing
     # twice in window op when calling execute on the ops.Lag node at the
     # beginning of execute and once before the actual computation
-    assert called[0] == 3
+    #
+    # this process happens twice because of the pre_execute call on the Alias
+    # operation
+    assert called[0] == 3 + 3
 
 
 def test_window_grouping_key_has_scope(t, df):
@@ -769,3 +780,66 @@ def test_rolling_window_udf_nan_and_non_numeric(t, group_by, order_by):
     tm.assert_series_equal(result_nan, expected, check_names=False)
     tm.assert_series_equal(result_non_numeric, expected, check_names=False)
     tm.assert_series_equal(result_nan_non_numeric, expected, check_names=False)
+
+
+@pytest.fixture
+def events():
+    df = pd.DataFrame(
+        {
+            "event_id": [1] * 4 + [2] * 6 + [3] * 2,
+            "measured_on": map(
+                pd.Timestamp,
+                map(
+                    date,
+                    [2021] * 12,
+                    [6] * 4 + [5] * 6 + [7] * 2,
+                    range(1, 13),
+                ),
+            ),
+            "measurement": np.nan,
+        }
+    )
+    df.at[1, "measurement"] = 5.0
+    df.at[4, "measurement"] = 42.0
+    df.at[5, "measurement"] = 42.0
+    df.at[7, "measurement"] = 11.0
+    return df
+
+
+def test_bfill(events):
+    con = ibis.pandas.connect({"t": events})
+    t = con.table("t")
+
+    win = ibis.window(
+        group_by=t.event_id, order_by=ibis.desc(t.measured_on), following=0
+    )
+    grouped = t.mutate(grouper=t.measurement.count().over(win))
+
+    expr = (
+        grouped.group_by([grouped.event_id, grouped.grouper])
+        .mutate(bfill=grouped.measurement.max())
+        .sort_by("measured_on")
+    )
+    result = expr.execute().reset_index(drop=True)
+
+    expected_raw = """\
+event_id measured_on  measurement  grouper  bfill
+       2  2021-05-05         42.0        3   42.0
+       2  2021-05-06         42.0        2   42.0
+       2  2021-05-07          NaN        1   11.0
+       2  2021-05-08         11.0        1   11.0
+       2  2021-05-09          NaN        0    NaN
+       2  2021-05-10          NaN        0    NaN
+       1  2021-06-01          NaN        1    5.0
+       1  2021-06-02          5.0        1    5.0
+       1  2021-06-03          NaN        0    NaN
+       1  2021-06-04          NaN        0    NaN
+       3  2021-07-11          NaN        0    NaN
+       3  2021-07-12          NaN        0    NaN"""
+    expected = pd.read_csv(
+        io.StringIO(expected_raw),
+        sep=r"\s+",
+        header=0,
+        parse_dates=["measured_on"],
+    )
+    tm.assert_frame_equal(result, expected)
