@@ -4,12 +4,13 @@ import numpy as np
 import pandas as pd
 import pandas._testing as tm
 import pytest
+from packaging.version import parse as vparse
 
 import ibis
 import ibis.expr.datatypes as dt
 import ibis.expr.types as ir
 from ibis.backends.pandas import Backend
-from ibis.backends.pandas.udf import nullable, udf
+from ibis.backends.pandas.udf import udf
 
 
 @pytest.fixture
@@ -211,16 +212,6 @@ def test_udaf_groupby():
     tm.assert_frame_equal(result[columns], expected[columns])
 
 
-def test_nullable():
-    t = ibis.table([('a', 'int64')])
-    assert nullable(t.a.type()) == (type(None),)
-
-
-def test_nullable_non_nullable_field():
-    t = ibis.table([('a', dt.String(nullable=False))])
-    assert nullable(t.a.type()) == ()
-
-
 def test_udaf_parameter_mismatch():
     with pytest.raises(TypeError):
 
@@ -266,6 +257,11 @@ def test_udaf_window(t2, df2):
     tm.assert_frame_equal(result, expected)
 
 
+@pytest.mark.xfail(
+    condition=vparse("1.4") <= vparse(pd.__version__) < vparse("1.4.2"),
+    raises=ValueError,
+    reason="https://github.com/pandas-dev/pandas/pull/44068",
+)
 def test_udaf_window_interval():
     df = pd.DataFrame(
         collections.OrderedDict(
