@@ -1,12 +1,18 @@
 """PostgreSQL backend."""
+
+from __future__ import annotations
+
 import contextlib
+from typing import Literal
 
-import sqlalchemy
+import sqlalchemy as sa
 
+import ibis.expr.schema as sch
+from ibis import util
 from ibis.backends.base.sql.alchemy import BaseAlchemyBackend
-
-from . import udf
-from .compiler import PostgreSQLCompiler
+from ibis.backends.postgres.compiler import PostgreSQLCompiler
+from ibis.backends.postgres.datatypes import _get_type
+from ibis.backends.postgres.udf import udf
 
 
 class Backend(BaseAlchemyBackend):
@@ -15,37 +21,40 @@ class Backend(BaseAlchemyBackend):
 
     def do_connect(
         self,
-        host='localhost',
-        user=None,
-        password=None,
-        port=5432,
-        database=None,
-        url=None,
-        driver='psycopg2',
-    ):
-        """Create an Ibis client located at `user`:`password`@`host`:`port`
-        connected to a PostgreSQL database named `database`.
+        host: str = 'localhost',
+        user: str | None = None,
+        password: str | None = None,
+        port: int = 5432,
+        database: str | None = None,
+        url: str | None = None,
+        driver: Literal["psycopg2"] = "psycopg2",
+    ) -> None:
+        """Create an Ibis client connected to PostgreSQL database.
 
         Parameters
         ----------
-        host : string, default 'localhost'
-        user : string, default None
-        password : string, default None
-        port : string or integer, default 5432
-        database : string, default None
-        url : string, default None
-            Complete SQLAlchemy connection string. If passed, the other
-            connection arguments are ignored.
-        driver : string, default 'psycopg2'
+        host
+            Hostname
+        user
+            Username
+        password
+            Password
+        port
+            Port number
+        database
+            Database to connect to
+        url
+            SQLAlchemy connection string.
 
-        Returns
-        -------
-        Backend
+            If passed, the other connection arguments are ignored.
+        driver
+            Database driver
 
         Examples
         --------
         >>> import os
         >>> import getpass
+        >>> import ibis
         >>> host = os.environ.get('IBIS_TEST_POSTGRES_HOST', 'localhost')
         >>> user = os.environ.get('IBIS_TEST_POSTGRES_USER', getpass.getuser())
         >>> password = os.environ.get('IBIS_TEST_POSTGRES_PASSWORD')
@@ -94,7 +103,7 @@ class Backend(BaseAlchemyBackend):
             driver=f'postgresql+{driver}',
         )
         self.database_name = alchemy_url.database
-        super().do_connect(sqlalchemy.create_engine(alchemy_url))
+        super().do_connect(sa.create_engine(alchemy_url))
 
     def list_databases(self, like=None):
         # http://dba.stackexchange.com/a/1304/58517
@@ -106,18 +115,13 @@ class Backend(BaseAlchemyBackend):
         ]
         return self._filter_with_like(databases, like)
 
+    @util.deprecated(version='2.0', instead='use `list_databases`')
     def list_schemas(self, like=None):
         """List all the schemas in the current database."""
         # In Postgres we support schemas, which in other engines (e.g. MySQL)
         # are databases
         return super().list_databases(like)
 
-    def list_tables(self, like=None, database=None):
-        # PostgreSQL requires passing `database=None` for current database
-        if database == self.current_database:
-            database = None
-        return super().list_tables(like, database)
-
     @contextlib.contextmanager
     def begin(self):
         with super().begin() as bind:
@@ -128,54 +132,39 @@ class Backend(BaseAlchemyBackend):
             finally:
                 bind.execute(f"SET TIMEZONE = '{previous_timezone}'")
 
-    def table(self, name, database=None, schema=None):
-        """Create a table expression that references a particular a table
-        called `name` in a PostgreSQL database called `database`.
-
-        Parameters
-        ----------
-        name : str
-            The name of the table to retrieve.
-        database : str, optional
-            The database in which the table referred to by `name` resides. If
-            ``None`` then the ``current_database`` is used.
-        schema : str, optional
-            The schema in which the table resides.  If ``None`` then the
-            `public` schema is assumed.
-
-        Returns
-        -------
-        table : TableExpr
-            A table expression.
-        """
-        if database is not None and database != self.current_database:
-            return self.database(name=database).table(name=name, schema=schema)
-        else:
-            alch_table = self._get_sqla_table(name, schema=schema)
-            node = self.table_class(alch_table, self, self._schemas.get(name))
-            return self.table_expr_class(node)
-
     def udf(
-        self, pyfunc, in_types, out_type, schema=None, replace=False, name=None
+        self,
+        pyfunc,
+        in_types,
+        out_type,
+        schema=None,
+        replace=False,
+        name=None,
+        language="plpythonu",
     ):
-        """Decorator that defines a PL/Python UDF in-database based on the
-        wrapped function and turns it into an ibis function expression.
+        """Decorator that defines a PL/Python UDF in-database.
 
         Parameters
         ----------
-        pyfunc : function
-        in_types : List[ibis.expr.datatypes.DataType]
-        out_type : ibis.expr.datatypes.DataType
-        schema : str
-            optionally specify the schema in which to define the UDF
-        replace : bool
+        pyfunc
+            Python function
+        in_types
+            Input types
+        out_type
+            Output type
+        schema
+            The postgres schema in which to define the UDF
+        replace
             replace UDF in database if already exists
-        name: str
+        name
             name for the UDF to be defined in database
+        language
+            Language extension to use for PL/Python
 
         Returns
         -------
         Callable
+            A callable ibis expression
 
         Function that takes in ColumnExpr arguments and returns an instance
         inheriting from PostgresUDFNode
@@ -189,4 +178,34 @@ class Backend(BaseAlchemyBackend):
             schema=schema,
             replace=replace,
             name=name,
+            language=language,
         )
+
+    def _get_schema_using_query(self, query: str) -> sch.Schema:
+        raw_name = util.guid()
+        name = self.con.dialect.identifier_preparer.quote_identifier(raw_name)
+        type_info_sql = f"""\
+SELECT
+  attname,
+  format_type(atttypid, atttypmod) AS type
+FROM pg_attribute
+WHERE attrelid = {raw_name!r}::regclass
+  AND attnum > 0
+  AND NOT attisdropped
+ORDER BY attnum
+"""
+        with self.con.connect() as con:
+            con.execute(f"CREATE TEMPORARY VIEW {name} AS {query}")
+            try:
+                type_info = con.execute(type_info_sql).fetchall()
+            finally:
+                con.execute(f"DROP VIEW {name}")
+        tuples = [(col, _get_type(typestr)) for col, typestr in type_info]
+        return sch.Schema.from_tuples(tuples)
+
+    def _get_temp_view_definition(
+        self,
+        name: str,
+        definition: sa.sql.compiler.Compiled,
+    ) -> str:
+        return f"CREATE OR REPLACE TEMPORARY VIEW {name} AS {definition}"
