@@ -1,25 +1,31 @@
+import toolz
 from sqlalchemy.dialects import postgresql
 
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
+import ibis.expr.rules as rlz
 from ibis.backends.base.sql.alchemy import (
     AlchemyCompiler,
     AlchemyExprTranslator,
 )
-
-from .registry import operation_registry
+from ibis.backends.base.sql.alchemy.registry import _geospatial_functions
+from ibis.backends.postgres.registry import operation_registry
 
 
 class PostgresUDFNode(ops.ValueOp):
-    pass
+    output_shape = rlz.shape_like("args")
 
 
 class PostgreSQLExprTranslator(AlchemyExprTranslator):
-    _registry = operation_registry
+    _registry = toolz.merge(operation_registry, _geospatial_functions)
     _rewrites = AlchemyExprTranslator._rewrites.copy()
     _type_map = AlchemyExprTranslator._type_map.copy()
     _type_map.update(
-        {dt.Double: postgresql.DOUBLE_PRECISION, dt.Float: postgresql.REAL}
+        {
+            dt.Float16: postgresql.REAL,
+            dt.Float32: postgresql.REAL,
+            dt.Float64: postgresql.DOUBLE_PRECISION,
+        }
     )
 
 
