@@ -8,9 +8,7 @@ import numpy as np
 import pandas as pd
 import pandas.testing as tm
 import pytest
-import sqlalchemy as sa
 from pytest import param
-from sqlalchemy.dialects import postgresql
 
 import ibis
 import ibis.config as config
@@ -19,6 +17,11 @@ import ibis.expr.types as ir
 from ibis import literal as L
 from ibis.expr.window import rows_with_max_lookback
 
+pytest.importorskip("psycopg2")
+sa = pytest.importorskip("sqlalchemy")
+
+from sqlalchemy.dialects import postgresql  # noqa: E402
+
 
 @pytest.fixture
 def guid(con):
@@ -57,13 +60,13 @@ def guid2(con):
             id='string_to_double',
         ),
         param(
-            lambda t: t.string_col.cast('float'),
-            lambda at: sa.cast(at.c.string_col, sa.REAL),
+            lambda t: t.string_col.cast('float32'),
+            lambda at: sa.cast(at.c.string_col, postgresql.REAL),
             id='string_to_float',
         ),
         param(
             lambda t: t.string_col.cast('decimal'),
-            lambda at: sa.cast(at.c.string_col, sa.NUMERIC(9, 0)),
+            lambda at: sa.cast(at.c.string_col, sa.NUMERIC()),
             id='string_to_decimal_no_params',
         ),
         param(
@@ -460,11 +463,6 @@ def test_coalesce(con, expr, expected):
     ('expr', 'expected'),
     [
         param(ibis.coalesce(ibis.NA, ibis.NA), None, id='all_null'),
-        param(
-            ibis.coalesce(ibis.NA, ibis.NA, ibis.NA.cast('double')),
-            None,
-            id='all_nulls_with_one_cast',
-        ),
         param(
             ibis.coalesce(
                 ibis.NA.cast('int8'),
@@ -477,7 +475,12 @@ def test_coalesce(con, expr, expected):
     ],
 )
 def test_coalesce_all_na(con, expr, expected):
-    assert con.execute(expr) == expected
+    assert con.execute(expr) is None
+
+
+def test_coalesce_all_na_double(con):
+    expr = ibis.coalesce(ibis.NA, ibis.NA, ibis.NA.cast('double'))
+    assert np.isnan(con.execute(expr))
 
 
 def test_numeric_builtins_work(alltypes, df):
@@ -1092,35 +1095,9 @@ def test_array_collect(array_types):
         (None, 3),
         (None, None),
         (3, None),
-        # negative slices are not supported
-        param(
-            -3,
-            None,
-            marks=pytest.mark.xfail(
-                raises=ValueError, reason='Negative slicing not supported'
-            ),
-        ),
-        param(
-            None,
-            -3,
-            marks=pytest.mark.xfail(
-                raises=ValueError, reason='Negative slicing not supported'
-            ),
-        ),
-        param(
-            -3,
-            -1,
-            marks=pytest.mark.xfail(
-                raises=ValueError, reason='Negative slicing not supported'
-            ),
-        ),
-        param(
-            -3,
-            -1,
-            marks=pytest.mark.xfail(
-                raises=ValueError, reason='Negative slicing not supported'
-            ),
-        ),
+        (-3, None),
+        (None, -3),
+        (-3, -1),
     ],
 )
 def test_array_slice(array_types, start, stop):
@@ -1132,14 +1109,14 @@ def test_array_slice(array_types, start, stop):
     tm.assert_frame_equal(result, expected)
 
 
-@pytest.mark.parametrize('index', [1, 3, 4, 11])
+@pytest.mark.parametrize('index', [0, 1, 3, 4, 11, -1, -3, -4, -11])
 def test_array_index(array_types, index):
     expr = array_types[array_types.y[index].name('indexed')]
     result = expr.execute()
     expected = pd.DataFrame(
         {
             'indexed': array_types.y.execute().map(
-                lambda x: x[index] if index < len(x) else None
+                lambda x: x[index] if -len(x) <= index < len(x) else None
             )
         }
     )
@@ -1239,8 +1216,10 @@ def test_semi_join(t, s):
     t_a, s_a = t.op().sqla_table.alias('t0'), s.op().sqla_table.alias('t1')
     expr = t.semi_join(s, t.id == s.id)
     result = expr.compile().compile(compile_kwargs={'literal_binds': True})
-    base = sa.select([t_a.c.id, t_a.c.name]).where(
-        sa.exists(sa.select([1]).where(t_a.c.id == s_a.c.id))
+    base = (
+        sa.select([t_a.c.id, t_a.c.name])
+        .where(sa.exists(sa.select([1]).where(t_a.c.id == s_a.c.id)))
+        .subquery()
     )
     expected = sa.select([base.c.id, base.c.name])
     assert str(result) == str(expected)
@@ -1250,8 +1229,10 @@ def test_anti_join(t, s):
     t_a, s_a = t.op().sqla_table.alias('t0'), s.op().sqla_table.alias('t1')
     expr = t.anti_join(s, t.id == s.id)
     result = expr.compile().compile(compile_kwargs={'literal_binds': True})
-    base = sa.select([t_a.c.id, t_a.c.name]).where(
-        ~sa.exists(sa.select([1]).where(t_a.c.id == s_a.c.id))
+    base = (
+        sa.select([t_a.c.id, t_a.c.name])
+        .where(~sa.exists(sa.select([1]).where(t_a.c.id == s_a.c.id)))
+        .subquery()
     )
     expected = sa.select([base.c.id, base.c.name])
     assert str(result) == str(expected)
@@ -1260,11 +1241,11 @@ def test_anti_join(t, s):
 def test_create_table_from_expr(con, trunc, guid2):
     con.create_table(guid2, expr=trunc)
     t = con.table(guid2)
-    assert list(t.name.execute()) == list('abc')
+    assert list(t['name'].execute()) == list('abc')
 
 
 def test_truncate_table(con, trunc):
-    assert list(trunc.name.execute()) == list('abc')
+    assert list(trunc['name'].execute()) == list('abc')
     con.truncate_table(trunc.op().name)
     assert not len(trunc.execute())
 
@@ -1364,22 +1345,6 @@ def test_negate_boolean(con, df):
     tm.assert_series_equal(result, expected)
 
 
-@pytest.mark.parametrize(
-    ('opname', 'expected'),
-    [
-        ('year', {2009, 2010}),
-        ('month', set(range(1, 13))),
-        ('day', set(range(1, 32))),
-    ],
-)
-def test_date_extract_field(db, opname, expected):
-    op = operator.methodcaller(opname)
-    t = db.functional_alltypes
-    expr = op(t.timestamp_col.cast('date')).distinct()
-    result = expr.execute().astype(int)
-    assert set(result) == expected
-
-
 @pytest.mark.parametrize('opname', ['sum', 'mean', 'min', 'max', 'std', 'var'])
 def test_boolean_reduction(alltypes, opname, df):
     op = operator.methodcaller(opname)
@@ -1581,12 +1546,13 @@ def test_string_to_binary_cast(con):
     t = con.table('functional_alltypes').limit(10)
     expr = t.string_col.cast('binary')
     result = expr.execute()
+    name = expr.get_name()
     sql_string = (
-        "SELECT decode(string_col, 'escape') AS tmp "
+        f"SELECT decode(string_col, 'escape') AS \"{name}\" "
         "FROM functional_alltypes LIMIT 10"
     )
     raw_data = [row[0][0] for row in con.raw_sql(sql_string).fetchall()]
-    expected = pd.Series(raw_data, name='tmp')
+    expected = pd.Series(raw_data, name=name)
     tm.assert_series_equal(result, expected)
 
 
@@ -1594,11 +1560,14 @@ def test_string_to_binary_round_trip(con):
     t = con.table('functional_alltypes').limit(10)
     expr = t.string_col.cast('binary').cast('string')
     result = expr.execute()
+    name = expr.get_name()
     sql_string = (
-        "SELECT encode(decode(string_col, 'escape'), 'escape') AS tmp "
+        "SELECT encode(decode(string_col, 'escape'), 'escape') AS "
+        f"\"{name}\""
         "FROM functional_alltypes LIMIT 10"
     )
     expected = pd.Series(
-        [row[0][0] for row in con.raw_sql(sql_string).fetchall()], name='tmp'
+        [row[0][0] for row in con.raw_sql(sql_string).fetchall()],
+        name=name,
     )
     tm.assert_series_equal(result, expected)
