@@ -3,7 +3,6 @@ import enum
 import functools
 import operator
 
-import numpy as np
 import pandas as pd
 import pyspark
 import pyspark.sql.functions as F
@@ -17,15 +16,17 @@ import ibis.expr.types as ir
 import ibis.expr.types as types
 from ibis import interval
 from ibis.backends.pandas.execution import execute
-from ibis.expr.timecontext import adjust_context
-from ibis.util import guid
-
-from .datatypes import (
+from ibis.backends.pyspark.datatypes import (
     ibis_array_dtype_to_spark_dtype,
     ibis_dtype_to_spark_dtype,
     spark_dtype,
 )
-from .timecontext import combine_time_context, filter_by_time_context
+from ibis.backends.pyspark.timecontext import (
+    combine_time_context,
+    filter_by_time_context,
+)
+from ibis.expr.timecontext import adjust_context
+from ibis.util import frozendict, guid
 
 
 class PySparkDatabaseTable(ops.DatabaseTable):
@@ -114,6 +115,13 @@ def _can_be_replaced_by_column_name(column_expr, table):
     )
 
 
+@compiles(ops.Alias)
+def compile_alias(t, expr, scope, timecontext, **kwargs):
+    op = expr.op()
+    arg = t.translate(op.arg, scope, timecontext, **kwargs)
+    return arg.alias(op.name)
+
+
 @compiles(ops.Selection)
 def compile_selection(t, expr, scope, timecontext, **kwargs):
     op = expr.op()
@@ -123,7 +131,7 @@ def compile_selection(t, expr, scope, timecontext, **kwargs):
     # time context among child nodes, and pass this as context to
     # source table to get all data within time context loaded.
     arg_timecontexts = [
-        adjust_context(node.op(), timecontext)
+        adjust_context(node.op(), scope, timecontext)
         for node in op.selections
         if timecontext
     ]
@@ -214,15 +222,6 @@ def compile_column(t, expr, scope, timecontext, **kwargs):
     return table[op.name]
 
 
-@compiles(ops.DistinctColumn)
-def compile_distinct(t, expr, scope, timecontext, **kwargs):
-    op = expr.op()
-    root_table_expr = op.root_tables()[0].to_expr()
-    src_table = t.translate(root_table_expr, scope, timecontext)
-    src_column_name = op.arg.get_name()
-    return src_table.select(src_column_name).distinct()[src_column_name]
-
-
 @compiles(ops.SelfReference)
 def compile_self_reference(t, expr, scope, timecontext, **kwargs):
     op = expr.op()
@@ -369,12 +368,8 @@ def compile_literal(t, expr, scope, timecontext, raw=False, **kwargs):
             return set(value)
         else:
             return value
-    elif isinstance(value, list):
-        return F.array(*(F.lit(v) for v in value))
-    elif isinstance(value, np.ndarray):
-        # Unpack np.generic's using .item(), otherwise Spark
-        # will not accept
-        return F.array(*(F.lit(v.item()) for v in value))
+    elif isinstance(value, tuple):
+        return F.array(*map(F.lit, value))
     else:
         return F.lit(value)
 
@@ -448,6 +443,10 @@ def compile_endswith(t, expr, scope, timecontext, **kwargs):
     return col.startswith(end)
 
 
+def _is_table(table):
+    return isinstance(table.op().arg, ir.TableExpr)
+
+
 def compile_aggregator(
     t, expr, scope, timecontext, *, fn, context=None, **kwargs
 ):
@@ -467,6 +466,8 @@ def compile_aggregator(
         # Here we get the root table df of that column and compile
         # the expr to:
         # df.select(max(some_col))
+        if _is_table(expr):
+            return src_col.select(col)
         return t.translate(
             expr.op().arg.op().table, scope, timecontext
         ).select(col)
@@ -548,10 +549,18 @@ def compile_notall(t, expr, scope, timecontext, *, context=None, **kwargs):
         )
 
 
+def _count_star(_):
+    return F.count(F.lit(1))
+
+
 @compiles(ops.Count)
 def compile_count(t, expr, scope, timecontext, context=None, **kwargs):
+    if _is_table(expr):
+        fn = _count_star
+    else:
+        fn = F.count
     return compile_aggregator(
-        t, expr, scope, timecontext, fn=F.count, context=context, **kwargs
+        t, expr, scope, timecontext, fn=fn, context=context, **kwargs
     )
 
 
@@ -635,6 +644,17 @@ def compile_arbitrary(t, expr, scope, timecontext, context=None, **kwargs):
     )
 
 
+@compiles(ops.Coalesce)
+def compile_coalesce(t, expr, scope, timecontext, **kwargs):
+    op = expr.op()
+
+    src_columns = t.translate(op.arg, scope, timecontext)
+    if len(src_columns) == 1:
+        return src_columns[0]
+    else:
+        return F.coalesce(*src_columns)
+
+
 @compiles(ops.Greatest)
 def compile_greatest(t, expr, scope, timecontext, **kwargs):
     op = expr.op()
@@ -763,10 +783,13 @@ def compile_log(t, expr, scope, timecontext, **kwargs):
     op = expr.op()
 
     src_column = t.translate(op.arg, scope, timecontext)
-    # Spark log method only takes float
-    return F.log(
-        float(t.translate(op.base, scope, timecontext, raw=True)), src_column
-    )
+    raw_base = t.translate(op.base, scope, timecontext, raw=True)
+    try:
+        base = float(raw_base)
+    except TypeError:
+        return F.log(src_column) / F.log(raw_base)
+    else:
+        return F.log(base, src_column)
 
 
 @compiles(ops.Ln)
@@ -1132,6 +1155,16 @@ def compile_outer_join(t, expr, scope, timecontext, **kwargs):
     return compile_join(t, expr, scope, timecontext, how='outer')
 
 
+@compiles(ops.LeftSemiJoin)
+def compile_left_semi_join(t, expr, scope, timecontext, **kwargs):
+    return compile_join(t, expr, scope, timecontext, how='leftsemi')
+
+
+@compiles(ops.LeftAntiJoin)
+def compile_left_anti_join(t, expr, scope, timecontext, **kwargs):
+    return compile_join(t, expr, scope, timecontext, how='leftanti')
+
+
 def compile_join(t, expr, scope, timecontext, *, how):
     op = expr.op()
 
@@ -1152,6 +1185,12 @@ def compile_join(t, expr, scope, timecontext, *, how):
     return left_df.join(right_df, pred_columns, how)
 
 
+@compiles(ops.Distinct)
+def compile_distinct(t, expr, scope, timecontext):
+    op = expr.op()
+    return t.translate(op.table, scope, timecontext).distinct()
+
+
 def _canonicalize_interval(t, interval, scope, timecontext, **kwargs):
     """Convert interval to integer timestamp of second
 
@@ -1213,8 +1252,10 @@ def compile_window_op(t, expr, scope, timecontext, **kwargs):
                 t, window.following, scope, timecontext, **kwargs
             )
 
-        if isinstance(window.preceding, ir.IntervalScalar) or isinstance(
-            window.following, ir.IntervalScalar
+        if (
+            isinstance(window.preceding, ir.IntervalScalar)
+            or isinstance(window.following, ir.IntervalScalar)
+            or window.how == "range"
         ):
             pyspark_window = pyspark_window.rangeBetween(start, end)
         else:
@@ -1278,11 +1319,7 @@ def compile_dense_rank(t, expr, scope, timecontext, **kwargs):
 
 @compiles(ops.PercentRank)
 def compile_percent_rank(t, expr, scope, timecontext, **kwargs):
-    raise com.UnsupportedOperationError(
-        'Pyspark percent_rank() function indexes from 0 '
-        'instead of 1, and does not match expected '
-        'output of ibis expressions.'
-    )
+    return F.percent_rank()
 
 
 @compiles(ops.NTile)
@@ -1338,7 +1375,7 @@ def _extract_component_from_datetime(
 ):
     op = expr.op()
     date_col = t.translate(op.arg, scope, timecontext)
-    return extract_fn(date_col)
+    return extract_fn(date_col).cast('integer')
 
 
 @compiles(ops.ExtractYear)
@@ -1784,10 +1821,11 @@ def compile_dropna_table(t, expr, scope, timecontext, **kwargs):
 def compile_fillna_table(t, expr, scope, timecontext, **kwargs):
     op = expr.op()
     table = t.translate(op.table, scope, timecontext)
+    raw_replacements = op.replacements
     replacements = (
-        op.replacements.op().value
-        if hasattr(op.replacements, 'op')
-        else op.replacements
+        dict(raw_replacements)
+        if isinstance(raw_replacements, frozendict)
+        else raw_replacements.op().value
     )
     return table.fillna(replacements)
 
@@ -1843,3 +1881,28 @@ def compile_searched_case(t, expr, scope, timecontext, **kwargs):
     return existing_when.otherwise(
         t.translate(op.default, scope, timecontext, **kwargs)
     )
+
+
+@compiles(ops.View)
+def compile_view(t, expr, scope, timecontext, **kwargs):
+    op = expr.op()
+    name = op.name
+    child = op.child
+    backend = child._find_backend()
+    tables = backend._session.catalog.listTables()
+    if any(name == table.name and not table.isTemporary for table in tables):
+        raise ValueError(
+            f"table or non-temporary view `{name}` already exists"
+        )
+    result = t.translate(child, scope, timecontext, **kwargs)
+    result.createOrReplaceTempView(name)
+    return result
+
+
+@compiles(ops.SQLStringView)
+def compile_sql_view(t, expr, scope, timecontext, **kwargs):
+    op = expr.op()
+    backend = op.child._find_backend()
+    result = backend._session.sql(op.query)
+    result.createOrReplaceTempView(op.name)
+    return result
