@@ -3,22 +3,41 @@ from datetime import datetime, timezone
 
 import numpy as np
 import pandas as pd
-import pyspark.sql.functions as F
-import pyspark.sql.types as pt
 import pytest
-from pyspark.sql import SparkSession
 
 import ibis
 from ibis import util
 from ibis.backends.tests.base import BackendTest, RoundAwayFromZero
 
-_pyspark_testing_client = None
+pytest.importorskip("pyspark")
+
+import pyspark.sql.functions as F  # noqa: E402
+import pyspark.sql.types as pt  # noqa: E402
+from pyspark.sql import SparkSession  # noqa: E402
 
 
 def get_common_spark_testing_client(data_directory, connect):
     spark = (
-        SparkSession.builder.config('spark.default.parallelism', 4)
-        .config('spark.driver.bindAddress', '127.0.0.1')
+        SparkSession.builder.appName("ibis_testing")
+        .master("local[1]")
+        .config("spark.cores.max", 1)
+        .config("spark.executor.heartbeatInterval", "3600s")
+        .config("spark.executor.instances", 1)
+        .config("spark.network.timeout", "4200s")
+        .config("spark.sql.execution.arrow.pyspark.enabled", False)
+        .config("spark.sql.legacy.timeParserPolicy", "LEGACY")
+        .config("spark.storage.blockManagerSlaveTimeoutMs", "4200s")
+        .config("spark.ui.showConsoleProgress", False)
+        .config('spark.default.parallelism', 1)
+        .config('spark.dynamicAllocation.enabled', False)
+        .config('spark.rdd.compress', False)
+        .config(
+            'spark.serializer', 'org.apache.spark.serializer.KryoSerializer'
+        )
+        .config('spark.shuffle.compress', False)
+        .config('spark.shuffle.spill.compress', False)
+        .config('spark.sql.shuffle.partitions', 1)
+        .config('spark.ui.enabled', False)
         .getOrCreate()
     )
     _spark_testing_client = connect(spark)
@@ -94,7 +113,7 @@ def get_common_spark_testing_client(data_directory, connect):
         .repartition(num_partitions)
         .sort('playerID')
     )
-    df_batting.createOrReplaceTempView('batting')
+    df_batting.createOrReplaceTempView("batting")
 
     df_awards_players = (
         s.read.csv(
@@ -131,6 +150,24 @@ def get_common_spark_testing_client(data_directory, connect):
         ],
     )
     df_nested_types.createOrReplaceTempView('nested_types')
+    df_array_types = s.createDataFrame(
+        [
+            ([1, 2, 3], ['a', 'b', 'c'], [1.0, 2.0, 3.0], 'a', 1.0),
+            ([4, 5], ['d', 'e'], [4.0, 5.0], 'a', 2.0),
+            ([6, None], ['f', None], [6.0, None], 'a', 3.0),
+            ([None, 1, None], [None, 'a', None], [], 'b', 4.0),
+            ([2, None, 3], ['b', None, 'c'], None, 'b', 5.0),
+            (
+                [4, None, None, 5],
+                ['d', None, None, 'e'],
+                [4.0, None, None, 5.0],
+                'c',
+                6.0,
+            ),
+        ],
+        ["x", "y", "z", "grouper", "scalar_column"],
+    )
+    df_array_types.createOrReplaceTempView("array_types")
 
     df_complicated = s.createDataFrame(
         [({(1, 3): [[2, 4], [3, 5]]},)], ['map_tuple_list_of_list_of_ints']
@@ -180,13 +217,10 @@ def get_common_spark_testing_client(data_directory, connect):
 
 
 def get_pyspark_testing_client(data_directory):
-    global _pyspark_testing_client
-    if _pyspark_testing_client is None:
-        _pyspark_testing_client = get_common_spark_testing_client(
-            data_directory,
-            lambda session: ibis.backends.pyspark.Backend().connect(session),
-        )
-    return _pyspark_testing_client
+    return get_common_spark_testing_client(
+        data_directory,
+        lambda session: ibis.backends.pyspark.Backend().connect(session),
+    )
 
 
 class TestConf(BackendTest, RoundAwayFromZero):
@@ -280,9 +314,9 @@ def _random_identifier(suffix):
 
 @pytest.fixture(scope='session', autouse=True)
 def test_data_db(client):
+    name = os.environ.get('IBIS_TEST_DATA_DB', 'ibis_testing')
+    client.create_database(name)
     try:
-        name = os.environ.get('IBIS_TEST_DATA_DB', 'ibis_testing')
-        client.create_database(name)
         client.set_database(name)
         yield name
     finally:
