@@ -1,4 +1,3 @@
-import os
 import uuid
 
 import numpy as np
@@ -8,22 +7,8 @@ import pytest
 import ibis
 import ibis.config as config
 import ibis.expr.types as ir
-from ibis.backends.sqlite import Backend
-from ibis.util import guid
 
-
-def test_file_not_exist_and_create():
-    path = f'__ibis_tmp_{guid()}.db'
-
-    with pytest.raises(FileNotFoundError):
-        ibis.sqlite.connect(path)
-
-    con = ibis.sqlite.connect(path, create=True)
-    try:
-        assert os.path.exists(path)
-    finally:
-        con.con.dispose()
-        os.remove(path)
+pytest.importorskip("sqlalchemy")
 
 
 def test_table(con):
@@ -57,7 +42,7 @@ def test_list_tables(con):
 
 
 def test_attach_file(dbpath):
-    client = Backend().connect(None)
+    client = ibis.sqlite.connect(None)
 
     client.attach('foo', dbpath)
     client.attach('bar', dbpath)
@@ -68,10 +53,6 @@ def test_attach_file(dbpath):
     assert foo_tables == bar_tables
 
 
-def test_database_layer(con, db):
-    assert db.list_tables() == con.list_tables()
-
-
 def test_compile_toplevel():
     t = ibis.table([('foo', 'double')], name='t0')
 
@@ -80,7 +61,7 @@ def test_compile_toplevel():
     result = ibis.sqlite.compile(expr)
     expected = """\
 SELECT sum(t0.foo) AS sum 
-FROM t0 AS t0"""  # noqa
+FROM t0 AS t0"""  # noqa: W291
     assert str(result) == expected
 
 
@@ -104,6 +85,27 @@ def test_verbose_log_queries(con):
     assert len(queries) == 1
     (query,) = queries
     expected = 'SELECT t0.year \n'
-    expected += 'FROM base.functional_alltypes AS t0\n'
+    expected += 'FROM main.functional_alltypes AS t0\n'
     expected += ' LIMIT ? OFFSET ?'
     assert query == expected
+
+
+def test_table_equality(dbpath):
+    con1 = ibis.sqlite.connect(dbpath)
+    batting1 = con1.table("batting")
+
+    con2 = ibis.sqlite.connect(dbpath)
+    batting2 = con2.table("batting")
+
+    assert batting1.op() == batting2.op()
+    assert batting1.equals(batting2)
+
+
+def test_table_inequality(dbpath):
+    con = ibis.sqlite.connect(dbpath)
+
+    batting = con.table("batting")
+    functional_alltypes = con.table("functional_alltypes")
+
+    assert batting.op() != functional_alltypes.op()
+    assert not batting.equals(functional_alltypes)
