@@ -1,6 +1,5 @@
 import datetime
 import math
-import operator
 import sqlite3
 import uuid
 
@@ -8,7 +7,6 @@ import numpy as np
 import pandas as pd
 import pandas.testing as tm
 import pytest
-import sqlalchemy as sa
 from packaging.version import parse
 
 import ibis
@@ -16,6 +14,8 @@ import ibis.expr.datatypes as dt
 from ibis import config
 from ibis import literal as L
 
+sa = pytest.importorskip("sqlalchemy")
+
 
 @pytest.mark.parametrize(
     ('func', 'expected'),
@@ -25,11 +25,11 @@ from ibis import literal as L
             lambda at: sa.cast(at.c.double_col, sa.SMALLINT),
         ),
         (
-            lambda t: t.string_col.cast(dt.double),
+            lambda t: t.string_col.cast(dt.float64),
             lambda at: sa.cast(at.c.string_col, sa.REAL),
         ),
         (
-            lambda t: t.string_col.cast(dt.float),
+            lambda t: t.string_col.cast(dt.float32),
             lambda at: sa.cast(at.c.string_col, sa.REAL),
         ),
     ],
@@ -38,11 +38,6 @@ def test_cast(alltypes, alltypes_sqla, translate, func, expected):
     assert translate(func(alltypes)) == str(expected(alltypes_sqla))
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYI')
-def test_decimal_cast():
-    assert False
-
-
 @pytest.mark.parametrize(
     ('func', 'expected_func'),
     [
@@ -320,11 +315,6 @@ def test_fillna_nullif(con, expr, expected):
     assert con.execute(expr) == expected
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYI')
-def test_coalesce():
-    assert False
-
-
 def test_numeric_builtins_work(alltypes, df):
     expr = alltypes.double_col.fillna(0)
     result = expr.execute()
@@ -613,15 +603,44 @@ def test_column_access_after_sort(alltypes, df, column):
     tm.assert_series_equal(result, expected)
 
 
-def test_materialized_join(con):
-    con.raw_sql('CREATE TABLE mj1 (id1 INTEGER, val1 REAL)')
-    con.raw_sql('INSERT INTO mj1 VALUES (1, 10), (2, 20)')
-    con.raw_sql('CREATE TABLE mj2 (id2 INTEGER, val2 REAL)')
-    con.raw_sql('INSERT INTO mj2 VALUES (1, 15), (2, 25)')
+@pytest.fixture
+def mj1(con):
+    con.create_table(
+        "mj1",
+        schema=ibis.schema(dict(id1="int32", val1="float64")),
+        force=True,
+    )
+    try:
+        con.insert(
+            "mj1",
+            pd.DataFrame(dict(id1=[1, 2], val1=[10, 20])),
+            overwrite=True,
+        )
+        yield con.table("mj1")
+    finally:
+        con.drop_table("mj1", force=True)
 
-    t1 = con.table('mj1', database="main")
-    t2 = con.table('mj2', database="main")
-    joined = t1.join(t2, t1.id1 == t2.id2).materialize()
+
+@pytest.fixture
+def mj2(con):
+    con.create_table(
+        "mj2",
+        schema=ibis.schema(dict(id2="int32", val2="float64")),
+        force=True,
+    )
+    try:
+        con.insert(
+            "mj2",
+            pd.DataFrame(dict(id2=[1, 2], val2=[15, 25])),
+            overwrite=True,
+        )
+        yield con.table("mj2")
+    finally:
+        con.drop_table("mj2", force=True)
+
+
+def test_simple_join(con, mj1, mj2):
+    joined = mj1.join(mj2, mj1.id1 == mj2.id2)
     result = joined.val2.execute()
     assert len(result) == 2
 
@@ -652,7 +671,10 @@ def test_identical_to(alltypes):
     tm.assert_series_equal(result, expected)
 
 
-@pytest.mark.xfail(raises=AttributeError, reason='NYI')
+@pytest.mark.xfail(
+    raises=AttributeError,
+    reason="truncate method is not yet implemented",
+)
 def test_truncate_method(con, alltypes):
     expr = alltypes.limit(5)
     name = str(uuid.uuid4())
@@ -719,25 +741,10 @@ def test_compile_with_one_unnamed_table():
     assert str(result) == str(expected)
 
 
-@pytest.mark.parametrize(
-    ('attr', 'expected'),
-    [
-        (operator.methodcaller('year'), {2009, 2010}),
-        (operator.methodcaller('month'), set(range(1, 13))),
-        (operator.methodcaller('day'), set(range(1, 32))),
-    ],
-)
-def test_date_extract_field(alltypes, attr, expected):
-    t = alltypes
-    expr = attr(t.timestamp_col.cast('date')).distinct()
-    result = expr.execute().astype(int)
-    assert set(result) == expected
-
-
-def test_scalar_parameter(db):
+def test_scalar_parameter(alltypes):
     start = ibis.param(dt.date)
     end = ibis.param(dt.date)
-    t = db.functional_alltypes
+    t = alltypes
     col = t.date_string_col.cast('date')
     expr = col.between(start, end)
     start_string, end_string = '2009-03-01', '2010-07-03'
@@ -765,8 +772,8 @@ def test_compile_twice(dbpath):
     assert result1 == result2
 
 
-def test_count_on_order_by(db):
-    t = db.batting
+def test_count_on_order_by(con):
+    t = con.table("batting")
     sort_key = ibis.desc(t.playerID)
     sorted_table = t.sort_by(sort_key)
     expr = sorted_table.count()
@@ -774,6 +781,6 @@ def test_count_on_order_by(db):
         expr.compile().compile(compile_kwargs={'literal_binds': True})
     )
     expected = (
-        'SELECT count(\'*\') AS count \n' 'FROM base.batting AS t0'
-    )  # noqa: W291
+        "SELECT count('*') AS count \nFROM main.batting AS t0"  # noqa: W291
+    )
     assert result == expected
