@@ -1,7 +1,7 @@
 import numpy as np
 import pandas as pd
 import pytest
-from pytest import param
+from pytest import mark, param
 
 import ibis.expr.datatypes as dt
 from ibis.udf.vectorized import reduction
@@ -24,7 +24,12 @@ aggregate_test_params = [
         lambda s: s.mean(),
         'double_col',
         id='mean_udf',
-        marks=pytest.mark.udf,
+        marks=[
+            pytest.mark.notimpl(
+                ["datafusion", "postgres", "clickhouse", "impala", "duckdb"]
+            ),
+            pytest.mark.never(["sqlite", "mysql"], reason="no udf support"),
+        ],
     ),
     param(
         lambda t: t.double_col.min(),
@@ -57,7 +62,6 @@ aggregate_test_params = [
     ('result_fn', 'expected_fn', 'expected_col'),
     aggregate_test_params,
 )
-@pytest.mark.xfail_unsupported
 def test_aggregate(
     backend, alltypes, df, result_fn, expected_fn, expected_col
 ):
@@ -75,7 +79,6 @@ def test_aggregate(
     ('result_fn', 'expected_fn', 'expected_col'),
     aggregate_test_params,
 )
-@pytest.mark.xfail_unsupported
 def test_aggregate_grouped(
     backend, alltypes, df, result_fn, expected_fn, expected_col
 ):
@@ -165,12 +168,6 @@ def test_aggregate_grouped(
             lambda t, where: t.double_col.max(),
             id='max',
         ),
-        param(
-            lambda t, where: t.double_col.approx_median(),
-            lambda t, where: t.double_col.median(),
-            id='approx_median',
-            marks=pytest.mark.xpass_backends(['clickhouse']),
-        ),
         param(
             lambda t, where: t.double_col.std(how='sample'),
             lambda t, where: t.double_col.std(ddof=1),
@@ -195,27 +192,63 @@ def test_aggregate_grouped(
             lambda t, where: t.double_col.cov(t.float_col),
             lambda t, where: t.double_col.cov(t.float_col),
             id='covar',
+            marks=[
+                pytest.mark.notimpl(
+                    [
+                        "clickhouse",
+                        "dask",
+                        "duckdb",
+                        "impala",
+                        "mysql",
+                        "pandas",
+                        "postgres",
+                        "pyspark",
+                        "sqlite",
+                    ]
+                )
+            ],
         ),
         param(
             lambda t, where: t.double_col.corr(t.float_col),
             lambda t, where: t.double_col.corr(t.float_col),
             id='corr',
+            marks=[
+                pytest.mark.notimpl(
+                    [
+                        "clickhouse",
+                        "dask",
+                        "duckdb",
+                        "impala",
+                        "mysql",
+                        "pandas",
+                        "postgres",
+                        "pyspark",
+                        "sqlite",
+                    ]
+                )
+            ],
         ),
         param(
             lambda t, where: t.string_col.approx_nunique(),
             lambda t, where: t.string_col.nunique(),
             id='approx_nunique',
-            marks=pytest.mark.xfail_backends(['mysql', 'sqlite']),
+            marks=pytest.mark.notyet(['mysql', 'sqlite', 'pyspark']),
         ),
         param(
             lambda t, where: t.double_col.arbitrary(how='first'),
             lambda t, where: t.double_col.iloc[0],
             id='arbitrary_first',
+            marks=pytest.mark.notimpl(
+                ['impala', 'postgres', 'mysql', 'sqlite', 'duckdb']
+            ),
         ),
         param(
             lambda t, where: t.double_col.arbitrary(how='last'),
             lambda t, where: t.double_col.iloc[-1],
             id='arbitrary_last',
+            marks=pytest.mark.notimpl(
+                ['impala', 'postgres', 'mysql', 'sqlite', 'duckdb']
+            ),
         ),
     ],
 )
@@ -230,7 +263,7 @@ def test_aggregate_grouped(
         ),
     ],
 )
-@pytest.mark.xfail_unsupported
+@mark.notimpl(["datafusion"])
 def test_reduction_ops(
     backend,
     alltypes,
@@ -247,6 +280,24 @@ def test_reduction_ops(
     np.testing.assert_allclose(result, expected)
 
 
+@pytest.mark.notimpl(
+    [
+        "dask",
+        "datafusion",
+        "duckdb",
+        "mysql",
+        "pandas",
+        "postgres",
+        "pyspark",
+        "sqlite",
+    ]
+)
+def test_approx_median(backend, alltypes, df):
+    expr = alltypes.double_col.approx_median()
+    result = expr.execute()
+    assert result is not None
+
+
 @pytest.mark.parametrize(
     ('result_fn', 'expected_fn'),
     [
@@ -266,7 +317,7 @@ def test_reduction_ops(
         )
     ],
 )
-@pytest.mark.xfail_unsupported
+@mark.notimpl(["datafusion", "duckdb"])
 def test_group_concat(backend, alltypes, df, result_fn, expected_fn):
     expr = result_fn(alltypes)
     result = expr.execute()
@@ -286,8 +337,8 @@ def test_group_concat(backend, alltypes, df, result_fn, expected_fn):
         )
     ],
 )
-@pytest.mark.xfail_unsupported
-@pytest.mark.xfail_backends(['pyspark', 'datafusion'])  # Issue #2130
+@mark.notimpl(["pandas", "dask"])
+@pytest.mark.notyet(["pyspark", "datafusion"])
 def test_topk_op(backend, alltypes, df, result_fn, expected_fn):
     # TopK expression will order rows by "count" but each backend
     # can have different result for that.
@@ -314,18 +365,16 @@ def test_topk_op(backend, alltypes, df, result_fn, expected_fn):
         )
     ],
 )
-@pytest.mark.xfail_unsupported
-# Issues #2369 #2133 #2131 #2132
-@pytest.mark.xfail_backends(['bigquery', 'clickhouse', 'mysql', 'postgres'])
-@pytest.mark.skip_backends(['sqlite'], reason='Issue #2128')
-def test_topk_filter_op(backend, alltypes, df, result_fn, expected_fn):
+@mark.notimpl(["datafusion", "pandas", "dask", "pyspark"])
+def test_topk_filter_op(alltypes, df, result_fn, expected_fn):
     # TopK expression will order rows by "count" but each backend
     # can have different result for that.
     # Note: Maybe would be good if TopK could order by "count"
     # and the field used by TopK
     t = alltypes.sort_by(alltypes.string_col)
     df = df.sort_values('string_col')
-    result = result_fn(t).execute()
+    expr = result_fn(t)
+    result = expr.execute()
     expected = expected_fn(df)
     assert result.shape[0] == expected.shape[0]
 
@@ -337,7 +386,17 @@ def test_topk_filter_op(backend, alltypes, df, result_fn, expected_fn):
         param(lambda s: np.array(s), id='agg_to_ndarray'),
     ],
 )
-@pytest.mark.xfail_unsupported
+@mark.notimpl(
+    [
+        "clickhouse",
+        "datafusion",
+        "duckdb",
+        "impala",
+        "mysql",
+        "postgres",
+        "sqlite",
+    ]
+)
 def test_aggregate_list_like(backend, alltypes, df, agg_fn):
     """Tests .aggregate() where the result of an aggregation is a list-like.
 
@@ -359,7 +418,17 @@ def test_aggregate_list_like(backend, alltypes, df, agg_fn):
     backend.assert_frame_equal(result, expected)
 
 
-@pytest.mark.xfail_unsupported
+@mark.notimpl(
+    [
+        "clickhouse",
+        "datafusion",
+        "duckdb",
+        "impala",
+        "mysql",
+        "postgres",
+        "sqlite",
+    ]
+)
 def test_aggregate_mixed(backend, alltypes, df):
     """Tests .aggregate() with multiple aggregations with mixed result types.
 
@@ -389,3 +458,20 @@ def test_aggregate_mixed(backend, alltypes, df):
     )
 
     backend.assert_frame_equal(result, expected, check_like=True)
+
+
+@pytest.mark.notimpl(["datafusion", "pyspark"])
+def test_binds_are_cast(alltypes):
+    expr = alltypes.aggregate(
+        high_line_count=(
+            alltypes.string_col.case().when('1-URGENT', 1).else_(0).end().sum()
+        )
+    )
+
+    expr.execute()
+
+
+def test_agg_sort(alltypes):
+    query = alltypes.aggregate(count=alltypes.count())
+    query = query.sort_by(alltypes.year)
+    query.execute()
