@@ -20,13 +20,11 @@ def test_exists_table(con, table_name):
     assert actual == expected
 
 
-# `list_databases` is underspecified for the HDF5 backend, so it's skipped
-# until that can be specified more clearly.
 # 1. `current_database` returns '.', but isn't listed in list_databases()
 # 2. list_databases() returns directories which don't make sense as HDF5
 #    databases
-@pytest.mark.xfail_backends(['hdf5'])
-@pytest.mark.xfail_unsupported
+@pytest.mark.never(["dask", "pandas"], reason="pass")
+@pytest.mark.notimpl(["datafusion", "duckdb"])
 def test_database_consistency(con):
     # every backend has a different set of databases, not testing the
     # exact names for now
@@ -40,8 +38,6 @@ def test_database_consistency(con):
     assert current_database in databases
 
 
-# HDF5 requires a path argument
-@pytest.mark.xfail_backends(['hdf5'])
 def test_list_tables(con):
     tables = con.list_tables()
     assert isinstance(tables, list)
