@@ -6,6 +6,7 @@ import numpy as np
 import pandas as pd
 import pandas.testing as tm
 import pytest
+from packaging.version import parse as vparse
 from pytest import param
 
 import ibis
@@ -21,29 +22,10 @@ else:
     from sqlalchemy.dialects import mysql, postgresql
 
 
-@pytest.fixture
-def df_decimal():
-    return pd.DataFrame(
-        {
-            'n1': [
-                decimal.Decimal('12.1234'),
-                decimal.Decimal('333.878787'),
-                decimal.Decimal('1090.4949'),
-            ],
-            'n2': [
-                decimal.Decimal('5454.0904'),
-                decimal.Decimal('904.889282'),
-                decimal.Decimal('9893.09022'),
-            ],
-        }
-    )
-
-
-@pytest.fixture
-def sch_decimal():
-    return ibis.schema(
-        [('index', 'int64'), ('n1', 'decimal'), ('n2', 'decimal')]
-    )
+try:
+    import duckdb
+except ImportError:
+    duckdb = None
 
 
 @pytest.mark.parametrize(
@@ -58,19 +40,16 @@ def sch_decimal():
             lambda t: ibis.literal(np.nan),
             lambda t: np.nan,
             id='nan-literal',
-            marks=pytest.mark.xfail_backends(['omniscidb']),
         ),
         param(
             lambda t: ibis.literal(np.inf),
             lambda t: np.inf,
             id='inf-literal',
-            marks=pytest.mark.xfail_backends(['omniscidb']),
         ),
         param(
             lambda t: ibis.literal(-np.inf),
             lambda t: -np.inf,
             id='-inf-literal',
-            marks=pytest.mark.xfail_backends(['omniscidb']),
         ),
     ],
 )
@@ -81,7 +60,11 @@ def sch_decimal():
         param(operator.methodcaller('isinf'), np.isinf, id='isinf'),
     ],
 )
-@pytest.mark.xfail_unsupported
+@pytest.mark.notimpl(["mysql", "sqlite", "datafusion"])
+@pytest.mark.xfail(
+    duckdb is not None and vparse(duckdb.__version__) < vparse("0.3.3"),
+    reason="<0.3.3 does not support isnan/isinf properly",
+)
 def test_isnan_isinf(
     backend,
     con,
@@ -112,27 +95,102 @@ def test_isnan_isinf(
     [
         param(L(-5).abs(), 5, id='abs-neg'),
         param(L(5).abs(), 5, id='abs'),
-        param(ibis.least(L(10), L(1)), 1, id='least'),
-        param(ibis.greatest(L(10), L(1)), 10, id='greatest'),
+        param(
+            ibis.least(L(10), L(1)),
+            1,
+            id='least',
+            marks=pytest.mark.notimpl(["datafusion"]),
+        ),
+        param(
+            ibis.greatest(L(10), L(1)),
+            10,
+            id='greatest',
+            marks=pytest.mark.notimpl(["datafusion"]),
+        ),
         param(L(5.5).round(), 6.0, id='round'),
-        param(L(5.556).round(2), 5.56, id='round-digits'),
+        param(
+            L(5.556).round(2),
+            5.56,
+            id='round-digits',
+            marks=pytest.mark.notimpl(["datafusion"]),
+        ),
         param(L(5.556).ceil(), 6.0, id='ceil'),
         param(L(5.556).floor(), 5.0, id='floor'),
-        param(L(5.556).exp(), math.exp(5.556), id='expr'),
-        param(L(5.556).sign(), 1, id='sign-pos'),
-        param(L(-5.556).sign(), -1, id='sign-neg'),
-        param(L(0).sign(), 0, id='sign-zero'),
+        param(
+            L(5.556).exp(),
+            math.exp(5.556),
+            id='expr',
+            marks=pytest.mark.notimpl(["datafusion"]),
+        ),
+        param(
+            L(5.556).sign(),
+            1,
+            id='sign-pos',
+            marks=pytest.mark.notimpl(["datafusion"]),
+        ),
+        param(
+            L(-5.556).sign(),
+            -1,
+            id='sign-neg',
+            marks=pytest.mark.notimpl(["datafusion"]),
+        ),
+        param(
+            L(0).sign(),
+            0,
+            id='sign-zero',
+            marks=pytest.mark.notimpl(["datafusion"]),
+        ),
         param(L(5.556).sqrt(), math.sqrt(5.556), id='sqrt'),
-        param(L(5.556).log(2), math.log(5.556, 2), id='log-base'),
+        param(
+            L(5.556).log(2),
+            math.log(5.556, 2),
+            id='log-base',
+            marks=pytest.mark.notimpl(["datafusion"]),
+        ),
         param(L(5.556).ln(), math.log(5.556), id='ln'),
         param(L(5.556).log2(), math.log(5.556, 2), id='log2'),
         param(L(5.556).log10(), math.log10(5.556), id='log10'),
-        param(L(5.556).radians(), math.radians(5.556), id='radians'),
-        param(L(5.556).degrees(), math.degrees(5.556), id='degrees'),
+        param(
+            L(5.556).radians(),
+            math.radians(5.556),
+            id='radians',
+            marks=pytest.mark.notimpl(
+                [
+                    "clickhouse",
+                    "dask",
+                    "datafusion",
+                    "duckdb",
+                    "impala",
+                    "mysql",
+                    "pandas",
+                    "postgres",
+                    "pyspark",
+                    "sqlite",
+                ]
+            ),
+        ),
+        param(
+            L(5.556).degrees(),
+            math.degrees(5.556),
+            id='degrees',
+            marks=pytest.mark.notimpl(
+                [
+                    "clickhouse",
+                    "dask",
+                    "datafusion",
+                    "duckdb",
+                    "impala",
+                    "mysql",
+                    "pandas",
+                    "postgres",
+                    "pyspark",
+                    "sqlite",
+                ]
+            ),
+        ),
         param(L(11) % 3, 11 % 3, id='mod'),
     ],
 )
-@pytest.mark.xfail_unsupported
 def test_math_functions_literals(backend, con, alltypes, df, expr, expected):
     result = con.execute(expr)
     if isinstance(result, decimal.Decimal):
@@ -151,6 +209,7 @@ def test_math_functions_literals(backend, con, alltypes, df, expr, expected):
             lambda t: (-t.double_col).abs(),
             lambda t: (-t.double_col).abs(),
             id='abs-neg',
+            marks=pytest.mark.notimpl(["datafusion"]),
         ),
         param(
             lambda t: t.double_col.abs(),
@@ -171,15 +230,16 @@ def test_math_functions_literals(backend, con, alltypes, df, expr, expected):
             lambda t: t.double_col.sign(),
             lambda t: np.sign(t.double_col),
             id='sign',
+            marks=pytest.mark.notimpl(["datafusion"]),
         ),
         param(
             lambda t: (-t.double_col).sign(),
             lambda t: np.sign(-t.double_col),
             id='sign-negative',
+            marks=pytest.mark.notimpl(["datafusion"]),
         ),
     ],
 )
-@pytest.mark.xfail_unsupported
 def test_simple_math_functions_columns(
     backend, con, alltypes, df, expr_fn, expected_fn
 ):
@@ -206,11 +266,13 @@ def test_simple_math_functions_columns(
             lambda t: t.double_col.add(1).exp(),
             lambda t: np.exp(t.double_col + 1),
             id='exp',
+            marks=pytest.mark.notimpl(["datafusion"]),
         ),
         param(
             lambda t: t.double_col.add(1).log(2),
             lambda t: np.log2(t.double_col + 1),
             id='log2',
+            marks=pytest.mark.notimpl(["datafusion"]),
         ),
         param(
             lambda t: t.double_col.add(1).ln(),
@@ -222,9 +284,22 @@ def test_simple_math_functions_columns(
             lambda t: np.log10(t.double_col + 1),
             id='log10',
         ),
+        param(
+            lambda t: (t.double_col + 1).log(
+                ibis.greatest(
+                    9_000,
+                    t.bigint_col,
+                )
+            ),
+            lambda t: (
+                np.log(t.double_col + 1)
+                / np.log(np.maximum(9_000, t.bigint_col))
+            ),
+            id="log_base_bigint",
+            marks=pytest.mark.notimpl(["clickhouse", "datafusion"]),
+        ),
     ],
 )
-@pytest.mark.xfail_unsupported
 def test_complex_math_functions_columns(
     backend, con, alltypes, df, expr_fn, expected_fn
 ):
@@ -246,11 +321,13 @@ def test_complex_math_functions_columns(
             lambda be, t: t.double_col.add(0.05).round(3),
             lambda be, t: be.round(t.double_col + 0.05, 3),
             id='round-with-param',
+            marks=pytest.mark.notimpl(["datafusion"]),
         ),
         param(
             lambda be, t: be.least(ibis.least, t.bigint_col, t.int_col),
             lambda be, t: pd.Series(list(map(min, t.bigint_col, t.int_col))),
             id='least-all-columns',
+            marks=pytest.mark.notimpl(["datafusion"]),
         ),
         param(
             lambda be, t: be.least(ibis.least, t.bigint_col, t.int_col, -2),
@@ -258,11 +335,13 @@ def test_complex_math_functions_columns(
                 list(map(min, t.bigint_col, t.int_col, [-2] * len(t)))
             ),
             id='least-scalar',
+            marks=pytest.mark.notimpl(["datafusion", "clickhouse"]),
         ),
         param(
             lambda be, t: be.greatest(ibis.greatest, t.bigint_col, t.int_col),
             lambda be, t: pd.Series(list(map(max, t.bigint_col, t.int_col))),
             id='greatest-all-columns',
+            marks=pytest.mark.notimpl(["datafusion"]),
         ),
         param(
             lambda be, t: be.greatest(
@@ -272,10 +351,10 @@ def test_complex_math_functions_columns(
                 list(map(max, t.bigint_col, t.int_col, [-2] * len(t)))
             ),
             id='greatest-scalar',
+            marks=pytest.mark.notimpl(["datafusion", "clickhouse"]),
         ),
     ],
 )
-@pytest.mark.xfail_unsupported
 def test_backend_specific_numerics(
     backend, con, df, alltypes, expr_fn, expected_fn
 ):
@@ -285,6 +364,7 @@ def test_backend_specific_numerics(
     backend.assert_series_equal(result, expected)
 
 
+# marks=pytest.mark.notimpl(["datafusion"]),
 @pytest.mark.parametrize(
     'op',
     [
@@ -293,11 +373,13 @@ def test_backend_specific_numerics(
         operator.mul,
         operator.truediv,
         operator.floordiv,
-        operator.pow,
+        param(
+            operator.pow,
+            marks=pytest.mark.notimpl(["datafusion"]),
+        ),
     ],
     ids=lambda op: op.__name__,
 )
-@pytest.mark.xfail_unsupported
 def test_binary_arithmetic_operations(backend, alltypes, df, op):
     smallint_col = alltypes.smallint_col + 1  # make it nonzero
     smallint_series = df.smallint_col + 1
@@ -326,8 +408,6 @@ def test_mod(backend, alltypes, df):
 
 
 def test_floating_mod(backend, alltypes, df):
-    if not backend.supports_floating_modulus:
-        pytest.skip(f'{backend} does not support floating modulus operation')
     expr = operator.mod(alltypes.double_col, alltypes.smallint_col + 1)
 
     result = expr.execute()
@@ -348,30 +428,32 @@ def test_floating_mod(backend, alltypes, df):
         'double_col',
     ],
 )
+@pytest.mark.notyet(
+    ["datafusion", "duckdb", "mysql", "postgres", "pyspark", "sqlite"]
+)
 @pytest.mark.parametrize('denominator', [0, 0.0])
 def test_divide_by_zero(backend, alltypes, df, column, denominator):
-    if not backend.supports_divide_by_zero:
-        pytest.skip(f'{backend} does not support safe division by zero')
     expr = alltypes[column] / denominator
     expected = backend.default_series_rename(df[column].div(denominator))
     result = expr.execute()
     backend.assert_series_equal(result, expected)
 
 
-@pytest.mark.skipif(
-    sa is None, reason="Tests available for sqlalchemy based backend"
-)
 @pytest.mark.parametrize(
     'dialects, default_precisions, default_scales',
     [
         (
             {'postgres': postgresql, 'mysql': mysql},
-            {'postgres': 1000, 'mysql': 10},
-            {'postgres': 0, 'mysql': 0},
+            {'postgres': None, 'mysql': 10},
+            {'postgres': None, 'mysql': 0},
         )
     ],
 )
-@pytest.mark.only_on_backends(['postgres', 'mysql'])
+@pytest.mark.notimpl(["sqlite", "duckdb"])
+@pytest.mark.never(
+    ["clickhouse", "dask", "datafusion", "impala", "pandas", "pyspark"],
+    reason="Not SQLAlchemy backends",
+)
 def test_sa_default_numeric_precision_and_scale(
     con, backend, dialects, default_precisions, default_scales
 ):
@@ -410,7 +492,17 @@ def test_sa_default_numeric_precision_and_scale(
     con.drop_table(table_name, force=True)
 
 
-@pytest.mark.only_on_backends(['postgres', 'mysql'])
+@pytest.mark.notimpl(
+    [
+        "clickhouse",
+        "dask",
+        "datafusion",
+        "impala",
+        "pandas",
+        "pyspark",
+        "sqlite",
+    ]
+)
 def test_random(con):
     expr = ibis.random()
     result = con.execute(expr)
@@ -424,8 +516,6 @@ def test_random(con):
     assert result != result2
 
 
-@pytest.mark.only_on_backends(['pandas', 'dask', 'pyspark'])
-@pytest.mark.xfail_unsupported
 @pytest.mark.parametrize(
     ('ibis_func', 'pandas_func'),
     [
@@ -446,9 +536,34 @@ def test_random(con):
         ),
     ],
 )
+@pytest.mark.notimpl(
+    [
+        "clickhouse",
+        "duckdb",
+        "datafusion",
+        "impala",
+        "mysql",
+        "postgres",
+        "sqlite",
+    ]
+)
 def test_clip(alltypes, df, ibis_func, pandas_func):
     result = ibis_func(alltypes.int_col).execute()
     expected = pandas_func(df.int_col).astype(result.dtype)
     # Names won't match in the Pyspark backend since Pyspark
     # gives 'tmp' name when executing a ColumnExpr
     tm.assert_series_equal(result, expected, check_names=False)
+
+
+@pytest.mark.notimpl(
+    [
+        "dask",
+        "datafusion",
+        "pandas",
+        "pyspark",
+    ]
+)
+def test_histogram(con, alltypes):
+    n = 10
+    results = con.execute(alltypes.int_col.histogram(n))
+    assert len(results.value_counts()) == n
