@@ -1,4 +1,4 @@
-import itertools
+import operator
 import warnings
 
 import numpy as np
@@ -8,21 +8,30 @@ import pytest
 from pytest import param
 
 import ibis
-import ibis.common.exceptions as com
 import ibis.expr.datatypes as dt
 from ibis.backends.pandas.execution.temporal import day_name
 
 
 @pytest.mark.parametrize('attr', ['year', 'month', 'day'])
-@pytest.mark.xfail_unsupported
-def test_date_extract(backend, alltypes, df, attr):
-    expr = getattr(alltypes.timestamp_col.date(), attr)()
+@pytest.mark.parametrize(
+    "expr_fn",
+    [
+        param(lambda c: c.date(), id="date"),
+        param(
+            lambda c: c.cast("date"),
+            id="cast",
+            marks=pytest.mark.notimpl(["impala"]),
+        ),
+    ],
+)
+@pytest.mark.notimpl(["datafusion"])
+def test_date_extract(backend, alltypes, df, attr, expr_fn):
+    expr = getattr(expr_fn(alltypes.timestamp_col), attr)()
     expected = getattr(df.timestamp_col.dt, attr).astype('int32')
 
-    result = expr.execute()
-    expected = backend.default_series_rename(expected)
+    result = expr.name(attr).execute()
 
-    backend.assert_series_equal(result, expected)
+    backend.assert_series_equal(result, expected.rename(attr))
 
 
 @pytest.mark.parametrize(
@@ -31,44 +40,53 @@ def test_date_extract(backend, alltypes, df, attr):
         'year',
         'month',
         'day',
-        'day_of_year',
+        param('day_of_year', marks=pytest.mark.notimpl(["impala"])),
         'quarter',
-        'epoch_seconds',
-        'week_of_year',
         'hour',
         'minute',
         'second',
-        'millisecond',
     ],
 )
-@pytest.mark.xfail_unsupported
+@pytest.mark.notimpl(["datafusion"])
 def test_timestamp_extract(backend, alltypes, df, attr):
-    if attr == 'millisecond':
-        if backend.name() == 'sqlite':
-            pytest.xfail(reason=('Issue #2156'))
-        if backend.name() == 'spark':
-            pytest.xfail(reason='Issue #2159')
-        expected = (df.timestamp_col.dt.microsecond // 1000).astype('int32')
-    elif attr == 'epoch_seconds':
-        expected = df.timestamp_col.astype('int64') // int(1e9)
-    elif attr == 'week_of_year':
-        expected = df.timestamp_col.dt.isocalendar().week.astype('int32')
-    else:
-        expected = getattr(df.timestamp_col.dt, attr.replace('_', '')).astype(
-            'int32'
-        )
-
-    expr = getattr(alltypes.timestamp_col, attr)()
+    method = getattr(alltypes.timestamp_col, attr)
+    expr = method().name(attr)
     result = expr.execute()
-    if attr == 'epoch_seconds' and backend.name() in [
-        'bigquery',
-        'postgres',
-        'spark',
-    ]:
-        # note: these backends cast to bigint are not changing the result
-        result = result.astype('int64')
-    expected = backend.default_series_rename(expected)
+    expected = backend.default_series_rename(
+        getattr(df.timestamp_col.dt, attr.replace('_', '')).astype('int32')
+    ).rename(attr)
+    backend.assert_series_equal(result, expected)
 
+
+@pytest.mark.notimpl(["datafusion", "clickhouse"])
+@pytest.mark.notyet(["sqlite", "pyspark"])
+def test_timestamp_extract_milliseconds(backend, alltypes, df):
+    expr = alltypes.timestamp_col.millisecond()
+    result = expr.execute()
+    expected = backend.default_series_rename(
+        (df.timestamp_col.dt.microsecond // 1_000).astype('int32')
+    ).rename("millisecond")
+    backend.assert_series_equal(result, expected)
+
+
+@pytest.mark.notimpl(["datafusion"])
+def test_timestamp_extract_epoch_seconds(backend, alltypes, df):
+    expr = alltypes.timestamp_col.epoch_seconds()
+    result = expr.execute()
+
+    expected = backend.default_series_rename(
+        (df.timestamp_col.view("int64") // 1_000_000_000).astype("int32")
+    )
+    backend.assert_series_equal(result, expected)
+
+
+@pytest.mark.notimpl(["datafusion"])
+def test_timestamp_extract_week_of_year(backend, alltypes, df):
+    expr = alltypes.timestamp_col.week_of_year()
+    result = expr.execute()
+    expected = backend.default_series_rename(
+        df.timestamp_col.dt.isocalendar().week.astype("int32")
+    )
     backend.assert_series_equal(result, expected)
 
 
@@ -78,23 +96,64 @@ def test_timestamp_extract(backend, alltypes, df, attr):
         'Y',
         'M',
         'D',
-        # Spark truncation to week truncates to different days than Pandas
-        # Pandas backend is probably doing this wrong
         param(
             'W',
-            marks=pytest.mark.xpass_backends(
-                ('csv', 'pandas', 'dask', 'parquet', 'hdf5')
+            marks=pytest.mark.notimpl(
+                [
+                    "clickhouse",
+                    "duckdb",
+                    "impala",
+                    "mysql",
+                    "postgres",
+                    "pyspark",
+                    "sqlite",
+                ]
+            ),
+        ),
+        param('h', marks=pytest.mark.notimpl(["sqlite"])),
+        param('m', marks=pytest.mark.notimpl(["sqlite"])),
+        param('s', marks=pytest.mark.notimpl(["impala", "sqlite"])),
+        param(
+            'ms',
+            marks=pytest.mark.notimpl(
+                [
+                    "clickhouse",
+                    "impala",
+                    "mysql",
+                    "pyspark",
+                    "sqlite",
+                ]
+            ),
+        ),
+        param(
+            'us',
+            marks=pytest.mark.notimpl(
+                [
+                    "clickhouse",
+                    "impala",
+                    "mysql",
+                    "pyspark",
+                    "sqlite",
+                ]
+            ),
+        ),
+        param(
+            'ns',
+            marks=pytest.mark.notimpl(
+                [
+                    "clickhouse",
+                    "duckdb",
+                    "impala",
+                    "mysql",
+                    "postgres",
+                    "pyspark",
+                    "sqlite",
+                ]
             ),
         ),
-        'h',
-        'm',
-        's',
-        'ms',
-        'us',
-        'ns',
     ],
 )
-@pytest.mark.xfail_unsupported
+@pytest.mark.notimpl(["datafusion"])
 def test_timestamp_truncate(backend, alltypes, df, unit):
     expr = alltypes.timestamp_col.truncate(unit)
 
@@ -115,17 +174,25 @@ def test_timestamp_truncate(backend, alltypes, df, unit):
         'D',
         param(
             'W',
-            marks=pytest.mark.xpass_backends(
-                ('csv', 'pandas', 'dask', 'parquet', 'hdf5')
+            marks=pytest.mark.notimpl(
+                [
+                    "clickhouse",
+                    "duckdb",
+                    "impala",
+                    "mysql",
+                    "postgres",
+                    "pyspark",
+                    "sqlite",
+                ]
             ),
         ),
     ],
 )
-@pytest.mark.xfail_unsupported
+@pytest.mark.notimpl(["datafusion"])
 def test_date_truncate(backend, alltypes, df, unit):
     expr = alltypes.timestamp_col.date().truncate(unit)
 
-    dtype = f'datetime64[{unit}]'
+    dtype = f"datetime64[{unit}]"
     expected = pd.Series(df.timestamp_col.values.astype(dtype))
 
     result = expr.execute()
@@ -141,50 +208,38 @@ def test_date_truncate(backend, alltypes, df, unit):
             'Y',
             pd.offsets.DateOffset,
             # TODO - DateOffset - #2553
-            marks=pytest.mark.xfail_backends(['dask']),
+            marks=pytest.mark.notimpl(['dask']),
         ),
         param('Q', pd.offsets.DateOffset, marks=pytest.mark.xfail),
         param(
             'M',
             pd.offsets.DateOffset,
             # TODO - DateOffset - #2553
-            marks=pytest.mark.xfail_backends(['dask']),
+            marks=pytest.mark.notimpl(['dask']),
         ),
         param(
             'W',
             pd.offsets.DateOffset,
             # TODO - DateOffset - #2553
-            marks=pytest.mark.xfail_backends(['dask']),
+            marks=pytest.mark.notimpl(['dask']),
         ),
-        ('D', pd.offsets.DateOffset),
-        ('h', pd.Timedelta),
-        ('m', pd.Timedelta),
-        ('s', pd.Timedelta),
+        param('D', pd.offsets.DateOffset),
+        param('h', pd.Timedelta),
+        param('m', pd.Timedelta),
+        param('s', pd.Timedelta),
         param(
             'ms',
             pd.Timedelta,
-            marks=pytest.mark.xpass_backends(
-                (
-                    'csv',
-                    'pandas',
-                    'parquet',
-                    'bigquery',
-                    'impala',
-                    'postgres',
-                    'dask',
-                    'hdf5',
-                )
-            ),
+            marks=pytest.mark.notimpl(["clickhouse", "mysql"]),
         ),
         param(
             'us',
             pd.Timedelta,
-            marks=pytest.mark.xfail_backends(('clickhouse', 'sqlite')),
+            marks=pytest.mark.notimpl(["clickhouse"]),
         ),
     ],
 )
-@pytest.mark.xfail_unsupported
-@pytest.mark.skip_backends(['spark'])
+@pytest.mark.notimpl(["datafusion", "pyspark", "sqlite"])
 def test_integer_to_interval_timestamp(
     backend, con, alltypes, df, unit, displacement_type
 ):
@@ -210,9 +265,17 @@ def test_integer_to_interval_timestamp(
 @pytest.mark.parametrize(
     'unit', ['Y', param('Q', marks=pytest.mark.xfail), 'M', 'W', 'D']
 )
-@pytest.mark.xfail_unsupported
 # TODO - DateOffset - #2553
-@pytest.mark.skip_backends(['dask', 'spark'])
+@pytest.mark.notimpl(
+    [
+        "dask",
+        "datafusion",
+        "impala",
+        "mysql",
+        "pyspark",
+        "sqlite",
+    ]
+)
 def test_integer_to_interval_date(backend, con, alltypes, df, unit):
     interval = alltypes.int_col.to_interval(unit=unit)
     array = alltypes.date_string_col.split('/')
@@ -238,17 +301,6 @@ def test_integer_to_interval_date(backend, con, alltypes, df, unit):
     backend.assert_series_equal(result, expected)
 
 
-@pytest.mark.parametrize('unit', ['h', 'm', 's', 'ms', 'us'])
-@pytest.mark.xfail_unsupported
-def test_integer_to_interval_date_failure(backend, con, alltypes, df, unit):
-    interval = alltypes.int_col.to_interval(unit=unit)
-    array = alltypes.date_string_col.split('/')
-    month, day, year = array[0], array[1], array[2]
-    date_col = ibis.literal('-').join(['20' + year, month, day]).cast('date')
-    with pytest.raises(TypeError):
-        date_col + interval
-
-
 date_value = pd.Timestamp('2017-12-31')
 timestamp_value = pd.Timestamp('2018-01-01 18:18:18')
 
@@ -267,6 +319,17 @@ timestamp_value = pd.Timestamp('2018-01-01 18:18:18')
             lambda t, be: t.timestamp_col
             + (pd.Timedelta(days=4) - pd.Timedelta(days=2)),
             id='timestamp-add-interval-binop',
+            marks=pytest.mark.notimpl(
+                [
+                    "clickhouse",
+                    "dask",
+                    "duckdb",
+                    "impala",
+                    "mysql",
+                    "pandas",
+                    "postgres",
+                ]
+            ),
         ),
         param(
             lambda t, be: t.timestamp_col - ibis.interval(days=17),
@@ -292,17 +355,17 @@ timestamp_value = pd.Timestamp('2018-01-01 18:18:18')
                 )
             ),
             id='timestamp-subtract-timestamp',
-            marks=pytest.mark.xfail_backends(['spark']),
+            marks=pytest.mark.notimpl(["duckdb", "pyspark"]),
         ),
         param(
             lambda t, be: t.timestamp_col.date() - ibis.date(date_value),
             lambda t, be: t.timestamp_col.dt.floor('d') - date_value,
             id='date-subtract-date',
+            marks=pytest.mark.notimpl(["pyspark"]),
         ),
     ],
 )
-@pytest.mark.xfail_unsupported
-@pytest.mark.skip_backends(['spark'])
+@pytest.mark.notimpl(["datafusion", "sqlite"])
 def test_temporal_binop(backend, con, alltypes, df, expr_fn, expected_fn):
     expr = expr_fn(alltypes, backend)
     expected = expected_fn(df, backend)
@@ -313,17 +376,30 @@ def test_temporal_binop(backend, con, alltypes, df, expr_fn, expected_fn):
     backend.assert_series_equal(result, expected)
 
 
+plus = lambda t, td: t.timestamp_col + pd.Timedelta(td)  # noqa: E731
+minus = lambda t, td: t.timestamp_col - pd.Timedelta(td)  # noqa: E731
+
+
 @pytest.mark.parametrize(
     ('timedelta', 'temporal_fn'),
-    itertools.product(
-        ['36500d', '5W', '3d', '1.5d', '2h', '3m', '10s'],
-        [
-            lambda t, td: t.timestamp_col + pd.Timedelta(td),
-            lambda t, td: t.timestamp_col - pd.Timedelta(td),
-        ],
-    ),
+    [
+        ('36500d', plus),
+        ('5W', plus),
+        ('3d', plus),
+        param('1.5d', plus, marks=pytest.mark.notimpl(["mysql"])),
+        param('2h', plus, marks=pytest.mark.notimpl(["mysql"])),
+        param('3m', plus, marks=pytest.mark.notimpl(["mysql"])),
+        param('10s', plus, marks=pytest.mark.notimpl(["mysql"])),
+        ('36500d', minus),
+        ('5W', minus),
+        ('3d', minus),
+        param('1.5d', minus, marks=pytest.mark.notimpl(["mysql"])),
+        param('2h', minus, marks=pytest.mark.notimpl(["mysql"])),
+        param('3m', minus, marks=pytest.mark.notimpl(["mysql"])),
+        param('10s', minus, marks=pytest.mark.notimpl(["mysql"])),
+    ],
 )
-@pytest.mark.only_on_backends(['dask', 'pandas', 'pyspark'])
+@pytest.mark.notimpl(["clickhouse", "datafusion", "impala", "sqlite"])
 def test_temporal_binop_pandas_timedelta(
     backend, con, alltypes, df, timedelta, temporal_fn
 ):
@@ -336,40 +412,32 @@ def test_temporal_binop_pandas_timedelta(
     backend.assert_series_equal(result, expected)
 
 
-@pytest.mark.only_on_backends(['pyspark'])
-def test_temporal_binop_invalid_interval_unit(con, alltypes):
-    expr = alltypes.timestamp_col + pd.Timedelta('1ns')
-    with pytest.raises(
-        com.UnsupportedArgumentError,
-        match='Interval unit "ns" is not allowed*',
-    ):
-        con.execute(expr)
-
-
 @pytest.mark.parametrize(
     'comparison_fn',
     [
-        lambda t: t.timestamp_col > pd.Timestamp('20100301'),
-        lambda t: t.timestamp_col >= pd.Timestamp('20100301'),
-        lambda t: t.timestamp_col < pd.Timestamp('20100301'),
-        lambda t: t.timestamp_col <= pd.Timestamp('20100301'),
-        lambda t: t.timestamp_col == pd.Timestamp('20100301'),
-        lambda t: t.timestamp_col != pd.Timestamp('20100301'),
+        operator.gt,
+        operator.ge,
+        operator.lt,
+        operator.le,
+        operator.eq,
+        operator.ne,
     ],
 )
-@pytest.mark.xfail_unsupported
-@pytest.mark.skip_backends(['spark', 'sqlite', 'datafusion'])
+@pytest.mark.notimpl(["sqlite"])
 def test_timestamp_comparison_filter(
     backend, con, alltypes, df, comparison_fn
 ):
-    expr = alltypes.filter(comparison_fn(alltypes))
-    expected = df[comparison_fn(df)]
+    ts = pd.Timestamp('20100301', tz="UTC").to_pydatetime()
+    expr = alltypes.filter(
+        comparison_fn(alltypes.timestamp_col.cast("timestamp('UTC')"), ts)
+    )
+    col = df.timestamp_col.dt.tz_localize("UTC")
+    expected = df[comparison_fn(col, ts)]
     result = con.execute(expr)
     backend.assert_frame_equal(result, expected)
 
 
-@pytest.mark.xfail_unsupported
-@pytest.mark.skip_backends(['spark'])
+@pytest.mark.notimpl(["datafusion", "sqlite"])
 def test_interval_add_cast_scalar(backend, alltypes):
     timestamp_date = alltypes.timestamp_col.date()
     delta = ibis.literal(10).cast("interval('D')")
@@ -379,10 +447,10 @@ def test_interval_add_cast_scalar(backend, alltypes):
     backend.assert_series_equal(result, expected)
 
 
-@pytest.mark.xfail_unsupported
-# PySpark does not support casting columns to intervals
-@pytest.mark.xfail_backends(['pyspark'])
-@pytest.mark.skip_backends(['spark'])
+@pytest.mark.never(
+    ['pyspark'], reason="PySpark does not support casting columns to intervals"
+)
+@pytest.mark.notimpl(["datafusion", "sqlite"])
 def test_interval_add_cast_column(backend, alltypes, df):
     timestamp_date = alltypes.timestamp_col.date()
     delta = alltypes.bigint_col.cast("interval('D')")
@@ -401,9 +469,7 @@ def test_interval_add_cast_column(backend, alltypes, df):
 @pytest.mark.parametrize(
     ('ibis_pattern', 'pandas_pattern'), [('%Y%m%d', '%Y%m%d')]
 )
-@pytest.mark.xfail_unsupported
-# Spark takes Java SimpleDateFormat instead of strftime
-@pytest.mark.skip_backends(['spark'])
+@pytest.mark.notimpl(["clickhouse", "datafusion", "duckdb", "impala"])
 def test_strftime(backend, con, alltypes, df, ibis_pattern, pandas_pattern):
     expr = alltypes.timestamp_col.strftime(ibis_pattern)
     expected = df.timestamp_col.dt.strftime(pandas_pattern)
@@ -420,35 +486,22 @@ unit_factors = {'s': int(1e9), 'ms': int(1e6), 'us': int(1e3), 'ns': 1}
     'unit',
     [
         's',
-        'ms',
+        param(
+            'ms',
+            marks=pytest.mark.notimpl(["clickhouse", "pyspark"]),
+        ),
         param(
             'us',
-            marks=pytest.mark.xpass_backends(
-                (
-                    'bigquery',
-                    'csv',
-                    'impala',
-                    'pandas',
-                    'parquet',
-                    'spark',
-                    'dask',
-                    'hdf5',
-                )
-            ),
+            marks=pytest.mark.notimpl(["clickhouse", "duckdb", "pyspark"]),
         ),
         param(
             'ns',
-            marks=pytest.mark.xpass_backends(
-                ('csv', 'pandas', 'parquet', 'dask', 'hdf5')
-            ),
+            marks=pytest.mark.notimpl(["clickhouse", "duckdb", "pyspark"]),
         ),
     ],
 )
-@pytest.mark.xfail_unsupported
+@pytest.mark.notimpl(["datafusion", "mysql", "postgres", "sqlite"])
 def test_to_timestamp(backend, con, unit):
-    if unit not in backend.supported_to_timestamp_units:
-        pytest.skip(f'Unit {unit!r} not supported by {backend} to_timestamp')
-
     backend_unit = backend.returned_timestamp_unit
     factor = unit_factors[unit]
 
@@ -467,16 +520,16 @@ def test_to_timestamp(backend, con, unit):
 @pytest.mark.parametrize(
     ('date', 'expected_index', 'expected_day'),
     [
-        ('2017-01-01', 6, 'Sunday'),
-        ('2017-01-02', 0, 'Monday'),
-        ('2017-01-03', 1, 'Tuesday'),
-        ('2017-01-04', 2, 'Wednesday'),
-        ('2017-01-05', 3, 'Thursday'),
-        ('2017-01-06', 4, 'Friday'),
-        ('2017-01-07', 5, 'Saturday'),
+        param('2017-01-01', 6, 'Sunday', id="sunday"),
+        param('2017-01-02', 0, 'Monday', id="monday"),
+        param('2017-01-03', 1, 'Tuesday', id="tuesday"),
+        param('2017-01-04', 2, 'Wednesday', id="wednesday"),
+        param('2017-01-05', 3, 'Thursday', id="thursday"),
+        param('2017-01-06', 4, 'Friday', id="friday"),
+        param('2017-01-07', 5, 'Saturday', id="saturday"),
     ],
 )
-@pytest.mark.xfail_unsupported
+@pytest.mark.notimpl(["datafusion", "impala"])
 def test_day_of_week_scalar(backend, con, date, expected_index, expected_day):
     expr = ibis.literal(date).cast(dt.date)
     result_index = con.execute(expr.day_of_week.index())
@@ -486,7 +539,7 @@ def test_day_of_week_scalar(backend, con, date, expected_index, expected_day):
     assert result_day.lower() == expected_day.lower()
 
 
-@pytest.mark.xfail_unsupported
+@pytest.mark.notimpl(["datafusion"])
 def test_day_of_week_column(backend, con, alltypes, df):
     expr = alltypes.timestamp_col.day_of_week
 
@@ -506,17 +559,19 @@ def test_day_of_week_column(backend, con, alltypes, df):
 @pytest.mark.parametrize(
     ('day_of_week_expr', 'day_of_week_pandas'),
     [
-        (
+        param(
             lambda t: t.timestamp_col.day_of_week.index().count(),
             lambda s: s.dt.dayofweek.count(),
+            id="day_of_week_index",
         ),
-        (
+        param(
             lambda t: t.timestamp_col.day_of_week.full_name().length().sum(),
             lambda s: day_name(s.dt).str.len().sum(),
+            id="day_of_week_full_name",
         ),
     ],
 )
-@pytest.mark.xfail_unsupported
+@pytest.mark.notimpl(["datafusion"])
 def test_day_of_week_column_group_by(
     backend, con, alltypes, df, day_of_week_expr, day_of_week_pandas
 ):
@@ -538,7 +593,7 @@ def test_day_of_week_column_group_by(
     backend.assert_frame_equal(result, expected, check_dtype=False)
 
 
-@pytest.mark.xfail_unsupported
+@pytest.mark.notimpl(["datafusion"])
 def test_now(backend, con):
     expr = ibis.now()
     result = con.execute(expr)
@@ -550,10 +605,9 @@ def test_now(backend, con):
     assert result.year == pandas_now.year
 
 
-# # TODO - limit - #2553
-@pytest.mark.xfail_backends(['dask'])
-@pytest.mark.xfail_unsupported
-def test_now_from_projection(backend, con, alltypes, df):
+@pytest.mark.notimpl(["dask"], reason="Limit #2553")
+@pytest.mark.notimpl(["datafusion"])
+def test_now_from_projection(backend, alltypes):
     n = 5
     expr = alltypes[[ibis.now().name('ts')]].limit(n)
     result = expr.execute()
@@ -567,3 +621,75 @@ def test_now_from_projection(backend, con, alltypes, df):
     now = pd.Timestamp('now')
     year_expected = pd.Series([now.year] * n, name='ts')
     tm.assert_series_equal(ts.dt.year, year_expected)
+
+
+@pytest.mark.notimpl(["pandas", "datafusion", "mysql", "dask", "pyspark"])
+@pytest.mark.notyet(["clickhouse", "impala"])
+def test_date_literal(con):
+    expr = ibis.date(2022, 2, 4)
+    result = con.execute(expr)
+    assert result.strftime('%Y-%m-%d') == '2022-02-04'
+
+
+@pytest.mark.notimpl(["pandas", "datafusion", "mysql", "dask", "pyspark"])
+@pytest.mark.notyet(["clickhouse", "impala"])
+def test_timestamp_literal(con):
+    expr = ibis.timestamp(2022, 2, 4, 16, 20, 0)
+    result = con.execute(expr)
+    if not isinstance(result, str):
+        result = result.strftime('%Y-%m-%d %H:%M:%S%Z')
+    assert result == '2022-02-04 16:20:00'
+
+
+@pytest.mark.notimpl(["pandas", "datafusion", "mysql", "dask", "pyspark"])
+@pytest.mark.notyet(["clickhouse", "impala"])
+def test_time_literal(con):
+    expr = ibis.time(16, 20, 0)
+    result = con.execute(expr)
+    if not isinstance(result, str):
+        result = result.strftime('%H:%M:%S')
+    assert result == '16:20:00'
+
+
+@pytest.mark.notimpl(["pandas", "datafusion", "mysql", "dask", "pyspark"])
+@pytest.mark.notyet(["clickhouse", "impala"])
+def test_date_column_from_ymd(con, alltypes, df):
+    c = alltypes.timestamp_col
+    expr = ibis.date(c.year(), c.month(), c.day())
+    tbl = alltypes[
+        expr.name('timestamp_col'),
+    ]
+    result = con.execute(tbl)
+
+    golden = df.timestamp_col.dt.date.astype('datetime64[ns]')
+    tm.assert_series_equal(golden, result.timestamp_col)
+
+
+@pytest.mark.notimpl(["datafusion", "impala"])
+def test_date_scalar_from_iso(con):
+    expr = ibis.literal('2022-02-24')
+    expr2 = ibis.date(expr)
+
+    result = con.execute(expr2)
+    assert result.strftime('%Y-%m-%d') == '2022-02-24'
+
+
+@pytest.mark.notimpl(["datafusion", "impala", "pyspark"])
+def test_date_column_from_iso(con, alltypes, df):
+    expr = (
+        alltypes.year.cast('string')
+        + '-'
+        + alltypes.month.cast('string').lpad(2, '0')
+        + '-13'
+    )
+    expr = ibis.date(expr)
+
+    result = con.execute(expr)
+    golden = (
+        df.year.astype(str)
+        + '-'
+        + df.month.astype(str).str.rjust(2, '0')
+        + '-13'
+    )
+    actual = result.dt.strftime('%Y-%m-%d')
+    tm.assert_series_equal(golden.rename('tmp'), actual.rename('tmp'))
