@@ -89,11 +89,11 @@ class Substitutor:
             except IbisTypeError:
                 return expr
 
-            try:
-                name = expr.get_name()
-            except ExpressionError:
-                name = None
-            return expr._factory(new_node, name=name)
+            new_expr = type(expr)(new_node)
+            if expr.has_name():
+                new_expr = new_expr.name(expr.get_name())
+
+            return new_expr
 
 
 class ScalarAggregate:
@@ -144,13 +144,12 @@ class ScalarAggregate:
                 subbed_arg = self._visit(arg)
             subbed_args.append(subbed_arg)
 
-        subbed_node = type(node)(*subbed_args)
-        if isinstance(expr, ir.ValueExpr):
-            result = expr._factory(subbed_node, name=expr._name)
-        else:
-            result = expr._factory(subbed_node)
+        new_op = type(node)(*subbed_args)
+        new_expr = new_op.to_expr()
+        if expr.has_name():
+            new_expr = new_expr.name(name=expr.get_name())
 
-        return result
+        return new_expr
 
     def _key(self, expr):
         return repr(expr.op())
@@ -202,24 +201,13 @@ def find_immediate_parent_tables(expr):
     >>> result = list(find_immediate_parent_tables(expr))
     >>> len(result)
     1
-    >>> result[0]  # doctest: +NORMALIZE_WHITESPACE
-    ref_0
-    UnboundTable[table]
-      name: t
-      schema:
-        a : int64
-    Selection[table]
-      table:
-        Table: ref_0
+    >>> result[0]
+    r0 := UnboundTable[t]
+      a int64
+    Selection[r0]
       selections:
-        Table: ref_0
-        foo = Add[int64*]
-          left:
-            a = Column[int64*] 'a' from table
-              ref_0
-          right:
-            Literal[int8]
-              1
+        r0
+        foo: r0.a + 1
     """
 
     def finder(expr):
@@ -289,10 +277,10 @@ class ExprSimplifier:
             return expr
 
         lifted_node = type(node)(*lifted_args)
-        if isinstance(expr, ir.ValueExpr):
-            result = expr._factory(lifted_node, name=expr._name)
-        else:
-            result = expr._factory(lifted_node)
+
+        result = type(expr)(lifted_node)
+        if isinstance(expr, ir.ValueExpr) and expr.has_name():
+            result = result.name(expr.get_name())
 
         return result
 
@@ -351,20 +339,6 @@ class ExprSimplifier:
 
         if isinstance(root, ops.Selection):
             can_lift = False
-            all_simple_columns = all(
-                isinstance(sel.op(), ops.TableColumn)
-                and sel.op().name == sel.get_name()
-                for sel in root.selections
-                if isinstance(sel, ir.ValueExpr)
-                if sel.has_name()
-            )
-
-            # If there are predicates or sort keys,
-            # then we cannot lift the root
-            no_predicates_or_sort_keys = not (
-                root.predicates or root.sort_keys
-            )
-
             for val in root.selections:
                 value_op = val.op()
                 if (
@@ -373,19 +347,10 @@ class ExprSimplifier:
                 ):
                     can_lift = True
                     lifted_root = self.lift(val)
-                elif (
-                    all_simple_columns
-                    and no_predicates_or_sort_keys
-                    and isinstance(val, ir.ValueExpr)
-                    and val.has_name()
-                    and node.name == val.get_name()
-                ):
-                    can_lift = True
-                    lifted_root = self.lift(value_op.table)
 
             if can_lift and not block:
                 lifted_node = ops.TableColumn(lifted_root, node.name)
-                result = expr._factory(lifted_node, name=expr._name)
+                result = type(expr)(lifted_node)
 
         return result
 
@@ -566,7 +531,7 @@ def get_mutation_exprs(
         is_first_overwrite = True
         expr_contains_overwrite = False
         if isinstance(expr, ir.DestructColumn):
-            if expr.get_name():
+            if expr.has_name():
                 raise ExpressionError(
                     f"Cannot name a destruct column: {expr.get_name()}"
                 )
@@ -619,12 +584,17 @@ def apply_filter(expr, predicates):
     elif isinstance(op, ops.Aggregation):
         # Potential fusion opportunity
         # GH1344: We can't sub in things with correlated subqueries
-        simplified_predicates = [
-            sub_for(predicate, [(expr, op.table)])
+        simplified_predicates = tuple(
+            # Originally this line tried substituting op.table in for expr, but
+            # that is too aggressive in the presence of filters that occur
+            # after aggregations.
+            #
+            # See https://github.com/ibis-project/ibis/pull/3341 for details
+            sub_for(predicate, [(op.table, expr)])
             if not has_reduction(predicate)
             else predicate
             for predicate in predicates
-        ]
+        )
 
         if op.table._is_valid(simplified_predicates):
             result = ops.Aggregation(
@@ -637,8 +607,6 @@ def apply_filter(expr, predicates):
             )
 
             return ir.TableExpr(result)
-    elif isinstance(op, ops.Join):
-        expr = expr.materialize()
 
     result = ops.Selection(expr, [], predicates)
     return ir.TableExpr(result)
@@ -663,12 +631,12 @@ def _filter_selection(expr, predicates):
         # Potential fusion opportunity. The predicates may need to be
         # rewritten in terms of the child table. This prevents the broken
         # ref issue (described in more detail in #59)
-        simplified_predicates = [
+        simplified_predicates = tuple(
             sub_for(predicate, [(expr, op.table)])
             if not has_reduction(predicate)
             else predicate
             for predicate in predicates
-        ]
+        )
 
         if op.table._is_valid(simplified_predicates):
             result = ops.Selection(
@@ -682,7 +650,9 @@ def _filter_selection(expr, predicates):
     can_pushdown = _can_pushdown(op, predicates)
 
     if can_pushdown:
-        simplified_predicates = [substitute_parents(x) for x in predicates]
+        simplified_predicates = tuple(
+            substitute_parents(x) for x in predicates
+        )
         fused_predicates = op.predicates + simplified_predicates
         result = ops.Selection(
             op.table,
@@ -766,7 +736,10 @@ class _PushdownValidate:
                 # Aliased table columns are no good
                 col_table = val.op().table.op()
 
-                lifted_node = substitute_parents(expr).op()
+                lifted_node = substitute_parents(
+                    expr,
+                    past_projection=False,
+                ).op()
 
                 is_valid = col_table.equals(
                     node.table.op()
@@ -788,9 +761,8 @@ def windowize_function(expr, w=None):
             walked_child = _walk(window_arg, w)
 
             if walked_child is not window_arg:
-                walked = x._factory(
-                    ops.WindowOp(walked_child, window_w), name=x._name
-                )
+                op = ops.WindowOp(walked_child, window_w)
+                walked = op.to_expr().name(x.get_name())
             else:
                 walked = x
 
@@ -801,7 +773,7 @@ def windowize_function(expr, w=None):
             return walked.over(w)
         elif isinstance(op, ops.WindowOp):
             if w is not None:
-                return walked.over(w)
+                return walked.over(w.combine(op.window))
             else:
                 return walked
         else:
@@ -823,7 +795,10 @@ def windowize_function(expr, w=None):
 
         if not unchanged:
             new_op = type(op)(*windowed_args)
-            return x._factory(new_op, name=x._name)
+            expr = new_op.to_expr()
+            if x.has_name():
+                expr = expr.name(x.get_name())
+            return expr
         else:
             return x
 
@@ -889,8 +864,7 @@ class Projector:
         root_selections = root.selections
         parent_op = self.parent.op()
         for val in resolved:
-            # XXX
-            lifted_val = substitute_parents(val)
+            lifted_val = substitute_parents(val, past_projection=False)
 
             # a * projection
             if isinstance(val, ir.TableExpr) and (
@@ -945,6 +919,7 @@ class ExprValidator:
 
     def validate(self, expr):
         op = expr.op()
+
         if isinstance(op, ops.TableColumn):
             if self._among_roots(op.table.op()):
                 return True
@@ -962,7 +937,10 @@ class ExprValidator:
         return self.roots_shared(node) > 0
 
     def roots_shared(self, node):
-        return sum(root.is_ancestor(node) for root in self.roots)
+        return sum(
+            root.is_ancestor(node) or node.compatible_with(root)
+            for root in self.roots
+        )
 
     def shares_some_roots(self, expr):
         expr_roots = expr.op().root_tables()
@@ -1074,31 +1052,18 @@ def find_source_table(expr):
     >>> import ibis
     >>> t = ibis.table([('a', 'double'), ('b', 'string')], name='t')
     >>> expr = t.mutate(c=t.a + 42.0)
-    >>> expr  # doctest: +NORMALIZE_WHITESPACE
-    ref_0
-    UnboundTable[table]
-      name: t
-      schema:
-        a : float64
-        b : string
-    Selection[table]
-      table:
-        Table: ref_0
+    >>> expr
+    r0 := UnboundTable[t]
+      a float64
+      b string
+    Selection[r0]
       selections:
-        Table: ref_0
-        c = Add[float64*]
-          left:
-            a = Column[float64*] 'a' from table
-              ref_0
-          right:
-            Literal[float64]
-              42.0
+        r0
+        c: r0.a + 42.0
     >>> find_source_table(expr)
-    UnboundTable[table]
-      name: t
-      schema:
-        a : float64
-        b : string
+    UnboundTable[t]
+      a float64
+      b string
     >>> left = ibis.table([('a', 'int64'), ('b', 'string')])
     >>> right = ibis.table([('c', 'int64'), ('d', 'string')])
     >>> result = left.inner_join(right, left.a == right.c)
@@ -1142,34 +1107,16 @@ def flatten_predicate(expr):
     >>> predicates = flatten_predicate(filt)
     >>> len(predicates)
     2
-    >>> predicates[0]  # doctest: +NORMALIZE_WHITESPACE
-    ref_0
-    UnboundTable[table]
-      name: t
-      schema:
-        a : int64
-        b : string
-    Equals[boolean*]
-      left:
-        a = Column[int64*] 'a' from table
-          ref_0
-      right:
-        Literal[int64]
-          1
-    >>> predicates[1]  # doctest: +NORMALIZE_WHITESPACE
-    ref_0
-    UnboundTable[table]
-      name: t
-      schema:
-        a : int64
-        b : string
-    Equals[boolean*]
-      left:
-        b = Column[string*] 'b' from table
-          ref_0
-      right:
-        Literal[string]
-          foo
+    >>> predicates[0]
+    r0 := UnboundTable[t]
+      a int64
+      b string
+    r0.a == 1
+    >>> predicates[1]
+    r0 := UnboundTable[t]
+      a int64
+      b string
+    r0.b == 'foo'
     """
 
     def predicate(expr):
