@@ -7,81 +7,6 @@ import ibis.expr.operations as ops
 import ibis.expr.types as ir
 
 
-def find_nodes(expr, node_types):
-    """Depth-first search of the expression tree yielding nodes of a given
-    type or set of types.
-
-    Parameters
-    ----------
-    expr: ibis.expr.types.Expr
-    node_types: type or tuple of types
-
-    Yields
-    ------
-    op: type
-        A node of given node_types
-    """
-
-    def extender(op):
-        return (arg for arg in op.args if isinstance(arg, ir.Expr))
-
-    return _search_for_nodes([expr], extender, node_types)
-
-
-def _search_for_nodes(stack, extender, node_types):
-    seen = set()
-    while stack:
-        expr = stack.pop()
-        op = expr.op()
-        if op not in seen:
-            if isinstance(op, node_types):
-                yield op
-            seen.add(op)
-            stack.extend(extender(op))
-
-
-def roots(expr, types=(ops.PhysicalTable,)):
-    """Yield every node of a particular type on which an expression depends.
-
-    Parameters
-    ----------
-    expr : Expr
-        The expression to analyze
-    types : tuple(type), optional, default
-            (:mod:`ibis.expr.operations.PhysicalTable`,)
-        The node types to traverse
-
-    Yields
-    ------
-    table : Expr
-        Unique node types on which an expression depends
-
-    Notes
-    -----
-    If your question is: "What nodes of type T does `expr` depend on?", then
-    you've come to the right place. By default, we yield the physical tables
-    that an expression depends on.
-    """
-    stack = [
-        arg.to_expr()
-        for arg in reversed(expr.op().root_tables())
-        if isinstance(arg, types)
-    ]
-
-    def extender(op):
-        return reversed(
-            list(
-                itertools.chain.from_iterable(
-                    arg.op().root_tables()
-                    for arg in op.flat_args()
-                    if isinstance(arg, types)
-                )
-            )
-        )
-
-    return _search_for_nodes(stack, extender, types)
-
-
 class Container:
 
     __slots__ = ('data',)
@@ -150,13 +75,13 @@ def _get_args(op, name):
 
         # if Selection.selections is always columnar, could use an
         # OrderedDict to prevent scanning the whole thing
-        return [col for col in result if col._name == name]
+        return [col for col in result if col.get_name() == name]
     elif isinstance(op, ops.Aggregation):
         assert name is not None, 'name is None'
         return [
             col
             for col in itertools.chain(op.by, op.metrics)
-            if col._name == name
+            if col.get_name() == name
         ]
     else:
         return op.args
@@ -186,7 +111,7 @@ def lineage(expr, container=Stack):
     if not isinstance(expr, ir.ColumnExpr):
         raise TypeError('Input expression must be an instance of ColumnExpr')
 
-    c = container([(expr, expr._name)])
+    c = container([(expr, expr.get_name() if expr.has_name() else None)])
 
     seen = set()
 
@@ -201,7 +126,7 @@ def lineage(expr, container=Stack):
         # add our dependencies to the container if they match our name
         # and are ibis expressions
         c.extend(
-            (arg, getattr(arg, '_name', name))
+            (arg, arg.get_name() if arg.has_name() else name)
             for arg in c.visitor(_get_args(node.op(), name))
             if isinstance(arg, ir.Expr)
         )
@@ -224,9 +149,10 @@ def traverse(fn, expr, type=ir.Expr, container=Stack):
     expr: ir.Expr
         The traversable expression or a list of expressions.
     type: Type
-        Only the instances if this type are traversed.
+        Only the instances if this expression type gets traversed.
     container: Union[Stack, Queue], default Stack
-        Defines the traversing order.
+        Defines the traversing order. Use Stack for depth-first and
+        Queue for breadth-first search.
     """
     args = expr if isinstance(expr, collections.abc.Iterable) else [expr]
     todo = container(arg for arg in args if isinstance(arg, type))
