@@ -1,18 +1,19 @@
+from __future__ import annotations
+
 import collections
 import itertools
-from typing import List
 
 from cached_property import cached_property
 from public import public
 
-from ... import util
-from ...common import exceptions as com
-from .. import rules as rlz
-from .. import schema as sch
-from .. import types as ir
-from ..signature import Argument as Arg
-from .core import Node, all_equal, distinct_roots
-from .sortkeys import _maybe_convert_sort_keys
+from ibis import util
+from ibis.common import exceptions as com
+from ibis.expr import datatypes as dt
+from ibis.expr import rules as rlz
+from ibis.expr import schema as sch
+from ibis.expr import types as ir
+from ibis.expr.operations.core import Node, ValueOp, distinct_roots
+from ibis.expr.operations.sortkeys import _maybe_convert_sort_keys
 
 _table_names = (f'unbound_table_{i:d}' for i in itertools.count())
 
@@ -24,12 +25,11 @@ def genname():
 
 @public
 class TableNode(Node):
+    output_type = ir.TableExpr
+
     def get_type(self, name):
         return self.schema[name]
 
-    def output_type(self):
-        return ir.TableExpr
-
     def aggregate(self, this, metrics, by=None, having=None):
         return Aggregation(this, metrics, by=by, having=having)
 
@@ -68,15 +68,21 @@ class PhysicalTable(TableNode, sch.HasSchema):
 
 @public
 class UnboundTable(PhysicalTable):
-    schema = Arg(sch.Schema)
-    name = Arg(str, default=genname)
+    schema = rlz.instance_of(sch.Schema)
+    name = rlz.optional(rlz.instance_of(str), default=genname)
+
+    def has_resolved_name(self):
+        return True
+
+    def resolve_name(self):
+        return self.name
 
 
 @public
 class DatabaseTable(PhysicalTable):
-    name = Arg(str)
-    schema = Arg(sch.Schema)
-    source = Arg(rlz.client)
+    name = rlz.instance_of(str)
+    schema = rlz.instance_of(sch.Schema)
+    source = rlz.client
 
     def change_name(self, new_name):
         return type(self)(new_name, self.args[1], self.source)
@@ -86,23 +92,34 @@ class DatabaseTable(PhysicalTable):
 class SQLQueryResult(TableNode, sch.HasSchema):
     """A table sourced from the result set of a select query"""
 
-    query = Arg(str)
-    schema = Arg(sch.Schema)
-    source = Arg(rlz.client)
+    query = rlz.instance_of(str)
+    schema = rlz.instance_of(sch.Schema)
+    source = rlz.client
 
     def blocks(self):
         return True
 
 
 def _make_distinct_join_predicates(left, right, predicates):
-    # see GH #667
-
-    # If left and right table have a common parent expression (e.g. they
-    # have different filters), must add a self-reference and make the
-    # appropriate substitution in the join predicates
+    import ibis.expr.analysis as L
 
     if left.equals(right):
+        # GH #667: If left and right table have a common parent expression,
+        # e.g. they have different filters, we need to add a self-reference and
+        # make the appropriate substitution in the join predicates
         right = right.view()
+    elif isinstance(right.op(), Join):
+        # for joins with joins on the right side we turn the right side into a
+        # view, otherwise the join tree is incorrectly flattened and tables on
+        # the right are incorrectly scoped
+        old = right
+        new = right = right.view()
+        predicates = [
+            L.sub_for(pred, [(old, new)])
+            if isinstance(pred, ir.Expr)
+            else pred
+            for pred in predicates
+        ]
 
     predicates = _clean_join_predicates(left, right, predicates)
     return left, right, predicates
@@ -113,9 +130,6 @@ def _clean_join_predicates(left, right, predicates):
 
     result = []
 
-    if not isinstance(predicates, (list, tuple)):
-        predicates = [predicates]
-
     for pred in predicates:
         if isinstance(pred, tuple):
             if len(pred) != 2:
@@ -136,7 +150,7 @@ def _clean_join_predicates(left, right, predicates):
         result.extend(preds)
 
     _validate_join_predicates(left, right, result)
-    return result
+    return tuple(result)
 
 
 def _validate_join_predicates(left, right, predicates):
@@ -155,40 +169,27 @@ def _validate_join_predicates(left, right, predicates):
 
 @public
 class Join(TableNode):
-    left = Arg(rlz.table)
-    right = Arg(rlz.table)
-    predicates = Arg(rlz.list_of(rlz.boolean), default=[])
+    left = rlz.table
+    right = rlz.table
+    # TODO(kszucs): convert to proper predicate rules
+    predicates = rlz.optional(lambda x, this: x, default=())
 
-    def __init__(self, left, right, predicates):
+    def __init__(self, left, right, predicates, **kwargs):
         left, right, predicates = _make_distinct_join_predicates(
-            left, right, predicates
+            left, right, util.promote_list(predicates)
+        )
+        super().__init__(
+            left=left, right=right, predicates=predicates, **kwargs
         )
-        super().__init__(left, right, predicates)
 
-    def _get_schema(self):
+    # could use @promoter here
+    @property
+    def schema(self):
         # For joins retaining both table schemas, merge them together here
-        left = self.left
-        right = self.right
-
-        if not left._is_materialized():
-            left = left.materialize()
-
-        if not right._is_materialized():
-            right = right.materialize()
-
-        sleft = left.schema()
-        sright = right.schema()
-
-        overlap = set(sleft.names) & set(sright.names)
-        if overlap:
-            raise com.RelationError(
-                'Joined tables have overlapping names: %s' % str(list(overlap))
-            )
-
-        return sleft.append(sright)
+        return self.left.schema().append(self.right.schema())
 
     def has_schema(self):
-        return False
+        return not set(self.left.columns) & set(self.right.columns)
 
     def root_tables(self):
         if util.all_of([self.left.op(), self.right.op()], (Join, Selection)):
@@ -230,13 +231,15 @@ class AnyLeftJoin(Join):
 
 @public
 class LeftSemiJoin(Join):
-    def _get_schema(self):
+    @property
+    def schema(self):
         return self.left.schema()
 
 
 @public
 class LeftAntiJoin(Join):
-    def _get_schema(self):
+    @property
+    def schema(self):
         return self.left.schema()
 
 
@@ -245,72 +248,32 @@ class CrossJoin(Join):
     pass
 
 
-@public
-class MaterializedJoin(TableNode, sch.HasSchema):
-    join = Arg(rlz.table)
-
-    def _validate(self):
-        assert isinstance(self.join.op(), Join)
-        # check whether the underlying schema has overlapping columns or not
-        assert self.schema
-
-    @cached_property
-    def schema(self):
-        return self.join.op()._get_schema()
-
-    def root_tables(self):
-        return self.join.op().root_tables()
-
-    def blocks(self):
-        return True
-
-
 @public
 class AsOfJoin(Join):
-    left = Arg(rlz.table)
-    right = Arg(rlz.table)
-    predicates = Arg(rlz.list_of(rlz.boolean))
-    by = Arg(
-        rlz.list_of(
-            rlz.one_of(
-                (
-                    rlz.function_of("table"),
-                    rlz.column_from("table"),
-                    rlz.any,
-                )
-            )
-        ),
-        default=[],
-    )
-    tolerance = Arg(rlz.interval, default=None)
+    # TODO(kszucs): convert to proper predicate rules
+    by = rlz.optional(lambda x, this: x, default=())
+    tolerance = rlz.optional(rlz.interval)
 
-    def __init__(self, left, right, predicates, by, tolerance):
-        super().__init__(left, right, predicates)
-        self.by = _clean_join_predicates(self.left, self.right, by)
-        self.tolerance = tolerance
-
-        self._validate_args(['by', 'tolerance'])
-
-    def _validate_args(self, args: List[str]):
-        # this should be removed altogether
-        for arg in args:
-            argument = self.__signature__.parameters[arg]
-            value = argument.validate(self, getattr(self, arg))
-            setattr(self, arg, value)
+    def __init__(self, left, right, by, predicates, **kwargs):
+        by = _clean_join_predicates(left, right, util.promote_list(by))
+        super().__init__(
+            left=left, right=right, by=by, predicates=predicates, **kwargs
+        )
 
 
 @public
 class SetOp(TableNode, sch.HasSchema):
-    left = Arg(rlz.table)
-    right = Arg(rlz.table)
+    left = rlz.table
+    right = rlz.table
 
-    def _validate(self):
-        if not self.left.schema().equals(self.right.schema()):
+    def __init__(self, left, right, **kwargs):
+        if not left.schema().equals(right.schema()):
             raise com.RelationError(
                 'Table schemas must be equal for set operations'
             )
+        super().__init__(left=left, right=right, **kwargs)
 
-    @cached_property
+    @property
     def schema(self):
         return self.left.schema()
 
@@ -320,7 +283,7 @@ class SetOp(TableNode, sch.HasSchema):
 
 @public
 class Union(SetOp):
-    distinct = Arg(rlz.instance_of(bool), default=False)
+    distinct = rlz.optional(rlz.instance_of(bool), default=False)
 
 
 @public
@@ -335,9 +298,9 @@ class Difference(SetOp):
 
 @public
 class Limit(TableNode):
-    table = Arg(rlz.table)
-    n = Arg(rlz.instance_of(int))
-    offset = Arg(rlz.instance_of(int))
+    table = rlz.table
+    n = rlz.instance_of(int)
+    offset = rlz.instance_of(int)
 
     def blocks(self):
         return True
@@ -355,9 +318,9 @@ class Limit(TableNode):
 
 @public
 class SelfReference(TableNode, sch.HasSchema):
-    table = Arg(rlz.table)
+    table = rlz.table
 
-    @cached_property
+    @property
     def schema(self):
         return self.table.schema()
 
@@ -373,9 +336,9 @@ class SelfReference(TableNode, sch.HasSchema):
 
 @public
 class Selection(TableNode, sch.HasSchema):
-    table = Arg(rlz.table)
-    selections = Arg(
-        rlz.list_of(
+    table = rlz.table
+    selections = rlz.optional(
+        rlz.tuple_of(
             rlz.one_of(
                 (
                     rlz.table,
@@ -386,11 +349,11 @@ class Selection(TableNode, sch.HasSchema):
                 )
             )
         ),
-        default=[],
+        default=(),
     )
-    predicates = Arg(rlz.list_of(rlz.boolean), default=[])
-    sort_keys = Arg(
-        rlz.list_of(
+    predicates = rlz.optional(rlz.tuple_of(rlz.boolean), default=())
+    sort_keys = rlz.optional(
+        rlz.tuple_of(
             rlz.one_of(
                 (
                     rlz.column_from("table"),
@@ -420,21 +383,28 @@ class Selection(TableNode, sch.HasSchema):
                 )
             )
         ),
-        default=[],
+        default=(),
     )
 
-    def _validate(self):
+    def __init__(self, table, selections, predicates, sort_keys):
         from ibis.expr.analysis import FilterValidator
 
         # Need to validate that the column expressions are compatible with the
         # input table; this means they must either be scalar expressions or
         # array expressions originating from the same root table expression
-        dependent_exprs = self.selections + self.sort_keys
-        self.table._assert_valid(dependent_exprs)
+        dependent_exprs = selections + sort_keys
+        table._assert_valid(dependent_exprs)
 
         # Validate predicates
-        validator = FilterValidator([self.table])
-        validator.validate_all(self.predicates)
+        validator = FilterValidator([table])
+        validator.validate_all(predicates)
+
+        super().__init__(
+            table=table,
+            selections=selections,
+            predicates=predicates,
+            sort_keys=sort_keys,
+        )
 
         # Validate no overlapping columns in schema
         assert self.schema
@@ -478,9 +448,18 @@ class Selection(TableNode, sch.HasSchema):
     def can_add_filters(self, wrapped_expr, predicates):
         pass
 
-    @staticmethod
-    def empty_or_equal(lefts, rights):
-        return not lefts or not rights or all_equal(lefts, rights)
+    def empty_or_equal(self, other) -> bool:
+        for field in "selections", "sort_keys", "predicates":
+            selfs = getattr(self, field)
+            others = getattr(other, field)
+            valid = (
+                not selfs
+                or not others
+                or (a.equals(b) for a, b in zip(selfs, others))
+            )
+            if not valid:
+                return False
+        return True
 
     def compatible_with(self, other):
         # self and other are equivalent except for predicates, selections, or
@@ -492,11 +471,7 @@ class Selection(TableNode, sch.HasSchema):
         if not isinstance(other, type(self)):
             return False
 
-        return self.table.equals(other.table) and (
-            self.empty_or_equal(self.predicates, other.predicates)
-            and self.empty_or_equal(self.selections, other.selections)
-            and self.empty_or_equal(self.sort_keys, other.sort_keys)
-        )
+        return self.table.equals(other.table) and self.empty_or_equal(other)
 
     # Operator combination / fusion logic
 
@@ -517,7 +492,7 @@ class Selection(TableNode, sch.HasSchema):
                     self.table,
                     self.selections,
                     predicates=self.predicates,
-                    sort_keys=self.sort_keys + resolved_keys,
+                    sort_keys=self.sort_keys + tuple(resolved_keys),
                 )
 
         return Selection(expr, [], sort_keys=resolved_keys)
@@ -597,9 +572,9 @@ class Aggregation(TableNode, sch.HasSchema):
     where : pre-aggregation predicate
     """
 
-    table = Arg(rlz.table)
-    metrics = Arg(
-        rlz.list_of(
+    table = rlz.table
+    metrics = rlz.optional(
+        rlz.tuple_of(
             rlz.one_of(
                 (
                     rlz.function_of(
@@ -610,16 +585,16 @@ class Aggregation(TableNode, sch.HasSchema):
                     ),
                     rlz.reduction,
                     rlz.scalar(rlz.any),
-                    rlz.list_of(rlz.scalar(rlz.any)),
+                    rlz.tuple_of(rlz.scalar(rlz.any)),
                     rlz.named_literal,
                 )
             ),
             flatten=True,
         ),
-        default=[],
+        default=(),
     )
-    by = Arg(
-        rlz.list_of(
+    by = rlz.optional(
+        rlz.tuple_of(
             rlz.one_of(
                 (
                     rlz.function_of("table"),
@@ -628,10 +603,10 @@ class Aggregation(TableNode, sch.HasSchema):
                 )
             )
         ),
-        default=[],
+        default=(),
     )
-    having = Arg(
-        rlz.list_of(
+    having = rlz.optional(
+        rlz.tuple_of(
             rlz.one_of(
                 (
                     rlz.function_of(
@@ -641,11 +616,11 @@ class Aggregation(TableNode, sch.HasSchema):
                 )
             ),
         ),
-        default=[],
+        default=(),
     )
-    predicates = Arg(rlz.list_of(rlz.boolean), default=[])
-    sort_keys = Arg(
-        rlz.list_of(
+    predicates = rlz.optional(rlz.tuple_of(rlz.boolean), default=())
+    sort_keys = rlz.optional(
+        rlz.tuple_of(
             rlz.one_of(
                 (
                     rlz.column_from("table"),
@@ -675,26 +650,31 @@ class Aggregation(TableNode, sch.HasSchema):
                 )
             )
         ),
-        default=[],
+        default=(),
     )
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-
-        if not self.by:
-            self.sort_keys.clear()
-
-    def _validate(self):
+    def __init__(self, table, metrics, by, having, predicates, sort_keys):
         from ibis.expr.analysis import FilterValidator
 
         # All non-scalar refs originate from the input table
-        all_exprs = self.metrics + self.by + self.having + self.sort_keys
-        self.table._assert_valid(all_exprs)
+        all_exprs = metrics + by + having + sort_keys
+        table._assert_valid(all_exprs)
 
         # Validate predicates
-        validator = FilterValidator([self.table])
-        validator.validate_all(self.predicates)
+        validator = FilterValidator([table])
+        validator.validate_all(predicates)
 
+        if not by:
+            sort_keys = tuple()
+
+        super().__init__(
+            table=table,
+            metrics=metrics,
+            by=by,
+            having=having,
+            predicates=predicates,
+            sort_keys=sort_keys,
+        )
         # Validate schema has no overlapping columns
         assert self.schema
 
@@ -736,7 +716,7 @@ class Aggregation(TableNode, sch.HasSchema):
                 by=self.by,
                 having=self.having,
                 predicates=self.predicates,
-                sort_keys=self.sort_keys + resolved_keys,
+                sort_keys=self.sort_keys + tuple(resolved_keys),
             )
 
         return Selection(expr, [], sort_keys=resolved_keys)
@@ -756,13 +736,14 @@ class Distinct(TableNode, sch.HasSchema):
     FROM table
     """
 
-    table = Arg(rlz.table)
+    table = rlz.table
 
-    def _validate(self):
+    def __init__(self, table):
         # check whether schema has overlapping columns or not
-        assert self.schema
+        assert table.schema()
+        super().__init__(table=table)
 
-    @cached_property
+    @property
     def schema(self):
         return self.table.schema()
 
@@ -771,39 +752,48 @@ class Distinct(TableNode, sch.HasSchema):
 
 
 @public
-class ExistsSubquery(Node):
-    foreign_table = Arg(rlz.table)
-    predicates = Arg(rlz.list_of(rlz.boolean))
+class ExistsSubquery(ValueOp):
+    foreign_table = rlz.table
+    predicates = rlz.tuple_of(rlz.boolean)
 
-    def output_type(self):
-        return ir.ExistsExpr
+    output_dtype = dt.boolean
+    output_shape = rlz.Shape.COLUMNAR
+    output_type = ir.ExistsExpr
 
 
 @public
 class NotExistsSubquery(Node):
-    foreign_table = Arg(rlz.table)
-    predicates = Arg(rlz.list_of(rlz.boolean))
+    foreign_table = rlz.table
+    predicates = rlz.tuple_of(rlz.boolean)
 
-    def output_type(self):
-        return ir.ExistsExpr
+    output_type = ir.ExistsExpr
 
 
 @public
 class FillNa(TableNode, sch.HasSchema):
     """Fill null values in the table."""
 
-    table = Arg(rlz.table)
-    replacements = Arg(
-        rlz.one_of(
-            (
-                rlz.numeric,
-                rlz.string,
-                rlz.instance_of(collections.abc.Mapping),
-            )
+    table = rlz.table
+    replacements = rlz.one_of(
+        (
+            rlz.numeric,
+            rlz.string,
+            rlz.instance_of(collections.abc.Mapping),
         )
     )
 
-    @cached_property
+    def __init__(self, table, replacements, **kwargs):
+        super().__init__(
+            table=table,
+            replacements=(
+                replacements
+                if not isinstance(replacements, collections.abc.Mapping)
+                else util.frozendict(replacements)
+            ),
+            **kwargs,
+        )
+
+    @property
     def schema(self):
         return self.table.schema()
 
@@ -812,10 +802,69 @@ class FillNa(TableNode, sch.HasSchema):
 class DropNa(TableNode, sch.HasSchema):
     """Drop null values in the table."""
 
-    table = Arg(rlz.table)
-    how = Arg(rlz.isin({'any', 'all'}))
-    subset = Arg(rlz.list_of(rlz.column_from("table")), default=[])
+    table = rlz.table
+    how = rlz.isin({'any', 'all'})
+    subset = rlz.optional(rlz.tuple_of(rlz.column_from("table")), default=())
+
+    @property
+    def schema(self):
+        return self.table.schema()
+
+
+@public
+class View(PhysicalTable):
+    """A view created from an expression."""
+
+    child = rlz.table
+    name = rlz.instance_of(str)
+
+    @property
+    def schema(self):
+        return self.child.schema()
+
+
+@public
+class SQLStringView(PhysicalTable):
+    """A view created from a SQL string."""
+
+    child = rlz.table
+    name = rlz.instance_of(str)
+    query = rlz.instance_of(str)
 
     @cached_property
     def schema(self):
-        return self.table.schema()
+        backend = self.child._find_backend()
+        return backend._get_schema_using_query(self.query)
+
+
+def _dedup_join_columns(
+    expr: ir.TableExpr,
+    *,
+    left: ir.TableExpr,
+    right: ir.TableExpr,
+    suffixes: tuple[str, str],
+):
+    right_columns = frozenset(right.columns)
+    overlap = frozenset(
+        column for column in left.columns if column in right_columns
+    )
+
+    if not overlap:
+        return expr
+
+    left_suffix, right_suffix = suffixes
+
+    left_projections = [
+        left[column].name(f"{column}{left_suffix}")
+        if column in overlap
+        else left[column]
+        for column in left.columns
+    ]
+
+    right_projections = [
+        right[column].name(f"{column}{right_suffix}")
+        if column in overlap
+        else right[column]
+        for column in right.columns
+    ]
+    return expr.projection(left_projections + right_projections)
