@@ -1,27 +1,31 @@
 import enum
 import functools
-from contextlib import suppress
 from itertools import product, starmap
 
-from toolz import compose, curry, identity
+from toolz import identity
 
 import ibis.common.exceptions as com
 import ibis.expr.datatypes as dt
 import ibis.expr.schema as sch
-import ibis.expr.signature as sig
 import ibis.expr.types as ir
 import ibis.util as util
-
-optional = sig.Optional
+from ibis.common.validators import (  # noqa: F401
+    immutable_property,
+    instance_of,
+    isin,
+    list_of,
+    map_to,
+    one_of,
+    optional,
+    tuple_of,
+    validator,
+)
 
 
-class validator(curry, sig.Validator):
-    def __repr__(self):
-        return '{}({}{})'.format(
-            self.func.__name__,
-            repr(self.args)[1:-1],
-            ', '.join(f'{k}={v!r}' for k, v in self.keywords.items()),
-        )
+class Shape(enum.IntEnum):
+    SCALAR = 0
+    COLUMNAR = 1
+    # TABULAR = 2
 
 
 def highest_precedence_dtype(exprs):
@@ -61,82 +65,10 @@ def comparable(left, right):
     return castable(left, right) or castable(right, left)
 
 
-def cast(source, target):
-    """Currently Literal to *Scalar implicit casts are allowed"""
-    import ibis.expr.operations as ops  # TODO: don't use ops here
-
-    if not castable(source, target):
-        raise com.IbisTypeError('Source is not castable to target type!')
-
-    # currently it prevents column -> scalar implicit castings
-    # however the datatypes are matching
-    op = source.op()
-    if not isinstance(op, ops.Literal):
-        raise com.IbisTypeError('Only able to implicitly cast literals!')
-
-    out_type = target.type().scalar_type()
-    return out_type(op)
-
-
 # ---------------------------------------------------------------------
 # Input type validators / coercion functions
 
 
-@validator
-def noop(arg, **kwargs):
-    return arg
-
-
-@validator
-def one_of(inners, arg, **kwargs):
-    """At least one of the inner validators must pass"""
-    for inner in inners:
-        with suppress(com.IbisTypeError, ValueError):
-            return inner(arg, **kwargs)
-
-    raise com.IbisTypeError(
-        "argument passes none of the following rules: "
-        f"{', '.join(map(repr, inners))}"
-    )
-
-
-@validator
-def all_of(inners, arg, *, this):
-    """All of the inner validators must pass.
-
-    The order of inner validators matters.
-
-    Parameters
-    ----------
-    inners : List[validator]
-      Functions are applied from right to left so allof([rule1, rule2], arg) is
-      the same as rule1(rule2(arg)).
-    arg : Any
-      Value to be validated.
-
-    Returns
-    -------
-    arg : Any
-      Value maybe coerced by inner validators to the appropiate types
-    """
-    return compose(*inners)(arg, this=this)
-
-
-@validator
-def isin(values, arg, **kwargs):
-    if arg not in values:
-        raise ValueError(f'Value with type {type(arg)} is not in {values!r}')
-    if isinstance(values, dict):  # TODO check for mapping instead
-        return values[arg]
-    else:
-        return arg
-
-
-@validator
-def map_to(mapping, variant, **kwargs):
-    return mapping[variant]
-
-
 @validator
 def member_of(obj, arg, **kwargs):
     if isinstance(arg, ir.EnumValue):
@@ -152,26 +84,6 @@ def member_of(obj, arg, **kwargs):
     return getattr(obj, arg)
 
 
-@validator
-def container_of(inner, arg, *, type, min_length=0, flatten=False, **kwargs):
-    if not util.is_iterable(arg):
-        raise com.IbisTypeError('Argument must be a sequence')
-
-    if len(arg) < min_length:
-        raise com.IbisTypeError(
-            f'Arg must have at least {min_length} number of elements'
-        )
-
-    if flatten:
-        arg = util.flatten_iterable(arg)
-
-    return type(inner(item, **kwargs) for item in arg)
-
-
-list_of = container_of(type=list)
-tuple_of = container_of(type=tuple)
-
-
 @validator
 def value_list_of(inner, arg, **kwargs):
     # TODO(kszucs): would be nice to remove ops.ValueList
@@ -180,7 +92,7 @@ def value_list_of(inner, arg, **kwargs):
     # the dispatcher in pandas requires operation objects
     import ibis.expr.operations as ops
 
-    values = list_of(inner, arg, **kwargs)
+    values = tuple_of(inner, arg, **kwargs)
     return ops.ValueList(values).to_expr()
 
 
@@ -188,7 +100,7 @@ def value_list_of(inner, arg, **kwargs):
 def sort_key(key, *, from_=None, this):
     import ibis.expr.operations as ops
 
-    table = getattr(this, from_) if from_ is not None else None
+    table = this[from_] if from_ is not None else None
     return ops.sortkeys._to_sort_key(key, table=table)
 
 
@@ -197,22 +109,6 @@ def datatype(arg, **kwargs):
     return dt.dtype(arg)
 
 
-@validator
-def instance_of(klasses, arg, **kwargs):
-    """Require that a value has a particular Python type."""
-    if not isinstance(arg, klasses):
-        raise com.IbisTypeError(
-            f'Given argument with type {type(arg)} '
-            f'is not an instance of {klasses}'
-        )
-    return arg
-
-
-@validator
-def coerce_to(klass, arg, **kwargs):
-    return klass(arg)
-
-
 @validator
 def value(dtype, arg, **kwargs):
     """Validates that the given argument is a Value with a particular datatype
@@ -345,41 +241,71 @@ def promoter(fn):
     return wrapper
 
 
-@promoter
-def shape_like(arg, dtype=None):
-    if isinstance(arg, (tuple, list, ir.ListExpr)):
-        datatype = dtype or highest_precedence_dtype(arg)
-        columnar = util.any_of(arg, ir.AnyColumn)
-    else:
-        datatype = dtype or arg.type()
-        columnar = isinstance(arg, ir.AnyColumn)
+def dtype_like(name):
+    @immutable_property
+    def output_dtype(self):
+        arg = getattr(self, name)
+        if util.is_iterable(arg):
+            return highest_precedence_dtype(arg)
+        else:
+            return arg.type()
 
-    dtype = dt.dtype(datatype)
-
-    if columnar:
-        return dtype.column_type()
-    else:
-        return dtype.scalar_type()
+    return output_dtype
 
 
-@promoter
-def scalar_like(arg):
-    output_dtype = arg.type()
-    return output_dtype.scalar_type()
+def shape_like(name):
+    @immutable_property
+    def output_shape(self):
+        arg = getattr(self, name)
+        if util.is_iterable(arg):
+            for expr in arg:
+                try:
+                    op = expr.op()
+                except AttributeError:
+                    continue
+                if op.output_shape is Shape.COLUMNAR:
+                    return Shape.COLUMNAR
+            return Shape.SCALAR
+        else:
+            return arg.op().output_shape
+
+    return output_shape
 
 
-@promoter
-def array_like(arg):
-    output_dtype = arg.type()
-    return output_dtype.column_type()
+# TODO(kszucs): might just use bounds instead of actual literal values
+# that could simplify interval binop output_type methods
+# TODO(kszucs): pre-generate mapping?
+def _promote_numeric_binop(exprs, op):
+    bounds, dtypes = [], []
+    for arg in exprs:
+        dtypes.append(arg.type())
+        if hasattr(arg.op(), 'value'):
+            # arg.op() is a literal
+            bounds.append([arg.op().value])
+        else:
+            bounds.append(arg.type().bounds)
+
+    # In some cases, the bounding type might be int8, even though neither
+    # of the types are that small. We want to ensure the containing type is
+    # _at least_ as large as the smallest type in the expression.
+    values = starmap(op, product(*bounds))
+    dtypes += [dt.infer(value) for value in values]
+
+    return dt.highest_precedence(dtypes)
 
 
-column_like = array_like
+def numeric_like(name, op):
+    @immutable_property
+    def output_dtype(self):
+        args = getattr(self, name)
+        if util.all_of(args, ir.IntegerValue):
+            result = _promote_numeric_binop(args, op)
+        else:
+            result = highest_precedence_dtype(args)
 
+        return result
 
-@promoter
-def typeof(arg):
-    return arg._factory
+    return output_dtype
 
 
 @validator
@@ -427,9 +353,9 @@ def column_from(name, column, *, this):
     checks if the column in the table is equal to the column being
     passed.
     """
-    if not hasattr(this, name):
+    if name not in this:
         raise com.IbisTypeError(f"Could not get table {name} from {this}")
-    table = getattr(this, name)
+    table = this[name]
 
     if isinstance(column, (str, int)):
         return table[column]
@@ -443,11 +369,11 @@ def column_from(name, column, *, this):
                 return column
             else:
                 raise com.IbisTypeError(
-                    f"Passed column is not a column in {table}"
+                    f"Passed column is not a column in {type(table)}"
                 )
         except com.IbisError:
             raise com.IbisTypeError(
-                f"Cannot get column {maybe_column} from {table}"
+                f"Cannot get column {maybe_column} from {type(table)}"
             )
 
     raise com.IbisTypeError(
@@ -468,7 +394,7 @@ def function_of(
     if not util.is_function(fn):
         raise com.IbisTypeError('argument `fn` must be a function or lambda')
 
-    return output_rule(fn(preprocess(getattr(this, argument))), this=this)
+    return output_rule(fn(preprocess(this[argument])), this=this)
 
 
 @validator
@@ -569,7 +495,7 @@ def window(win, *, from_base_table_of, this):
             "`win` argument should be of type `ibis.expr.window.Window`; "
             f"got type {type(win).__name__}"
         )
-    table = ir.find_base_table(getattr(this, from_base_table_of))
+    table = ir.relations.find_base_table(this[from_base_table_of])
     if table is not None:
         win = win.bind(table)
 
@@ -585,34 +511,4 @@ def window(win, *, from_base_table_of, this):
     return win
 
 
-# TODO: might just use bounds instead of actual literal values
-# that could simplify interval binop output_type methods
-def _promote_numeric_binop(exprs, op):
-    bounds, dtypes = [], []
-    for arg in exprs:
-        dtypes.append(arg.type())
-        if hasattr(arg.op(), 'value'):
-            # arg.op() is a literal
-            bounds.append([arg.op().value])
-        else:
-            bounds.append(arg.type().bounds)
-
-    # In some cases, the bounding type might be int8, even though neither
-    # of the types are that small. We want to ensure the containing type is
-    # _at least_ as large as the smallest type in the expression.
-    values = starmap(op, product(*bounds))
-    dtypes += [dt.infer(value, allow_overflow=True) for value in values]
-
-    return dt.highest_precedence(dtypes)
-
-
-@promoter
-def numeric_like(args, op):
-    if util.all_of(args, ir.IntegerValue):
-        dtype = _promote_numeric_binop(args, op)
-        return shape_like(args, dtype=dtype)
-    else:
-        return shape_like(args)
-
-
 # TODO: create varargs marker for impala udfs
