@@ -1,22 +1,27 @@
-"""Encapsulation of SQL window clauses."""
+"""Encapsulation of SQL window frames."""
+
+from __future__ import annotations
 
 import functools
-from typing import NamedTuple, Union
+from typing import NamedTuple
 
 import numpy as np
 import pandas as pd
+import toolz
+from cached_property import cached_property
 
-import ibis.common.exceptions as com
-import ibis.expr.types as ir
-import ibis.util as util
+from ibis.common.exceptions import IbisInputError
+from ibis.common.grounds import Comparable
+from ibis.expr import types as ir
+from ibis.util import is_iterable, promote_list
 
 
 def _sequence_to_tuple(x):
-    return tuple(x) if util.is_iterable(x) else x
+    return tuple(x) if is_iterable(x) else x
 
 
 class RowsWithMaxLookback(NamedTuple):
-    rows: Union[int, np.integer]
+    rows: int | np.integer
     max_lookback: ir.IntervalValue
 
 
@@ -70,7 +75,7 @@ def get_preceding_value_simple(preceding):
 
 
 @get_preceding_value.register(RowsWithMaxLookback)
-def get_preceding_value_mlb(preceding):
+def get_preceding_value_mlb(preceding: RowsWithMaxLookback):
     preceding_value = preceding.rows
     if not isinstance(preceding_value, (int, np.integer)):
         raise TypeError(
@@ -80,16 +85,16 @@ def get_preceding_value_mlb(preceding):
     return preceding_value
 
 
-class Window:
-    """Class to encapsulate the details of a window frame.
+class Window(Comparable):
+    """A window frame.
 
     Notes
     -----
-    This class is patterned after SQL window clauses.
+    This class is patterned after SQL window frame clauses.
 
-    Using None for preceding or following currently indicates unbounded. Use 0
-    for ``CURRENT ROW``.
+    Using `None` for `preceding` or `following` indicates an unbounded frame.
 
+    Use 0 for `CURRENT ROW`.
     """
 
     def __init__(
@@ -103,21 +108,24 @@ class Window:
     ):
         import ibis.expr.operations as ops
 
-        if group_by is None:
-            group_by = []
+        self._group_by = tuple(
+            toolz.unique(
+                promote_list([] if group_by is None else group_by),
+                key=lambda value: getattr(value, "_key", value),
+            )
+        )
 
-        if order_by is None:
-            order_by = []
+        _order_by = []
+        for expr in promote_list([] if order_by is None else order_by):
+            if isinstance(expr, ir.Expr) and not isinstance(expr, ir.SortExpr):
+                expr = ops.SortKey(expr).to_expr()
+            _order_by.append(expr)
 
-        self._group_by = util.promote_list(group_by)
-
-        self._order_by = []
-        for x in util.promote_list(order_by):
-            if isinstance(x, ir.SortExpr):
-                pass
-            elif isinstance(x, ir.Expr):
-                x = ops.SortKey(x).to_expr()
-            self._order_by.append(x)
+        self._order_by = tuple(
+            toolz.unique(
+                _order_by, key=lambda value: getattr(value, "_key", value)
+            )
+        )
 
         if isinstance(preceding, RowsWithMaxLookback):
             # the offset interval is used as the 'preceding' value of a window
@@ -133,17 +141,30 @@ class Window:
 
         self._validate_frame()
 
-    def __hash__(self) -> int:
+    @cached_property
+    def _hash(self) -> int:
         return hash(
             (
-                tuple(gb.op() for gb in self._group_by),
-                tuple(ob.op() for ob in self._order_by),
-                self.preceding,
-                self.following,
+                *(gb.op() for gb in self._group_by),
+                *(ob.op() for ob in self._order_by),
+                (
+                    self.preceding.op()
+                    if isinstance(self.preceding, ir.Expr)
+                    else self.preceding
+                ),
+                (
+                    self.following.op()
+                    if isinstance(self.following, ir.Expr)
+                    else self.following
+                ),
                 self.how,
+                self.max_lookback,
             )
         )
 
+    def __hash__(self) -> int:
+        return self._hash
+
     def _validate_frame(self):
         preceding_tuple = has_preceding = False
         following_tuple = has_following = False
@@ -158,50 +179,48 @@ class Window:
         if (preceding_tuple and has_following) or (
             following_tuple and has_preceding
         ):
-            raise com.IbisInputError(
+            raise IbisInputError(
                 'Can only specify one window side when you want an '
                 'off-center window'
             )
         elif preceding_tuple:
             start, end = self.preceding
             if end is None:
-                raise com.IbisInputError("preceding end point cannot be None")
+                raise IbisInputError("preceding end point cannot be None")
             if end < 0:
-                raise com.IbisInputError(
+                raise IbisInputError(
                     "preceding end point must be non-negative"
                 )
             if start is not None:
                 if start < 0:
-                    raise com.IbisInputError(
+                    raise IbisInputError(
                         "preceding start point must be non-negative"
                     )
                 if start <= end:
-                    raise com.IbisInputError(
+                    raise IbisInputError(
                         "preceding start must be greater than preceding end"
                     )
         elif following_tuple:
             start, end = self.following
             if start is None:
-                raise com.IbisInputError(
-                    "following start point cannot be None"
-                )
+                raise IbisInputError("following start point cannot be None")
             if start < 0:
-                raise com.IbisInputError(
+                raise IbisInputError(
                     "following start point must be non-negative"
                 )
             if end is not None:
                 if end < 0:
-                    raise com.IbisInputError(
+                    raise IbisInputError(
                         "following end point must be non-negative"
                     )
                 if start >= end:
-                    raise com.IbisInputError(
+                    raise IbisInputError(
                         "following start must be less than following end"
                     )
         else:
             if not isinstance(self.preceding, ir.Expr):
                 if has_preceding and self.preceding < 0:
-                    raise com.IbisInputError(
+                    raise IbisInputError(
                         "'preceding' must be positive, got {}".format(
                             self.preceding
                         )
@@ -209,13 +228,13 @@ class Window:
 
             if not isinstance(self.following, ir.Expr):
                 if has_following and self.following < 0:
-                    raise com.IbisInputError(
+                    raise IbisInputError(
                         "'following' must be positive, got {}".format(
                             self.following
                         )
                     )
         if self.how not in {'rows', 'range'}:
-            raise com.IbisInputError(
+            raise IbisInputError(
                 f"'how' must be 'rows' or 'range', got {self.how}"
             )
 
@@ -223,7 +242,7 @@ class Window:
             if not isinstance(
                 self.preceding, (ir.IntervalValue, pd.Timedelta)
             ):
-                raise com.IbisInputError(
+                raise IbisInputError(
                     "'max_lookback' must be specified as an interval "
                     "or pandas.Timedelta object"
                 )
@@ -241,28 +260,22 @@ class Window:
 
     def combine(self, window):
         if self.how != window.how:
-            raise com.IbisInputError(
-                (
-                    "Window types must match. "
-                    "Expecting '{}' Window, got '{}'"
-                ).format(self.how.upper(), window.how.upper())
+            raise IbisInputError(
+                "Window types must match. "
+                f"Expecting {self.how!r} window, got {window.how!r}"
             )
 
-        kwds = {
-            'preceding': _choose_non_empty_val(
-                self.preceding, window.preceding
-            ),
-            'following': _choose_non_empty_val(
-                self.following, window.following
-            ),
-            'max_lookback': self.max_lookback or window.max_lookback,
-            'group_by': self._group_by + window._group_by,
-            'order_by': self._order_by + window._order_by,
-        }
-        return Window(**kwds)
+        return Window(
+            preceding=_choose_non_empty_val(self.preceding, window.preceding),
+            following=_choose_non_empty_val(self.following, window.following),
+            group_by=self._group_by + window._group_by,
+            order_by=self._order_by + window._order_by,
+            max_lookback=self.max_lookback or window.max_lookback,
+            how=self.how,
+        )
 
     def group_by(self, expr):
-        new_groups = self._group_by + util.promote_list(expr)
+        new_groups = self._group_by + tuple(promote_list(expr))
         return self._replace(group_by=new_groups)
 
     def _replace(self, **kwds):
@@ -277,82 +290,86 @@ class Window:
         return Window(**new_kwds)
 
     def order_by(self, expr):
-        new_sorts = self._order_by + util.promote_list(expr)
+        new_sorts = self._order_by + tuple(promote_list(expr))
         return self._replace(order_by=new_sorts)
 
-    def equals(self, other, cache=None):
-        import ibis.expr.operations as ops
-
-        if cache is None:
-            cache = {}
-
-        if self is other:
-            cache[self, other] = True
-            return True
-
-        if not isinstance(other, Window):
-            cache[self, other] = False
-            return False
-
-        try:
-            return cache[self, other]
-        except KeyError:
-            pass
-
-        if len(self._group_by) != len(other._group_by) or not ops.all_equal(
-            self._group_by, other._group_by, cache=cache
-        ):
-            cache[self, other] = False
-            return False
-
-        if len(self._order_by) != len(other._order_by) or not ops.all_equal(
-            self._order_by, other._order_by, cache=cache
-        ):
-            cache[self, other] = False
-            return False
-
-        equal = (
-            ops.all_equal(self.preceding, other.preceding, cache=cache)
-            and ops.all_equal(self.following, other.following, cache=cache)
-            and ops.all_equal(
-                self.max_lookback, other.max_lookback, cache=cache
+    def __equals__(self, other):
+        return (
+            len(self._group_by) == len(other._group_by)
+            and len(self._order_by) == len(other._order_by)
+            and self.max_lookback == other.max_lookback
+            and (
+                self.preceding.equals(other.preceding)
+                if isinstance(self.preceding, ir.Expr)
+                else self.preceding == other.preceding
+            )
+            and (
+                self.following.equals(other.following)
+                if isinstance(self.following, ir.Expr)
+                else self.following == other.following
+            )
+            and all(
+                a.equals(b) for a, b in zip(self._group_by, other._group_by)
+            )
+            and all(
+                a.equals(b) for a, b in zip(self._order_by, other._order_by)
             )
         )
-        cache[self, other] = equal
-        return equal
+
+    def equals(self, other):
+        if not isinstance(other, Window):
+            raise TypeError(
+                "invalid equality comparison between Window and "
+                f"{type(other)}"
+            )
+        return self.__cached_equals__(other)
 
 
-def rows_with_max_lookback(rows, max_lookback):
-    """Create a bound preceding value for use with trailing window functions"""
+def rows_with_max_lookback(
+    rows: int | np.integer,
+    max_lookback: ir.IntervalValue,
+) -> RowsWithMaxLookback:
+    """Create a bound preceding value for use with trailing window functions.
+
+    Parameters
+    ----------
+    rows
+        Number of rows
+    max_lookback
+        Maximum lookback in time
+
+    Returns
+    -------
+    RowsWithMaxLookback
+        A named tuple of rows and maximum look-back in time
+    """
     return RowsWithMaxLookback(rows, max_lookback)
 
 
 def window(preceding=None, following=None, group_by=None, order_by=None):
     """Create a window clause for use with window functions.
 
-    This ROW window clause aggregates adjacent rows based on differences in row
-    number.
+    The `ROWS` window clause includes peer rows based on differences in row
+    **number** whereas `RANGE` includes rows based on the differences in row
+    **value** of a single `order_by` expression.
 
-    All window frames / ranges are inclusive.
+    All window frame bounds are inclusive.
 
     Parameters
     ----------
-    preceding : int, tuple, or None, default None
-        Specify None for unbounded, 0 to include current row tuple for
-        off-center window
-    following : int, tuple, or None, default None
-        Specify None for unbounded, 0 to include current row tuple for
-        off-center window
-    group_by : expressions, default None
-        Either specify here or with TableExpr.group_by
-    order_by : expressions, default None
-        For analytic functions requiring an ordering, specify here, or let Ibis
-        determine the default ordering (for functions like rank)
+    preceding
+        Number of preceding rows in the window
+    following
+        Number of following rows in the window
+    group_by
+        Grouping key
+    order_by
+        Ordering key
 
     Returns
     -------
     Window
-
+        A window frame
     """
     return Window(
         preceding=preceding,
@@ -373,22 +390,19 @@ def range_window(preceding=None, following=None, group_by=None, order_by=None):
 
     Parameters
     ----------
-    preceding : int, tuple, or None, default None
-        Specify None for unbounded, 0 to include current row tuple for
-        off-center window
-    following : int, tuple, or None, default None
-        Specify None for unbounded, 0 to include current row tuple for
-        off-center window
-    group_by : expressions, default None
-        Either specify here or with TableExpr.group_by
-    order_by : expressions, default None
-        For analytic functions requiring an ordering, specify here, or let Ibis
-        determine the default ordering (for functions like rank)
+    preceding
+        Number of preceding rows in the window
+    following
+        Number of following rows in the window
+    group_by
+        Grouping key
+    order_by
+        Ordering key
 
     Returns
     -------
     Window
-
+        A window frame
     """
     return Window(
         preceding=preceding,
@@ -399,23 +413,22 @@ def range_window(preceding=None, following=None, group_by=None, order_by=None):
     )
 
 
-def cumulative_window(group_by=None, order_by=None):
-    """Create a cumulative window for use with aggregate window functions.
+def cumulative_window(group_by=None, order_by=None) -> Window:
+    """Create a cumulative window for use with window functions.
 
     All window frames / ranges are inclusive.
 
     Parameters
     ----------
-    group_by : expressions, default None
-        Either specify here or with TableExpr.group_by
-    order_by : expressions, default None
-        For analytic functions requiring an ordering, specify here, or let Ibis
-        determine the default ordering (for functions like rank)
+    group_by
+        Grouping key
+    order_by
+        Ordering key
 
     Returns
     -------
     Window
-
+        A window frame
     """
     return Window(
         preceding=None, following=0, group_by=group_by, order_by=order_by
@@ -427,22 +440,17 @@ def trailing_window(preceding, group_by=None, order_by=None):
 
     Parameters
     ----------
-    preceding : int, float or expression of intervals, i.e.
-        ibis.interval(days=1) + ibis.interval(hours=5)
-        Int indicates number of trailing rows to include;
-        0 includes only the current row, 1 includes the current row and one
-        preceding row.
-        Interval indicates a trailing range window.
-    group_by : expressions, default None
-        Either specify here or with TableExpr.group_by
-    order_by : expressions, default None
-        For analytic functions requiring an ordering, specify here, or let Ibis
-        determine the default ordering (for functions like rank)
+    preceding
+        The number of preceding rows
+    group_by
+        Grouping key
+    order_by
+        Ordering key
 
     Returns
     -------
     Window
-
+        A window frame
     """
     how = _determine_how(preceding)
     return Window(
@@ -454,22 +462,22 @@ def trailing_window(preceding, group_by=None, order_by=None):
     )
 
 
-def trailing_range_window(preceding, order_by, group_by=None):
-    """Create a trailing time window for use with aggregate window functions.
+def trailing_range_window(preceding, order_by, group_by=None) -> Window:
+    """Create a trailing range window for use with window functions.
 
     Parameters
     ----------
-    preceding : float or expression of intervals, i.e.
-        ibis.interval(days=1) + ibis.interval(hours=5)
-    order_by : expressions, default None
-        For analytic functions requiring an ordering, specify here, or let Ibis
-        determine the default ordering (for functions like rank)
-    group_by : expressions, default None
-        Either specify here or with TableExpr.group_by
+    preceding
+        A value expression
+    order_by
+        Ordering key
+    group_by
+        Grouping key
 
     Returns
     -------
     Window
+        A window frame
 
     """
     return Window(
@@ -481,7 +489,7 @@ def trailing_range_window(preceding, order_by, group_by=None):
     )
 
 
-def propagate_down_window(expr, window):
+def propagate_down_window(expr: ir.ValueExpr, window: Window):
     import ibis.expr.operations as ops
 
     op = expr.op()
