@@ -75,13 +75,13 @@ def test_view_new_relation(table):
     assert roots[0] is tview.op()
 
 
-def test_get_type(table, schema_dict):
-    for k, v in schema_dict.items():
+def test_get_type(table, schema):
+    for k, v in schema:
         assert table._get_type(k) == dt.dtype(v)
 
 
-def test_getitem_column_select(table, schema_dict):
-    for k, v in schema_dict.items():
+def test_getitem_column_select(table):
+    for k in table.columns:
         col = table[k]
 
         # Make sure it's the right type
@@ -112,7 +112,7 @@ def test_projection(table):
     assert isinstance(proj, TableExpr)
     assert isinstance(proj.op(), ops.Selection)
 
-    assert proj.schema().names == cols
+    assert proj.schema().names == tuple(cols)
     for c in cols:
         expr = proj[c]
         assert isinstance(expr, type(table[c]))
@@ -133,8 +133,8 @@ def test_projection_with_exprs(table):
 
     proj = table[col_exprs + ['g']]
     schema = proj.schema()
-    assert schema.names == ['log_b', 'mean_diff', 'g']
-    assert schema.types == [dt.double, dt.double, dt.string]
+    assert schema.names == ('log_b', 'mean_diff', 'g')
+    assert schema.types == (dt.double, dt.double, dt.string)
 
     # Test with unnamed expr
     with pytest.raises(ExpressionError):
@@ -169,14 +169,6 @@ def test_projection_unnamed_literal_interactive_blowup(con):
             assert 'named' in e.args[0]
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_projection_of_aggregated(table):
-    # Fully-formed aggregations "block"; in a projection, column
-    # expressions referencing table expressions below the aggregation are
-    # invalid.
-    assert False
-
-
 def test_projection_with_star_expr(table):
     new_expr = (table['a'] * 5).name('bigger_a')
 
@@ -186,7 +178,7 @@ def test_projection_with_star_expr(table):
     proj = t[t, new_expr]
     repr(proj)
 
-    ex_names = table.schema().names + ['bigger_a']
+    ex_names = table.schema().names + ('bigger_a',)
     assert proj.schema().names == ex_names
 
     # cannot pass an invalid table expression
@@ -228,20 +220,6 @@ def test_projection_array_expr(table):
     assert_equal(result, expected)
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_add_column_aggregate_crossjoin(table):
-    # A new column that depends on a scalar value produced by this or some
-    # other table.
-    #
-    # For example:
-    # SELECT *, b - VAL
-    # FROM table1
-    #
-    # Here, VAL could be something produced by aggregating table1 or any
-    # other table for that matter.
-    assert False
-
-
 def test_mutate(table):
     one = table.f * 2
     foo = (table.a + table.b).name('foo')
@@ -408,6 +386,17 @@ def test_sort_by_desc_deferred_sort_key(table):
     assert_equal(result, expected2)
 
 
+def test_sort_by_asc_deferred_sort_key(table):
+    result = table.group_by('g').size().sort_by(ibis.asc('count'))
+
+    tmp = table.group_by('g').size()
+    expected = tmp.sort_by(tmp['count'])
+    expected2 = tmp.sort_by(ibis.asc(tmp['count']))
+
+    assert_equal(result, expected)
+    assert_equal(result, expected2)
+
+
 def test_slice_convenience(table):
     expr = table[:5]
     expr2 = table[:5:1]
@@ -435,7 +424,8 @@ def test_slice_convenience(table):
 def test_table_count(table):
     result = table.count()
     assert isinstance(result, ir.IntegerScalar)
-    assert isinstance(result.op(), ops.Count)
+    assert isinstance(result.op(), ops.Alias)
+    assert isinstance(result.op().arg.op(), ops.Count)
     assert result.get_name() == 'count'
 
 
@@ -446,24 +436,28 @@ def test_len_raises_expression_error(table):
 
 def test_sum_expr_basics(table, int_col):
     # Impala gives bigint for all integer types
-    ex_class = ir.IntegerScalar
     result = table[int_col].sum()
-    assert isinstance(result, ex_class)
-    assert isinstance(result.op(), ops.Sum)
+    assert isinstance(result, ir.IntegerScalar)
+    assert isinstance(result.op(), ops.Alias)
+    assert isinstance(result.op().arg.op(), ops.Sum)
+    assert result.get_name() == "sum"
 
 
 def test_sum_expr_basics_floats(table, float_col):
     # Impala gives double for all floating point types
-    ex_class = ir.FloatingScalar
     result = table[float_col].sum()
-    assert isinstance(result, ex_class)
-    assert isinstance(result.op(), ops.Sum)
+    assert isinstance(result, ir.FloatingScalar)
+    assert isinstance(result.op(), ops.Alias)
+    assert isinstance(result.op().arg.op(), ops.Sum)
+    assert result.get_name() == "sum"
 
 
 def test_mean_expr_basics(table, numeric_col):
     result = table[numeric_col].mean()
     assert isinstance(result, ir.FloatingScalar)
-    assert isinstance(result.op(), ops.Mean)
+    assert isinstance(result.op(), ops.Alias)
+    assert isinstance(result.op().arg.op(), ops.Mean)
+    assert result.get_name() == "mean"
 
 
 def test_aggregate_no_keys(table):
@@ -510,7 +504,7 @@ def test_aggregate_keywords(table):
     expr = t.aggregate(foo=t.f.sum(), bar=lambda x: x.f.mean(), by='g')
     expr2 = t.group_by('g').aggregate(foo=t.f.sum(), bar=lambda x: x.f.mean())
     expected = t.aggregate(
-        [t.f.mean().name('bar'), t.f.sum().name('foo')], by='g'
+        [t.f.sum().name('foo'), t.f.mean().name('bar')], by='g'
     )
 
     assert_equal(expr, expected)
@@ -574,12 +568,6 @@ def test_summary_prefix_suffix(table):
     ]
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_aggregate_invalid(table):
-    # Pass a non-aggregation or non-scalar expr
-    assert False
-
-
 def test_filter_aggregate_pushdown_predicate(table):
     # In the case where we want to add a predicate to an aggregate
     # expression after the fact, rather than having to backpedal and add it
@@ -598,11 +586,6 @@ def test_filter_aggregate_pushdown_predicate(table):
     assert_equal(filtered, expected)
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_filter_aggregate_partial_pushdown(table):
-    assert False
-
-
 @pytest.mark.parametrize(
     "case_fn",
     [
@@ -642,11 +625,6 @@ def test_group_by_kwargs(table):
     assert_equal(expr, expected)
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_aggregate_root_table_internal(table):
-    assert False
-
-
 def test_compound_aggregate_expr(table):
     # See ibis #24
     compound_expr = (table['a'].sum() / table['a'].mean()).name('foo')
@@ -759,7 +737,14 @@ def test_asof_join():
     left = ibis.table([('time', 'int32'), ('value', 'double')])
     right = ibis.table([('time', 'int32'), ('value2', 'double')])
     joined = api.asof_join(left, right, 'time')
-    pred = joined.op().predicates[0].op()
+
+    assert joined.columns == [
+        "time_x",
+        "value",
+        "time_y",
+        "value2",
+    ]
+    pred = joined.op().table.op().predicates[0].op()
     assert pred.left.op().name == pred.right.op().name == 'time'
 
 
@@ -771,7 +756,15 @@ def test_asof_join_with_by():
         [('time', 'int32'), ('key', 'int32'), ('value2', 'double')]
     )
     joined = api.asof_join(left, right, 'time', by='key')
-    by = joined.op().by[0].op()
+    assert joined.columns == [
+        "time_x",
+        "key_x",
+        "value",
+        "time_y",
+        "key_y",
+        "value2",
+    ]
+    by = joined.op().table.op().by[0].op()
     assert by.left.op().name == by.right.op().name == 'key'
 
 
@@ -802,11 +795,11 @@ def test_asof_join_with_tolerance(ibis_interval, timedelta_interval):
     )
 
     joined = api.asof_join(left, right, 'time', tolerance=ibis_interval)
-    tolerance = joined.op().tolerance
+    tolerance = joined.op().table.op().tolerance
     assert_equal(tolerance, ibis_interval)
 
     joined = api.asof_join(left, right, 'time', tolerance=timedelta_interval)
-    tolerance = joined.op().tolerance
+    tolerance = joined.op().table.op().tolerance
     assert isinstance(tolerance, ir.IntervalScalar)
     assert isinstance(tolerance.op(), ops.Literal)
 
@@ -825,7 +818,7 @@ def test_equijoin_schema_merge():
 
     for fname in join_types:
         f = getattr(table1, fname)
-        joined = f(table2, [pred]).materialize()
+        joined = f(table2, [pred])
         assert_equal(joined.schema(), ex_schema)
 
 
@@ -875,14 +868,6 @@ def test_self_join(table):
     metric = (left['a'] - right['b']).mean().name('metric')
 
     joined = left.inner_join(right, [right['g'] == left['g']])
-    # basic check there's no referential problems
-    result_repr = repr(joined)
-    assert 'ref_0' in result_repr
-    assert 'ref_1' in result_repr
-
-    # Cannot be immediately materialized because of the schema overlap
-    with pytest.raises(RelationError):
-        joined.materialize()
 
     # Project out left table schema
     proj = joined[[left]]
@@ -900,24 +885,20 @@ def test_self_join_no_view_convenience(table):
 
     result = table.join(table, [('g', 'g')])
 
-    t2 = table.view()
-    expected = table.join(t2, table.g == t2.g)
-    assert_equal(result, expected)
+    assert result.columns == [f"{column}_x" for column in table.columns] + [
+        f"{column}_y" for column in table.columns
+    ]
 
 
-def test_materialized_join_reference_bug(con):
+def test_join_reference_bug(con):
     # GH#403
     orders = con.table('tpch_orders')
     customer = con.table('tpch_customer')
     lineitem = con.table('tpch_lineitem')
 
-    items = (
-        orders.join(lineitem, orders.o_orderkey == lineitem.l_orderkey)[
-            lineitem, orders.o_custkey, orders.o_orderpriority
-        ]
-        .join(customer, [('o_custkey', 'c_custkey')])
-        .materialize()
-    )
+    items = orders.join(lineitem, orders.o_orderkey == lineitem.l_orderkey)[
+        lineitem, orders.o_custkey, orders.o_orderpriority
+    ].join(customer, [('o_custkey', 'c_custkey')])
     items['o_orderpriority'].value_counts()
 
 
@@ -942,10 +923,10 @@ def test_join_project_after(table):
 
     joined = table1.left_join(table2, [pred])
     projected = joined.projection([table1, table2['stuff']])
-    assert projected.schema().names == ['key1', 'value1', 'stuff']
+    assert projected.schema().names == ('key1', 'value1', 'stuff')
 
     projected = joined.projection([table2, table1['key1']])
-    assert projected.schema().names == ['key2', 'stuff', 'key1']
+    assert projected.schema().names == ('key2', 'stuff', 'key1')
 
 
 def test_semi_join_schema(table):
@@ -954,7 +935,7 @@ def test_semi_join_schema(table):
     table2 = ibis.table([('key2', 'string'), ('stuff', 'double')])
 
     pred = table1['key1'] == table2['key2']
-    semi_joined = table1.semi_join(table2, [pred]).materialize()
+    semi_joined = table1.semi_join(table2, [pred])
 
     result_schema = semi_joined.schema()
     assert_equal(result_schema, table1.schema())
@@ -967,7 +948,7 @@ def test_cross_join(table):
     ]
     scalar_aggs = table.aggregate(metrics)
 
-    joined = table.cross_join(scalar_aggs).materialize()
+    joined = table.cross_join(scalar_aggs)
     agg_schema = api.Schema(['sum_a', 'mean_b'], ['int64', 'double'])
     ex_schema = table.schema().append(agg_schema)
     assert_equal(joined.schema(), ex_schema)
@@ -983,13 +964,7 @@ def test_cross_join_multiple(table):
     assert joined.equals(expected)
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_join_compound_boolean_predicate(table):
-    # The user might have composed predicates through logical operations
-    assert False
-
-
-def test_filter_join_unmaterialized(table):
+def test_filter_join(table):
     table1 = ibis.table(
         {'key1': 'string', 'key2': 'string', 'value1': 'double'}
     )
@@ -1054,10 +1029,7 @@ def test_join_invalid_expr_type(con):
     invalid_right = left.foo_id
     join_key = ['bar_id']
 
-    with pytest.raises(
-        NotImplementedError,
-        match=r'string __getitem__\[str\]',
-    ):
+    with pytest.raises(com.IbisTypeError, match="Argument is not a table"):
         left.inner_join(invalid_right, join_key)
 
 
@@ -1101,16 +1073,6 @@ def test_unravel_compound_equijoin(table):
     assert_equal(joined, expected)
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_join_add_prefixes(table):
-    assert False
-
-
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_join_nontrivial_exprs(table):
-    assert False
-
-
 def test_union(
     setops_table_foo,
     setops_table_bar,
@@ -1200,12 +1162,6 @@ def test_simple_existence_predicate(t1, t2):
     assert isinstance(expr.op(), ops.Selection)
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_cannot_use_existence_expression_in_join(table):
-    # Join predicates must consist only of comparisons
-    assert False
-
-
 def test_not_exists_predicate(t1, t2):
     cond = -((t1.key1 == t2.key1).any())
     assert isinstance(cond.op(), ops.NotAny)
@@ -1279,6 +1235,10 @@ def test_sort_by2(table):
     expected = m.sort_by(ibis.desc('foo'))
     assert_equal(result, expected)
 
+    result = m.sort_by(ibis.asc(lambda x: x.foo))
+    expected = m.sort_by('foo')
+    assert_equal(result, expected)
+
 
 def test_projection2(table):
     m = table.mutate(foo=table.f * 2)
@@ -1402,9 +1362,13 @@ def test_mutate_chain():
     a, b = three.op().selections
 
     # we can't fuse these correctly yet
-    assert isinstance(a.op(), ops.IfNull)
+    assert isinstance(a.op(), ops.Alias)
+    assert isinstance(a.op().arg.op(), ops.IfNull)
     assert isinstance(b.op(), ops.TableColumn)
-    assert isinstance(b.op().table.op().selections[1].op(), ops.IfNull)
+
+    expr = b.op().table.op().selections[1]
+    assert isinstance(expr.op(), ops.Alias)
+    assert isinstance(expr.op().arg.op(), ops.IfNull)
 
 
 def test_multiple_dbcon():
@@ -1417,11 +1381,99 @@ def test_multiple_dbcon():
     con1.table('alltypes').union(con2.table('alltypes')).execute()
 
 
-@pytest.mark.xfail(
-    raises=RelationError, reason='Expr from different DB should fail #64'
-)
-def test_multiple_db():
+def test_multiple_db_different_backends():
     con1 = MockBackend()
     con2 = MockAlchemyBackend()
 
-    con1.table('alltypes').union(con2.table('alltypes')).execute()
+    backend1_table = con1.table('alltypes')
+    backend2_table = con2.table('alltypes')
+
+    expr = backend1_table.union(backend2_table)
+    with pytest.raises(
+        ValueError,
+        match=re.compile("multiple backends", flags=re.IGNORECASE),
+    ):
+        expr.compile()
+
+
+def test_merge_as_of_allows_overlapping_columns():
+    # GH3295
+    table = ibis.table(
+        [
+            ("field", "string"),
+            ("value", "float64"),
+            ("timestamp_received", "timestamp"),
+        ],
+        name="t",
+    )
+
+    signal_one = table[
+        table['field'].contains('signal_one')
+        & table['field'].contains('current')
+    ]
+    signal_one = signal_one[
+        'value', 'timestamp_received', 'field'
+    ]  # select columns we care about
+    signal_one = signal_one.relabel(
+        {'value': 'current', 'field': 'signal_one'}
+    )
+
+    signal_two = table[
+        table['field'].contains('signal_two')
+        & table['field'].contains('voltage')
+    ]
+    signal_two = signal_two[
+        'value', 'timestamp_received', 'field'
+    ]  # select columns we care about
+    signal_two = signal_two.relabel(
+        {'value': 'voltage', 'field': 'signal_two'}
+    )
+
+    merged = ibis.api.asof_join(signal_one, signal_two, 'timestamp_received')
+    assert merged.columns == [
+        'current',
+        'timestamp_received_x',
+        'signal_one',
+        'voltage',
+        'timestamp_received_y',
+        'signal_two',
+    ]
+
+
+def test_select_from_unambiguous_join_with_strings():
+    # GH1387
+    t = ibis.table([('a', 'int64'), ('b', 'string')])
+    s = ibis.table([('b', 'int64'), ('c', 'string')])
+    joined = t.left_join(s, [t.b == s.c])
+    expr = joined[t, 'c']
+    assert expr.columns == ["a", "b", "c"]
+
+
+def test_filter_applied_to_join():
+    # GH2437
+    countries = ibis.table([("iso_alpha3", "string")])
+    gdp = ibis.table([("country_code", "string"), ("year", "int64")])
+
+    expr = countries.inner_join(
+        gdp,
+        predicates=[countries["iso_alpha3"] == gdp["country_code"]],
+    ).filter(gdp["year"] == 2017)
+    assert expr.columns == ["iso_alpha3", "country_code", "year"]
+
+
+@pytest.mark.parametrize("how", ["inner", "left", "outer", "right"])
+def test_join_suffixes(how):
+    left = ibis.table([("id", "int64"), ("first_name", "string")])
+    right = ibis.table([("id", "int64"), ("last_name", "string")])
+
+    method = getattr(left, f"{how}_join")
+    expr = method(right, suffixes=("_left", "_right"))
+    assert expr.columns == ["id_left", "first_name", "id_right", "last_name"]
+
+
+def test_materialize_no_op():
+    left = ibis.table([("id", "int64")])
+    right = ibis.table([("id", "int64")])
+    expr = left.inner_join(right, "id")
+    with pytest.warns(FutureWarning):
+        expr.materialize()
