@@ -1,15 +1,15 @@
-import functools
 import operator
-import os
 import uuid
 from collections import OrderedDict
 from datetime import date, datetime, time
+from decimal import Decimal
 from operator import methodcaller
 
 import numpy as np
 import pandas as pd
 import pytest
 import toolz
+from pytest import param
 
 import ibis
 import ibis.common.exceptions as com
@@ -21,7 +21,6 @@ import ibis.expr.rules as rlz
 import ibis.expr.types as ir
 from ibis import literal
 from ibis.common.exceptions import IbisTypeError
-from ibis.expr.signature import Argument as Arg
 from ibis.tests.util import assert_equal
 
 
@@ -39,14 +38,6 @@ def test_null():
     assert expr2.type().equals(dt.null)
 
 
-@pytest.mark.xfail(
-    raises=AssertionError,
-    reason='UTF-8 support in Impala non-existent at the moment?',
-)
-def test_unicode():
-    assert False
-
-
 @pytest.mark.parametrize(
     ['value', 'expected_type'],
     [
@@ -62,9 +53,9 @@ def test_unicode():
         (-129, 'int16'),
         (-32769, 'int32'),
         (-2147483649, 'int64'),
-        (1.5, 'double'),
+        (1.5, 'float64'),
         ('foo', 'string'),
-        (b'fooblob', 'binary'),
+        (b'fooblob', 'bytes'),
         ([1, 2, 3], 'array<int8>'),
     ],
 )
@@ -75,7 +66,24 @@ def test_literal_with_implicit_type(value, expected_type):
     assert expr.type() == dt.dtype(expected_type)
 
     assert isinstance(expr.op(), ops.Literal)
-    assert expr.op().value is value
+    assert expr.op().value == dt._normalize(expr.type(), value)
+
+
+@pytest.mark.parametrize(
+    ['value', 'expected_type', 'expected_value'],
+    [
+        ([1, 2, 3], 'array<int8>', (1, 2, 3)),
+        ([[1, 2], [3, 4]], 'array<array<int8>>', ((1, 2), (3, 4))),
+    ],
+)
+def test_listeral_with_unhashable_values(value, expected_type, expected_value):
+    expr = ibis.literal(value)
+
+    assert isinstance(expr, ir.ScalarExpr)
+    assert expr.type() == dt.dtype(expected_type)
+
+    assert isinstance(expr.op(), ops.Literal)
+    assert expr.op().value == expected_value
 
 
 pointA = (1, 2)
@@ -95,17 +103,17 @@ multipolygon1 = [polygon1, polygon2]
     ['value', 'expected_type'],
     [
         (5, 'int16'),
-        (127, 'double'),
+        (127, 'float64'),
         (128, 'int64'),
-        (32767, 'double'),
-        (32768, 'float'),
+        (32767, 'float64'),
+        (32768, 'float32'),
         (2147483647, 'int64'),
         (-5, 'int16'),
         (-128, 'int32'),
         (-129, 'int64'),
-        (-32769, 'float'),
-        (-2147483649, 'double'),
-        (1.5, 'double'),
+        (-32769, 'float32'),
+        (-2147483649, 'float64'),
+        (1.5, 'float64'),
         ('foo', 'string'),
         (list(pointA), 'point'),
         (tuple(pointA), 'point'),
@@ -119,8 +127,10 @@ multipolygon1 = [polygon1, polygon2]
         (tuple(multipoint), 'multipoint'),
         (list(multipolygon1), 'multipolygon'),
         (tuple(multipolygon1), 'multipolygon'),
-        pytest.param(uuid.uuid4(), 'uuid', id='uuid'),
-        pytest.param(str(uuid.uuid4()), 'uuid', id='uuid_str'),
+        param(uuid.uuid4(), 'uuid', id='uuid'),
+        param(str(uuid.uuid4()), 'uuid', id='uuid_str'),
+        param(Decimal("234.234"), "decimal(6, 3)", id="decimal_native"),
+        param(234234, "decimal(6, 3)", id="decimal_int"),
     ],
 )
 def test_literal_with_explicit_type(value, expected_type):
@@ -128,6 +138,25 @@ def test_literal_with_explicit_type(value, expected_type):
     assert expr.type().equals(dt.validate_type(expected_type))
 
 
+@pytest.mark.parametrize(
+    ("value", "expected", "dtype"),
+    [
+        # precision > scale
+        (Decimal("234.234"), Decimal("234.234"), "decimal(6, 3)"),
+        (234234, Decimal("234.234"), "decimal(6, 3)"),
+        # scale == 0
+        (Decimal("234"), Decimal("234"), "decimal(6, 0)"),
+        (234, Decimal("234"), "decimal(6, 0)"),
+        # precision == scale
+        (Decimal(".234"), Decimal(".234"), "decimal(3, 3)"),
+        (234, Decimal(".234"), "decimal(3, 3)"),
+    ],
+)
+def test_normalize_decimal_literal(value, expected, dtype):
+    expr = ibis.literal(value, type=dtype)
+    assert expr.op().value == expected
+
+
 @pytest.mark.parametrize(
     ['value', 'expected_type', 'expected_class'],
     [
@@ -159,7 +188,7 @@ def test_literal_complex_types(value, expected_type, expected_class):
     assert expr_type.equals(dt.validate_type(expected_type))
     assert isinstance(expr, expected_class)
     assert isinstance(expr.op(), ops.Literal)
-    assert expr.op().value is value
+    assert expr.op().value == dt._normalize(dt.dtype(expected_type), value)
 
 
 def test_simple_map_operations():
@@ -204,30 +233,25 @@ def test_literal_with_non_coercible_type(value, expected_type):
         ibis.literal(value, type=expected_type)
 
 
-def test_non_inferrable_literal():
-    expected_msg = (
-        'The datatype of value .* cannot be inferred, try '
-        'passing it explicitly with the `type` keyword.'
-    )
-
-    value = tuple(pointA)
-
-    with pytest.raises(TypeError, match=expected_msg):
-        ibis.literal(value)
-
-    point = ibis.literal(value, type='point')
-    assert point.type() == dt.point
-
-
-def test_literal_list():
+def test_list_and_tuple_literals():
     what = [1, 2, 1000]
     expr = api.literal(what)
-
     assert isinstance(expr, ir.ArrayScalar)
-
     # it works!
     repr(expr)
 
+    what = (1, 2, 1000)
+    expr = api.literal(what)
+    assert isinstance(expr, ir.ArrayScalar)
+    # it works!
+    repr(expr)
+
+    # test using explicit type
+    point = ibis.literal((1, 2, 1000), type='point')
+    assert point.type() == dt.point
+    point = ibis.literal([1, 2, 1000], type='point')
+    assert point.type() == dt.point
+
 
 def test_literal_array():
     what = []
@@ -267,23 +291,6 @@ def test_value_counts(table, string_col):
     assert isinstance(expr, ir.TableExpr)
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_isin_not_comparable():
-    assert False
-
-
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_isin_array_expr():
-    assert False
-
-
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_isin_invalid_cases():
-    # For example, array expression in a list of values, where the inner
-    # array values originate from some other table
-    assert False
-
-
 def test_isin_notin_scalars():
     a, b, c = (ibis.literal(x) for x in [1, 1, 2])
 
@@ -294,17 +301,6 @@ def test_isin_notin_scalars():
     assert isinstance(result, ir.BooleanScalar)
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_isin_null():
-    assert False
-
-
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_negate_isin():
-    # Should yield a NotContains
-    assert False
-
-
 def test_scalar_isin_list_with_array(table):
     val = ibis.literal(2)
 
@@ -317,78 +313,17 @@ def test_scalar_isin_list_with_array(table):
     assert isinstance(not_expr, ir.BooleanColumn)
 
 
-def test_distinct_basic(functional_alltypes):
+def test_distinct_table(functional_alltypes):
     expr = functional_alltypes.distinct()
     assert isinstance(expr.op(), ops.Distinct)
     assert isinstance(expr, ir.TableExpr)
     assert expr.op().table is functional_alltypes
 
-    expr = functional_alltypes.string_col.distinct()
-    assert isinstance(expr.op(), ops.DistinctColumn)
-
-    assert isinstance(expr, ir.StringColumn)
-
-
-@pytest.mark.xfail(reason='NYT')
-def test_distinct_array_interactions(functional_alltypes):
-    # array cardinalities / shapes are likely to be different.
-    a = functional_alltypes.int_col.distinct()
-    b = functional_alltypes.bigint_col
-
-    with pytest.raises(ir.RelationError):
-        a + b
-
-
-@pytest.mark.parametrize('where', [lambda t: None, lambda t: t.int_col != 0])
-def test_distinct_count(functional_alltypes, where):
-    result = functional_alltypes.string_col.distinct().count(
-        where=where(functional_alltypes)
-    )
-    assert isinstance(result.op(), ops.CountDistinct)
-
-    expected = functional_alltypes.string_col.nunique(
-        where=where(functional_alltypes)
-    ).name('count')
-    assert result.equals(expected)
-
-
-def test_distinct_unnamed_array_expr():
-    table = ibis.table(
-        [('year', 'int32'), ('month', 'int32'), ('day', 'int32')], 'foo'
-    )
-
-    # it works!
-    expr = (
-        ibis.literal('-')
-        .join(
-            [
-                table.year.cast('string'),
-                table.month.cast('string'),
-                table.day.cast('string'),
-            ]
-        )
-        .distinct()
-    )
-    repr(expr)
-
-
-def test_distinct_count_numeric_types(functional_alltypes):
-    metric = (
-        functional_alltypes.bigint_col.distinct()
-        .count()
-        .name('unique_bigints')
-    )
-    functional_alltypes.group_by('string_col').aggregate(metric)
-
 
 def test_nunique(functional_alltypes):
     expr = functional_alltypes.string_col.nunique()
-    assert isinstance(expr.op(), ops.CountDistinct)
-
-
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_project_with_distinct():
-    assert False
+    assert isinstance(expr.op(), ops.Alias)
+    assert isinstance(expr.op().arg.op(), ops.CountDistinct)
 
 
 def test_isnull(table):
@@ -411,7 +346,7 @@ def test_notnull(table):
     assert isinstance(expr.op(), ops.NotNull)
 
 
-@pytest.mark.parametrize('column', ['e', 'f'], ids=['float', 'double'])
+@pytest.mark.parametrize('column', ['e', 'f'], ids=['float32', 'double'])
 def test_isnan_isinf_column(table, column):
     expr = table[column].isnan()
     assert isinstance(expr, ir.BooleanColumn)
@@ -433,11 +368,6 @@ def test_isnan_isinf_scalar(value):
     assert isinstance(expr.op(), ops.IsInf)
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_null_literal():
-    assert False
-
-
 @pytest.mark.parametrize(
     ['column', 'operation'],
     [
@@ -495,26 +425,6 @@ def test_log_literal(log):
     assert isinstance(log(ibis.literal(5.5)), ir.FloatingScalar)
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_exp():
-    assert False
-
-
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_sqrt():
-    assert False
-
-
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_trig_functions():
-    assert False
-
-
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_round():
-    assert False
-
-
 def test_cast_same_type_noop(table):
     c = table.g
     assert c.cast('string') is c
@@ -523,7 +433,7 @@ def test_cast_same_type_noop(table):
     assert i.cast('int8') is i
 
 
-@pytest.mark.parametrize('type', ['int8', 'int32', 'double', 'float'])
+@pytest.mark.parametrize('type', ['int8', 'int32', 'double', 'float32'])
 def test_string_to_number(table, type):
     casted = table.g.cast(type)
     casted_literal = ibis.literal('5').cast(type).name('bar')
@@ -556,11 +466,6 @@ def test_casted_exprs_are_named(table):
     expr.value_counts()
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_nonzero():
-    assert False
-
-
 @pytest.mark.parametrize('col', list('abcdefh'))
 def test_negate(table, col):
     c = table[col]
@@ -575,11 +480,6 @@ def test_negate_boolean_scalar():
     assert isinstance(result.op(), ops.Negate)
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_isnull_notnull():
-    assert False
-
-
 @pytest.mark.parametrize('column', ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'])
 @pytest.mark.parametrize('how', [None, 'first', 'last', 'heavy'])
 @pytest.mark.parametrize('condition_fn', [lambda t: None, lambda t: t.a > 8])
@@ -620,7 +520,7 @@ def test_any_all_notany(table, column, operation):
     ],
 )
 @pytest.mark.parametrize('column', list('abcdef'))
-@pytest.mark.parametrize('case', [2, 2 ** 9, 2 ** 17, 2 ** 33, 1.5])
+@pytest.mark.parametrize('case', [2, 2**9, 2**17, 2**33, 1.5])
 def test_numbers_compare_numeric_literal(table, operation, column, case):
     ex_op_class = {
         operator.eq: ops.Equals,
@@ -750,18 +650,26 @@ def test_chained_comparisons_not_allowed(table):
 
 
 @pytest.mark.parametrize(
-    'operation', [operator.add, operator.mul, operator.truediv, operator.sub]
+    'operation',
+    [operator.add, operator.sub, operator.truediv],
 )
-def test_binop_string_type_error(table, operation):
-    # Strings are not valid for any numeric arithmetic
-    ints = table.d
-    strs = table.g
+@pytest.mark.parametrize(("left", "right"), [("d", "g"), ("g", "d")])
+def test_binop_string_type_error(table, operation, left, right):
+    a = table[left]
+    b = table[right]
 
     with pytest.raises(TypeError):
-        operation(ints, strs)
+        operation(a, b)
 
-    with pytest.raises(TypeError):
-        operation(strs, ints)
+
+@pytest.mark.parametrize(("left", "right"), [("d", "g"), ("g", "d")])
+def test_string_mul(table, left, right):
+    a = table[left]
+    b = table[right]
+
+    expr = a * b
+    assert isinstance(expr, ir.StringColumn)
+    assert isinstance(expr.op(), ops.Repeat)
 
 
 @pytest.mark.parametrize(
@@ -782,8 +690,8 @@ def test_binop_string_type_error(table, operation):
         (operator.add, 'd', 5, 'int64'),
         (operator.mul, 'a', 0, 'int8'),
         (operator.mul, 'a', 5, 'int16'),
-        (operator.mul, 'a', 2 ** 24, 'int32'),
-        (operator.mul, 'a', -(2 ** 24) + 1, 'int32'),
+        (operator.mul, 'a', 2**24, 'int32'),
+        (operator.mul, 'a', -(2**24) + 1, 'int32'),
         (operator.mul, 'a', 1.5, 'double'),
         (operator.mul, 'b', 0, 'int16'),
         (operator.mul, 'b', 5, 'int32'),
@@ -812,7 +720,7 @@ def test_binop_string_type_error(table, operation):
         (operator.pow, 'b', 0, 'double'),
         (operator.pow, 'c', 0, 'double'),
         (operator.pow, 'd', 0, 'double'),
-        (operator.pow, 'e', 0, 'float'),
+        (operator.pow, 'e', 0, 'float32'),
         (operator.pow, 'f', 0, 'double'),
         (operator.pow, 'a', 2, 'double'),
         (operator.pow, 'b', 2, 'double'),
@@ -822,7 +730,7 @@ def test_binop_string_type_error(table, operation):
         (operator.pow, 'b', 1.5, 'double'),
         (operator.pow, 'c', 1.5, 'double'),
         (operator.pow, 'd', 1.5, 'double'),
-        (operator.pow, 'e', 2, 'float'),
+        (operator.pow, 'e', 2, 'float32'),
         (operator.pow, 'f', 2, 'double'),
         (operator.pow, 'a', -2, 'double'),
         (operator.pow, 'b', -2, 'double'),
@@ -863,31 +771,6 @@ def test_zero_subtract_literal_promotions(
     assert result.type() == dt.dtype(ex_type)
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_add_array_promotions():
-    assert False
-
-
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_subtract_array_promotions():
-    assert False
-
-
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_multiply_array_promotions():
-    assert False
-
-
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_divide_array_promotions():
-    assert False
-
-
-@pytest.mark.xfail(raises=AssertionError, reason='NYT')
-def test_string_add_concat():
-    assert False
-
-
 def test_substitute_dict():
     table = ibis.table([('foo', 'string'), ('bar', 'string')], 't1')
     subs = {'a': 'one', 'b': table.bar}
@@ -919,7 +802,7 @@ def test_substitute_dict():
         'array<map<string, array<array<double>>>>',
         'string',
         'double',
-        'float',
+        'float32',
         'int64',
     ],
 )
@@ -1009,8 +892,8 @@ def test_string_temporal_compare(op, left, right):
     ('value', 'type', 'expected_type_class'),
     [
         (2.21, 'decimal', dt.Decimal),
-        (3.14, 'double', dt.Double),
-        (4.2, 'int64', dt.Double),
+        (3.14, 'float64', dt.Float64),
+        (4.2, 'int64', dt.Float64),
         (4, 'int64', dt.Int64),
     ],
 )
@@ -1063,14 +946,6 @@ def test_scalar_parameter_set():
     assert value.type().equals(dt.Set(dt.int64))
 
 
-def test_scalar_parameter_repr():
-    value = ibis.param(dt.timestamp).name('value')
-    assert repr(value) == 'value = ScalarParameter[timestamp]'
-
-    value_op = value.op()
-    assert repr(value_op) == "ScalarParameter(type=timestamp)"
-
-
 @pytest.mark.parametrize(
     ('left', 'right', 'expected'),
     [
@@ -1098,24 +973,26 @@ def test_scalar_parameter_repr():
             ibis.param(dt.timestamp),
             False,
         ),
-        (
-            # different Python class, left side is param
-            ibis.param(dt.timestamp),
-            dt.date,
-            False,
-        ),
-        (
-            # different Python class, right side is param
-            dt.date,
-            ibis.param(dt.timestamp),
-            False,
-        ),
     ],
 )
 def test_scalar_parameter_compare(left, right, expected):
     assert left.equals(right) == expected
 
 
+@pytest.mark.parametrize(
+    ("left", "right"),
+    [
+        # different Python class, left side is param
+        param(ibis.param(dt.timestamp), dt.date, id="param_dtype"),
+        # different Python class, right side is param
+        param(dt.date, ibis.param(dt.timestamp), id="dtype_param"),
+    ],
+)
+def test_scalar_parameter_invalid_compare(left, right):
+    with pytest.raises(TypeError):
+        left.equals(right)
+
+
 @pytest.mark.parametrize(
     ('case', 'creator'),
     [
@@ -1142,11 +1019,12 @@ def test_custom_type_binary_operations():
 
         __radd__ = __add__
 
-    class FooNode(ops.ValueOp):
-        value = Arg(rlz.integer)
+    class FooNode(ops.Node):
+        value = rlz.integer
 
+        @property
         def output_type(self):
-            return functools.partial(Foo, dtype=dt.int64)
+            return Foo
 
     left = ibis.literal(2)
     right = FooNode(3).to_expr()
@@ -1165,8 +1043,8 @@ def test_empty_array_as_argument():
     class Foo(ir.Expr):
         pass
 
-    class FooNode(ops.ValueOp):
-        value = Arg(rlz.value(dt.Array(dt.int64)))
+    class FooNode(ops.Node):
+        value = rlz.value(dt.Array(dt.int64))
 
         def output_type(self):
             return Foo
@@ -1185,8 +1063,8 @@ def test_nullable_column_propagated():
             ('a', dt.Int32(nullable=True)),
             ('b', dt.Int32(nullable=False)),
             ('c', dt.String(nullable=False)),
-            ('d', dt.double),  # nullable by default
-            ('f', dt.Double(nullable=False)),
+            ('d', dt.float64),  # nullable by default
+            ('f', dt.Float64(nullable=False)),
         ]
     )
 
@@ -1377,18 +1255,6 @@ def test_valid_negate(type):
     assert -expr is not None
 
 
-@pytest.mark.xfail(
-    reason='Type not supported in most backends', raises=TypeError
-)
-@pytest.mark.skipif(
-    os.name == 'nt', reason='np.float128 not appear to exist on windows'
-)
-def test_valid_negate_float128():
-    value = np.float128(1)
-    expr = ibis.literal(value)
-    assert -expr is not None
-
-
 @pytest.mark.parametrize(
     ('kind', 'begin', 'end'),
     [
@@ -1418,8 +1284,8 @@ def test_window_unbounded_invalid(kind, begin, end):
         (ibis.literal(1.0), ibis.literal(1), dt.float64),
         (ibis.literal(1), ibis.literal(1), dt.int8),
         (ibis.literal(1), ibis.literal(1000), dt.int16),
-        (ibis.literal(2 ** 16), ibis.literal(2 ** 17), dt.int32),
-        (ibis.literal(2 ** 50), ibis.literal(1000), dt.int64),
+        (ibis.literal(2**16), ibis.literal(2**17), dt.int32),
+        (ibis.literal(2**50), ibis.literal(1000), dt.int64),
         (ibis.literal([1, 2]), ibis.literal([1, 2]), dt.Array(dt.int8)),
         (ibis.literal(['a']), ibis.literal([]), dt.Array(dt.string)),
         (ibis.literal([]), ibis.literal(['a']), dt.Array(dt.string)),
@@ -1444,31 +1310,12 @@ def test_nullif_fail(left, right):
     "join_method",
     [
         "left_join",
-        pytest.param(
-            "right_join",
-            marks=pytest.mark.xfail(
-                raises=AttributeError, reason="right_join is not an ibis API"
-            ),
-        ),
+        "right_join",
         "inner_join",
         "outer_join",
         "asof_join",
-        pytest.param(
-            "semi_join",
-            marks=pytest.mark.xfail(
-                raises=com.IbisTypeError,
-                reason=(
-                    "semi_join only gives access to the left table's "
-                    "columns"
-                ),
-            ),
-        ),
     ],
 )
-@pytest.mark.xfail(
-    raises=(com.IbisError, AttributeError),
-    reason="Select from unambiguous joins not implemented",
-)
 def test_select_on_unambiguous_join(join_method):
     t = ibis.table([("a0", dt.int64), ("b1", dt.string)], name="t")
     s = ibis.table([("a1", dt.int64), ("b2", dt.string)], name="s")
@@ -1492,30 +1339,56 @@ def test_chained_select_on_join():
 
 def test_repr_list_of_lists():
     lit = ibis.literal([[1]])
-    result = repr(lit)
-    expected = """\
-Literal[array<array<int8>>]
-  [[1]]"""
-    assert result == expected
+    repr(lit)
 
 
 def test_repr_list_of_lists_in_table():
     t = ibis.table([('a', 'int64')], name='t')
     lit = ibis.literal([[1]])
     expr = t[t, lit.name('array_of_array')]
-    result = repr(expr)
-    expected = """\
-ref_0
-UnboundTable[table]
-  name: t
-  schema:
-    a : int64
+    repr(expr)
 
-Selection[table]
-  table:
-    Table: ref_0
-  selections:
-    Table: ref_0
-    array_of_array = Literal[array<array<int8>>]
-      [[1]]"""
-    assert result == expected
+
+def test_repr_html():
+    con = ibis.pandas.connect({"t": pd.DataFrame({"a": [1, 2, 3]})})
+    t = con.table("t")
+
+    assert t._repr_html_() is None
+    assert t.a._repr_html_() is None
+    assert t.a.sum()._repr_html_() is None
+
+    interactive = ibis.options.interactive
+    ibis.options.interactive = True
+    try:
+        assert 'table' in t._repr_html_()
+        assert 'table' in t.a._repr_html_()
+        assert t.a.sum()._repr_html_() is None
+    finally:
+        ibis.options.interactive = interactive
+
+
+@pytest.mark.parametrize(
+    ("expr", "expected_type"),
+    [
+        (ibis.coalesce(ibis.NA, 1), dt.int8),
+        (ibis.coalesce(1, ibis.NA), dt.int8),
+        (ibis.coalesce(ibis.NA, 1000), dt.int16),
+        (ibis.coalesce(ibis.NA), dt.null),
+        (ibis.coalesce(ibis.NA, ibis.NA), dt.null),
+        (
+            ibis.coalesce(ibis.NA, ibis.NA.cast("array<string>")),
+            dt.Array(dt.string),
+        ),
+    ],
+)
+def test_coalesce_type_inference_with_nulls(expr, expected_type):
+    assert expr.type() == expected_type
+
+
+def test_literal_hash():
+    expr = ibis.literal(1)
+    op = expr.op()
+    # hashing triggers computation and storage of the op's hash
+    result1 = hash(op)
+    assert op._hash == result1
+    assert hash(op) == result1
